/* File AVModel_modMain.c */
/*
 ##############################################################
 # (C) 2015 BAE SYSTEMS
 #
 # ECOA Data Servers Sample - Aircraft Model
 #
 # $Id: AVModel_modMain.c,v 2.4 2017/11/06 15:33:32 ksappleb Exp $
 #
 # Part of the BAE Systems, Rochester, ECOA Samples collection
 #
 # Created by K S Appleby, BAE Systems, Airport Works, Rochester, Kent
 #
 # This software is developed for and on behalf of BAE Systems (Operations)
 # Limited, Dassault Aviation, Bull SAS, Thales Systemes Aeroportes,
 # GE Aviation Systems Limited,
 # General Dynamics United Kingdom Limited and Leonardo MW Ltd,
 # and the copyright is owned by BAE Systems (Operations) Limited,
 # Dassault Aviation, Bull SAS, Thales Systemes Aeroportes, 
 # GE Aviation Systems Limited, General Dynamics United Kingdom
 # Limited and Leonardo MW Ltd
 #
 # This software is developed by BAE Systems (Operations)
 # Limited, Electronic Systems, and is the Intellectual
 # Property of BAE Systems (Operations) Limited, Electronic Systems.
 #
 # The information set out in this document is provided solely
 # on an 'as is' basis and the co-developers of this software make
 # no warranties expressed or implied, including no warranties
 # as to completeness, accuracy or fitness for purpose, with respect
 # to any of the information.
 ##############################################################
*/
#include "AVModel_modMain.h"
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#ifndef M_PI
#  define M_PI 3.14159265358979
#endif

void AVModel_modMain__INITIALIZE__received(AVModel_modMain__context *context)
{
	context->user.Latitude  = 51.0;
	context->user.Longitude = 0.0;
	context->user.Height    = 666.0;
	context->user.Speed     = 100.0;
	context->user.Heading   = 0.0;
}

void AVModel_modMain__START__received(AVModel_modMain__context *context)
{
   /* User Code Here */
}

void AVModel_modMain__STOP__received(AVModel_modMain__context *context)
{
   /* User Code Here */
}

void AVModel_modMain__SHUTDOWN__received(AVModel_modMain__context *context)
{
   /* User Code Here */
}

static ECOA__double64 frand( double lo, double hi )
{
	double frnd = (double)rand()/(double)RAND_MAX; // 0.0 ... 1.0
	return (ECOA__double64)( lo + (frnd * ( hi - lo )));
}

void AVModel_modMain__Tick__received(AVModel_modMain__context *context)
{
	ECOA__double64 Hrads;
	ECOA__log      msg;
	ECOA__hr_time  now;
	AVS__AVS       avs;
	//
	fflush(stdout); // Make sure that any piped IO is completed
	//
	Hrads = context->user.Heading * M_PI / 180.0;
	context->user.Latitude   = 53.446145 + 0.05 * sin( Hrads );
	context->user.Longitude  = -1.952925 + 0.05 * -cos( Hrads ) / cos( context->user.Latitude * M_PI /180.0 );
	context->user.Height     = context->user.Height + frand( -0.3, 0.301 );
	context->user.Speed      = context->user.Speed  + frand( -0.3, 0.301 );
	msg.current_size = sprintf( msg.data, "AVS = %9.5f %9.5f %4.0f %4.0f %4.0f",
			context->user.Latitude, context->user.Longitude,
			context->user.Height,
			context->user.Speed,
			context->user.Heading );
	AVModel_modMain_container__get_relative_local_time( context, &now );
	if( now.nanoseconds < 50000000 )
		;//AVModel_modMain_container__log_info( context, msg );
	//
	// Build the XML output message
	avs.current_size = sprintf( (char*)avs.data,
			"<AVS>"
				"<Latitude units=\"Degrees\" direction=\"North\">%g</Latitude>"
				"<Longitude units=\"Degrees\" direction=\"East\">%g</Longitude>"
				"<Height units=\"feet\" reference=\"MeanSeaLevel\">%g</Height>"
				"<Speed units=\"metresPerSec\">%g</Speed>"
				"<Heading units=\"Degrees\" reference=\"NorthClockwise\">%g</Heading>"
			"</AVS>",
			context->user.Latitude, context->user.Longitude,
				context->user.Height,
					context->user.Speed,
						context->user.Heading );
	AVModel_modMain_container__AVSUpdate__send( context, &avs );
	//
	context->user.Heading += 0.05;
	if( context->user.Heading > 360.0 )
		context->user.Heading -= 360.0;
	if( context->user.Heading <   0.0 )
		context->user.Heading += 360.0;
}
