/* File HUMS_modMain.c */
/*
 ##############################################################
 # (C) 2015 BAE SYSTEMS
 #
 # ECOA Data Servers Sample - Example Data Server Client
 #
 # $Id: HUMS_modMain.c,v 2.4 2017/11/06 15:33:32 ksappleb Exp $
 #
 # Part of the BAE Systems, Rochester, ECOA Samples collection
 #
 # Created by K S Appleby, BAE Systems, Airport Works, Rochester, Kent
 #
 # This software is developed for and on behalf of BAE Systems (Operations)
 # Limited, Dassault Aviation, Bull SAS, Thales Systemes Aeroportes,
 # GE Aviation Systems Limited,
 # General Dynamics United Kingdom Limited and Leonardo MW Ltd,
 # and the copyright is owned by BAE Systems (Operations) Limited,
 # Dassault Aviation, Bull SAS, Thales Systemes Aeroportes, 
 # GE Aviation Systems Limited, General Dynamics United Kingdom
 # Limited and Leonardo MW Ltd
 #
 # This software is developed by BAE Systems (Operations)
 # Limited, Electronic Systems, and is the Intellectual
 # Property of BAE Systems (Operations) Limited, Electronic Systems.
 #
 # The information set out in this document is provided solely
 # on an 'as is' basis and the co-developers of this software make
 # no warranties expressed or implied, including no warranties
 # as to completeness, accuracy or fitness for purpose, with respect
 # to any of the information.
 ##############################################################
*/
#include "HUMS_modMain.h"
#include "subsys.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <libxml/parser.h>
#include <libxml/tree.h>
#include <libxml/xpath.h>
#include <gdbm.h>

//*************************************************************************************
typedef struct {
	ECOA__hr_time date;
	ECOA__uint32  seqNo;
} TSIndx;

typedef struct {
	ECOA__hr_time ts;
	ECOA__uint32  subsys;
	ECOA__uint32  dType;
} DataIndx;

//*************************************************************************************
void doHUMSDataProcessing( HUMS_modMain__context *context, ECOA__hr_time ts );
void doHUMSDataRecording( HUMS_modMain__context *context, ECOA__hr_time ts );

//*************************************************************************************
void HUMS_modMain__INITIALIZE__received(HUMS_modMain__context *context)
{
	context->user.humsConnection = 0;
	context->user.humsDBHandle = 0;
	context->user.seqNo = 0;
	xmlInitParser();
}

void HUMS_modMain__START__received(HUMS_modMain__context *context)
{
   /* User Code Here */
}

void HUMS_modMain__STOP__received(HUMS_modMain__context *context)
{
   /* User Code Here */
}

void HUMS_modMain__SHUTDOWN__received(HUMS_modMain__context *context)
{
   /* User Code Here */
}

//*************************************************************************************

static void dbConnect( HUMS_modMain__context *context )
{
	ECOA__SQL__String dataSource;
	ECOA__SQL__NameString userName, password;
	ECOA__SQL__StringZ dataSourceZ, userNameZ, passwordZ;
	int l;
	//
    HUMS_modMain_container__get_CIMSDB_value( context, &dataSourceZ );
    HUMS_modMain_container__get_UserName_value( context, &userNameZ );
    HUMS_modMain_container__get_Password_value( context, &passwordZ );
    //
   	l = dataSource.current_size = strlen( dataSourceZ ); strncpy( dataSource.data, dataSourceZ, l );
   	l = userName.current_size = strlen( userNameZ );     strncpy( userName.data, userNameZ, l );
   	l = password.current_size = strlen( passwordZ );     strncpy( password.data, passwordZ,  l );
	//
	HUMS_modMain_container__getConnection__request_async( context, &(context->user.connectionRequestID), &dataSource, &userName, &password );
}

//*************************************************************************************
void HUMS_modMain__getConnection__response_received
   (HUMS_modMain__context* context,    const ECOA__uint32 ID, const ECOA__return_status status,
    const ECOA__SQL__ConnectionId connection,
    const ECOA__SQL__Error requestStatus)
{
	ECOA__log msg;
	//
	if( requestStatus != ECOA__SQL__Error_NO_ERROR ){
		msg.current_size = sprintf( msg.data, "HUMS: connecting to datasource returned code %d", requestStatus );
		HUMS_modMain_container__raise_error( context, msg );
	}else{
		context->user.humsConnection = connection;
//		printf( "HUMS_modMain__getConnection__response_received connectionID = 0x%08x\n", connection );
	}
}

//*************************************************************************************
void HUMS_modMain__tableUpdateNotification__received(HUMS_modMain__context *context,
    const ECOA__SQL__NameString *tableName)
{
   /* User Code Here */
}

//*************************************************************************************
void HUMS_modMain__AVSUpdate__received(HUMS_modMain__context *context,
    const AVS__AVS *avs)
{
	ECOA__log msg;
	xmlDocPtr  pDoc = NULL;
	xmlXPathContextPtr xpathCtx;
	xmlXPathObjectPtr xpathObj;
	xmlChar* p;
	//
	if(( pDoc = xmlParseMemory( (char*)avs->data, avs->current_size )) == NULL ){
		msg.current_size = sprintf( msg.data, "Invalid AVS %s", avs->data );
		HUMS_modMain_container__log_warning( context, msg );
		return;
	}
	if(( xpathCtx = xmlXPathNewContext(pDoc)) == NULL ){
		msg.current_size = sprintf( msg.data, "Invalid AVS content" );
		HUMS_modMain_container__log_warning( context, msg );
        xmlFreeDoc(pDoc);
		return;
	}
	if(( xpathObj = xmlXPathEval((xmlChar*)"/AVS/Latitude", xpathCtx)) == NULL ){
		msg.current_size = sprintf( msg.data, "Invalid AVS content: missing Latitude" );
		HUMS_modMain_container__log_warning( context, msg );
        xmlXPathFreeContext(xpathCtx);
        xmlFreeDoc(pDoc);
		return;
	}else{
		context->user.Latitude = atof( (char*)(p = xmlNodeGetContent( xpathObj->nodesetval->nodeTab[0] )));
		xmlFree(p);
		xmlXPathFreeObject( xpathObj );
	}
	if(( xpathObj = xmlXPathEval((xmlChar*)"/AVS/Longitude", xpathCtx)) == NULL ){
		msg.current_size = sprintf( msg.data, "Invalid AVS content: missing Longitude" );
		HUMS_modMain_container__log_warning( context, msg );
        xmlXPathFreeContext(xpathCtx);
        xmlFreeDoc(pDoc);
		return;
	}else{
		context->user.Longitude = atof( (char*)(p = xmlNodeGetContent( xpathObj->nodesetval->nodeTab[0] )));
		xmlFree(p);
		xmlXPathFreeObject( xpathObj );
	}
	if(( xpathObj = xmlXPathEval((xmlChar*)"/AVS/Height", xpathCtx)) == NULL ){
		msg.current_size = sprintf( msg.data, "Invalid AVS content: missing Height" );
		HUMS_modMain_container__log_warning( context, msg );
        xmlXPathFreeContext(xpathCtx);
        xmlFreeDoc(pDoc);
		return;
	}else{
		context->user.Height = atof( (char*)(p = xmlNodeGetContent( xpathObj->nodesetval->nodeTab[0] )));
		xmlFree(p);
		xmlXPathFreeObject( xpathObj );
	}
	if(( xpathObj = xmlXPathEval((xmlChar*)"/AVS/Speed", xpathCtx)) == NULL ){
		msg.current_size = sprintf( msg.data, "Invalid AVS content: missing Speed" );
		HUMS_modMain_container__log_warning( context, msg );
        xmlXPathFreeContext(xpathCtx);
        xmlFreeDoc(pDoc);
		return;
	}else{
		context->user.Speed = atof( (char*)(p = xmlNodeGetContent( xpathObj->nodesetval->nodeTab[0] )));
		xmlFree(p);
		xmlXPathFreeObject( xpathObj );
	}
	if(( xpathObj = xmlXPathEval((xmlChar*)"/AVS/Heading", xpathCtx)) == NULL ){
		msg.current_size = sprintf( msg.data, "Invalid AVS content: missing Heading" );
		HUMS_modMain_container__log_warning( context, msg );
        xmlXPathFreeContext(xpathCtx);
        xmlFreeDoc(pDoc);
		return;
	}else{
		context->user.Heading = atof( (char*)(p = xmlNodeGetContent( xpathObj->nodesetval->nodeTab[0] )));
		xmlFree(p);
		xmlXPathFreeObject( xpathObj );
	}
    xmlXPathFreeContext(xpathCtx);
    xmlFreeDoc(pDoc);
	//
#if 0
	msg.current_size = sprintf( msg.data, "AVS = %9.5f %9.5f %4.0f %4.0f %4.0f",
			context->user.Latitude, context->user.Longitude,
			context->user.Height,
			context->user.Speed,
			context->user.Heading );
	HUMS_modMain_container__log_info( context, msg );
#endif
}

//*************************************************************************************
void HUMS_modMain__Tick__received(HUMS_modMain__context *context)
{
	ECOA__return_status errc;
	ECOA__log msg;
    ECOA__int32 dbmStatus;
	//
	fflush(stdout); // Make sure that any piped IO is completed
	//
	// If not connected to the HUMS long-term store (SQL data server), do so...
	if( context->user.humsConnection < 1 ){
		dbConnect( context );
	}
	// If not connected to the HUMS short-term store (dbm data server), do so...
	if( context->user.humsDBHandle < 1 ){
		ECOA__DBM__String name;
	    ECOA__DBM__OpenFlags flags = ECOA__DBM__OpenFlag_READER|ECOA__DBM__OpenFlag_WRCREAT|ECOA__DBM__OpenFlag_SYNC;
	    ECOA__DBM__Permissions mode = { ECOA__DBM__Permission_ALL, ECOA__DBM__Permission_ALL, ECOA__DBM__Permission_ALL };
	    //
		name.current_size = sprintf( name.data, "./resource/HUMSra.db" );
		errc = HUMS_modMain_container__open__request_sync( context, &name, 1024, flags, &mode, &(context->user.humsDBHandle), &dbmStatus );
		if( errc != ECOA__return_status_OK ){
			msg.current_size = sprintf( msg.data, "HUMS: opening dbm '%s' returned code %d, status code %d (%s)", name.data, errc, dbmStatus, gdbm_errlist[dbmStatus] );
			HUMS_modMain_container__log_warning( context, msg );
		}
	}
	if( context->user.humsDBHandle > 0 && context->user.humsConnection > 0){
		//
		ECOA__DBM__Datum key, value;
		ECOA__hr_time ts;
		TSIndx tsindx;
		//
		// Get a timestamp
		HUMS_modMain_container__get_relative_local_time( context, &ts );
		tsindx.date.nanoseconds = 0;
		tsindx.date.seconds = ts.seconds % 86400; // reduce to 1 whole day
		tsindx.seqNo = ++context->user.seqNo;
		//
		// Add this timestamp to the data store
		memcpy( &(key.data), &tsindx, (key.current_size = sizeof( tsindx )));
		memcpy( &(value.data), &ts, (value.current_size = sizeof( ts )));
		errc = HUMS_modMain_container__store__request_sync( context, context->user.humsDBHandle, &key, &value, ECOA__DBM__StoreFlag_INSERT, &dbmStatus );
		if( errc != ECOA__return_status_OK ){
			msg.current_size = sprintf( msg.data, "HUMS: saving timestamp returned code %d, status code %d (%s)", errc, dbmStatus, gdbm_errlist[dbmStatus] );
			HUMS_modMain_container__log_warning( context, msg );
		}
		//
		//*************************************************************************************
		// HUMS Data Processing (Scenario 4.1):
		//*************************************************************************************
		doHUMSDataProcessing( context, ts );
		//
		//*************************************************************************************
		// HUMS Data Recording (Scenario 4.2)
		//*************************************************************************************
		doHUMSDataRecording( context, ts );
	}
}

#define NSYS 10

//*************************************************************************************
// HUMS Data Processing (Scenario 4.1):
//*************************************************************************************
void doHUMSDataProcessing( HUMS_modMain__context *context, ECOA__hr_time ts )
{
	ECOA__return_status errc;
	ECOA__log msg;
	ECOA__global_time now;
	ECOA__SQL__String SQLString;
	ECOA__SQL__ResultSetId transactionId;
	ECOA__DBM__Datum key, value;
    ECOA__int32 dbmStatus, subsys;
	HUMSData humsData;
	char*    humsRule;
	DataIndx dindx;
	//
	// for each subsystem in SubSystemList do
	//   get subsystem HUMSData
	//   cache subsystem HUMSData to HUMS_ShortTerm
	//   compute subsystem trends:
	//     get subsystem trend rules from HUMS_LongTerm
	//     get past subsystem HUMSData from HUMS_ShortTerm
	//     compute trend(s) according to the rules
	//     check for, record, and signal, potential exceedence(s)
	// done
	//
	// Get the HUMSData for each subsystem
	for( subsys = 1; subsys <= NSYS; subsys++ ){
		ECOA__float32 trendCoefficient = 0.0;
		//
		humsData = HUMSData_Create();
		getHUMSData( subsys, humsData );
		//
		// Add this sub-system's HUMSData snapshot to the data store
		dindx.subsys = subsys;
		dindx.ts = ts;
		dindx.dType = 0; // 0 ==> "data"
		memcpy( &(key.data), &dindx, (key.current_size = sizeof( dindx )));
		memcpy( &(value.data), &humsData, (value.current_size = HUMSData_Size( humsData )));
		errc = HUMS_modMain_container__store__request_sync( context, context->user.humsDBHandle, &key, &value, ECOA__DBM__StoreFlag_INSERT, &dbmStatus );
		if( errc != ECOA__return_status_OK ){
			msg.current_size = sprintf( msg.data, "HUMS: saving subsystem data returned code %d, status code %d (%s)", errc, dbmStatus, gdbm_errlist[dbmStatus] );
			HUMS_modMain_container__log_warning( context, msg );
		}
		//
		// Get the HUMS rule(s) for this subsystem from the long-term store
		humsRule = NULL; // skip in this demo...
		//
		// Compute a trend coefficient for this subsystem...
		//
		// In real life, this would have a time element too...
		trendCoefficient = getTrendCoefficient( subsys, humsData, humsRule );
		//
		// Add the trendCoeff for this subsystem to the short-term store
		dindx.subsys = subsys;
		dindx.ts = ts;
		dindx.dType = 1;// 1 ==> Trend Coeff.
		memcpy( &(key.data), &dindx, (key.current_size = sizeof( dindx )));
		memcpy( &(value.data), &trendCoefficient, (value.current_size = sizeof( trendCoefficient )));
		errc = HUMS_modMain_container__store__request_sync( context, context->user.humsDBHandle, &key, &value, ECOA__DBM__StoreFlag_INSERT, &dbmStatus );
		if( errc != ECOA__return_status_OK ){
			msg.current_size = sprintf( msg.data, "HUMS: storing sub-system TrendCoefficient returned code %d", errc );
			HUMS_modMain_container__log_warning( context, msg );
		}
		HUMSData_Destroy( humsData );
	}
	// compute system trends:
	//   get system trend rules from HUMS_LongTerm
	//   get past system HUMSData from HUMS_ShortTerm
	//   compute trend(s) according to the rules
	//   check for, record, and signal potential exceedence(s)
	// done
	{
		ECOA__float32 trendCoefficient[NSYS+1], systemTrendCoefficient = 0;
		ECOA__SQL__String errText;
		//
		// Get the HUMS rule(s) for the system from the long-term store
		humsRule = NULL; // skip in this demo...
		//
		for( subsys = 1; subsys <= NSYS; subsys++ ){
			//
			trendCoefficient[subsys] = 0.0;
			//
			// Retrieve the HUMS trend data for this subsystem
			//
			// In real life, this would have a time element too...
			dindx.subsys = subsys;
			dindx.ts = ts;
			dindx.dType = 1;// 1 ==> Trend Coeff.
			memcpy( &(key.data), &dindx, (key.current_size = sizeof( dindx )));
			errc = HUMS_modMain_container__fetch__request_sync( context, context->user.humsDBHandle, &key, &value, &dbmStatus );
			if( errc != ECOA__return_status_OK ){
				msg.current_size = sprintf( msg.data, "HUMS: retrieving sub-system TrendCoefficient returned code %d", errc );
				HUMS_modMain_container__log_warning( context, msg );
				return;
			}
			memcpy( &(trendCoefficient[subsys]), &(value.data), value.current_size );
			//
			// Sum the trend coeffs. over the subsystems
			systemTrendCoefficient += trendCoefficient[subsys];
		}
		// trendCoefficient across the subsystems is the RSS value...
		systemTrendCoefficient = sqrt( systemTrendCoefficient );
		//
		HUMS_modMain_container__get_UTC_time( context, &now );
		SQLString.current_size = sprintf( SQLString.data,
				"insert into VehicleHUM values( %d.%d, %g );",
					now.seconds, now.nanoseconds, systemTrendCoefficient );
		errc = HUMS_modMain_container__SQLExecute__request_sync( context, context->user.humsConnection, &SQLString, &transactionId );
		if( errc != ECOA__return_status_OK ){
			msg.current_size = sprintf( msg.data, "HUMS: storing system trend data returned code %d", errc );
			HUMS_modMain_container__log_warning( context, msg );
		}
	}
}

//*************************************************************************************
// HUMS Data Recording (Scenario 4.2)
//*************************************************************************************
void doHUMSDataRecording( HUMS_modMain__context *context, ECOA__hr_time ts )
{
	ECOA__return_status errc;
	ECOA__log msg;
	ECOA__global_time now;
	ECOA__SQL__String SQLString;
	ECOA__SQL__ResultSetId transactionId;
	ECOA__SQL__String errText;
	ECOA__DBM__Datum key, value;
    ECOA__int32 dbmStatus, subsys;
	ECOA__float32 trendCoefficient;
	DataIndx dindx;
	//
	// for each subsystem in SubSystemList do
	//   get subsystem HUMSData from HUMS_ShortTerm cache
	//   timestamp and package the snapshot
	//   post the package to HUMS_LongTerm;
	// done
	HUMS_modMain_container__get_UTC_time( context, &now );
	//
	for( subsys = 1; subsys <= NSYS; subsys++ ){
		//
		trendCoefficient = 0.0;
		//
		// Retrieve a snapshot of the HUMS trend data for this subsystem
		dindx.subsys = subsys;
		dindx.ts = ts;
		dindx.dType = 1;// 1 ==> Trend Coeff.
		memcpy( &(key.data), &dindx, (key.current_size = sizeof( dindx )));
		errc = HUMS_modMain_container__fetch__request_sync( context, context->user.humsDBHandle, &key, &value, &dbmStatus );
		if( errc != ECOA__return_status_OK ){
			msg.current_size = sprintf( msg.data, "HUMS: retrieving sub-system TrendCoefficient returned code %d", errc );
			HUMS_modMain_container__log_warning( context, msg );
			return;
		}
		memcpy( &trendCoefficient, &(value.data), value.current_size );
		//
		SQLString.current_size = sprintf( SQLString.data, "insert into SubSys%d values( %d.%d, %g );",
																	subsys, now.seconds, now.nanoseconds, trendCoefficient );
//		printf( "HUMS: Saving record using:\n  %s\n", SQLString.data );
		errc = HUMS_modMain_container__SQLExecute__request_sync( context, context->user.humsConnection, &SQLString, &transactionId );
		if( errc != ECOA__return_status_OK || transactionId == 0 ){
			msg.current_size = sprintf( msg.data, "HUMS: storing a HUMS snapshot of sub system %d returned code %d", subsys, errc );
			HUMS_modMain_container__log_warning( context, msg );
		}
	}
	//
	SQLString.current_size = sprintf( SQLString.data,
			"insert into FlightRecord values( %d.%d, %g, %g, %g, %g, %g );",
				now.seconds, now.nanoseconds,
				context->user.Latitude, context->user.Longitude,
				context->user.Height,
				context->user.Speed,
				context->user.Heading );
//	printf( "HUMS: Saving record using:\n  %s\n", SQLString.data );
	errc = HUMS_modMain_container__SQLExecute__request_sync( context, context->user.humsConnection, &SQLString, &transactionId );
	if( errc != ECOA__return_status_OK ){
		msg.current_size = sprintf( msg.data, "HUMS: storing a flight record returned code %d", errc );
		HUMS_modMain_container__log_warning( context, msg );
	}
	if( transactionId == 0 ){
		msg.current_size = sprintf( msg.data, "HUMS: storing a flight record" );
		HUMS_modMain_container__log_warning( context, msg );
		return;
	}
	msg.current_size = sprintf( msg.data, "HUMS: snapshot record stored" );
	HUMS_modMain_container__log_info( context, msg );
}

