/*
 * subsys.c
 *
 *  Created on: 12 Jul 2016
 *      Author: SW_APPLEBY
 */

#include "ECOA.h"
#include "subsys.h"
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

#if !defined( MINDOUBLE )
#  define MINDOUBLE 2.25e-308
#endif

#if !defined( min )
# define min(a,b) ((a)<(b)?(a):(b))
#endif

/* HUMSData is an opaque type
 *
 * Users can map onto it any data (up to the size limit).
 */
struct HUMSData_t {
	ECOA__int32 SysInfoSize;
	char        SysInfo[HUMSData_MAXSIZE];
};

//*************************************************************************************
HUMSData HUMSData_Create()
{
	HUMSData humsData = (HUMSData)malloc( sizeof( struct HUMSData_t ));
	humsData->SysInfoSize = 0;
	return humsData;
}

//*************************************************************************************
void HUMSData_Destroy( HUMSData humsData )
{
	if( humsData ){
		free( humsData );
	}
}

//*************************************************************************************
ECOA__int32 HUMSData_Size( HUMSData humsData )
{
	return sizeof( humsData->SysInfoSize ) + humsData->SysInfoSize;
}

//*************************************************************************************
// Random number 0.0 .. 1.0
static double urand()
{
	return (double)rand() / (double)RAND_MAX;
}

//*************************************************************************************
// Normally distributed random number
static double nrand( double mean, double std_dev )
{
	double urandp;
    /* Need to ensure that urandp != 0.0.
     * Otherwise log( urandp ) is undefined. */
	urandp = (( urandp = urand()) <= MINDOUBLE ? MINDOUBLE : urandp );
	  return mean + std_dev * sqrt( -2.0 * log( urandp )) * cos( M_PI * urand());
}

//*************************************************************************************
// Fake some data from the sub-systems
//
// The data we'll fake will be a normal distributed random ordinate with
//   mean    = 0.65
//   std dev = 0.1
void getHUMSData( ECOA__int32 subsys, HUMSData humsData  )
{
	float data[20];
	int   i, N;
	/**/
	N = min( (int)(20.0 * (float)rand()/(float)RAND_MAX), 1 );
	while(( humsData->SysInfoSize = N * sizeof( float )) > HUMSData_MAXSIZE )
		N--;
	/**/
	for( i = 0; i < N; i++ ){
		data[i] = nrand( 0.65, 0.1 );
	}
	memcpy( &(humsData->SysInfo), data, N * sizeof( float ));
}

//*************************************************************************************
// Create a per-subsystem SQL command
char *getSQLInsertCommand( ECOA__int32 subsys, ECOA__hr_time ts, HUMSData humsData )
{
	static char command[10240], buf[64];
	float data[20];
	int i, N;
	/**/
	N = humsData->SysInfoSize / sizeof(float);
	memcpy( data, humsData->SysInfo, humsData->SysInfoSize );
	sprintf( command, "insert into SubSys%d values( %d.%d", subsys, ts.seconds, ts.nanoseconds );
	for ( i = 0; i < N; i++ ){
		sprintf( buf, ", %g", data[i] );
		strcat( command, buf );
	}
	strcat( command, " );" );
	/**/
	return command;
}

//*************************************************************************************
// Compute a "trend coefficient" for this subsystem
//
ECOA__float32 getTrendCoefficient( ECOA__int32 subsys, HUMSData humsData, char* humsRule )
{
	ECOA__float32 SS = 0.0, RSS;
	/**/
	float data[20];
	int i, N;
	/*
	 * The coefficient would be computed according to the "humsRule",
	 * presumably some value proportional to each parameter's deviation from "the norm".
	 *
	 * We'll ignore "rules" in this demo., and simply use the RSS of the parameters for this subsystem...
	 */
	N = humsData->SysInfoSize / sizeof(float);
	memcpy( data, humsData->SysInfo, humsData->SysInfoSize );
	for ( i = 0; i < N; i++ ){
		SS += pow((data[i] - 0.65), 2.0 ); // We know the supposed mean value of each param.
	}
	RSS = sqrt( SS );
	return RSS;
}


