/*
 * @file MissionSys_modANOther.c
 * Module Interface for Module MissionSys_modANOther
 */

#include "MissionSys_modANOther.h"
#include <stdio.h>
#include <string.h>
#include <math.h>
#ifndef M_PI
#  define M_PI 3.14159265358979
#endif
#include <gdbm.h>
#include <demo1b.h>

typedef struct {
	ECOA__hr_time  ts;
} VSSKey;

typedef struct {
	ECOA__boolean8 IsValid;
	ECOA__hr_time  ts;
	AVS__AVSType   avs;
} VSSValue;

struct timespec A_MO = {0,10000000};

// Mean Earth circumference based on mean radius of 6371km (WGS84 ellipsoid)
#define METER_PER_RAD (40030174.0/(2.0*M_PI))

#define toDegrees(x) ((x)*180.0/M_PI)
#define toRadians(x) ((x)*M_PI/180.0)

//####################################################################################################

void MissionSys_modANOther__INITIALIZE__received(MissionSys_modANOther__context *context)
{
	context->user.vssDBHandle = 0;
	context->user.dsplAddr    = NULL;
	context->user.dsplSock    = NULL;
}

void MissionSys_modANOther__START__received(MissionSys_modANOther__context *context)
{
	context->user.dsplSock = datagramSocketCreate( 0, NULL ); // open a write (unbound) socket
	context->user.dsplAddr = inetAddressGetLocalHost();
}

void MissionSys_modANOther__STOP__received(MissionSys_modANOther__context *context)
{
   /* User Code Here */
}

void MissionSys_modANOther__SHUTDOWN__received(MissionSys_modANOther__context *context)
{
   /* User Code Here */
}

static int tryOpenLclDB( MissionSys_modANOther__context *context, char *dbSource, ECOA__DBM__ConnectionId* dbHndl )
{
	ECOA__log msg;
	ECOA__return_status errc;
	ECOA__DBM__String name;
	ECOA__int32 dbmStatus = 0;
    ECOA__DBM__OpenFlags flags = ECOA__DBM__OpenFlag_READER;
    ECOA__DBM__Permissions mode = { ECOA__DBM__Permission_ALL, ECOA__DBM__Permission_ALL, ECOA__DBM__Permission_ALL };
    //
	nanosleep( &A_MO, NULL );
	name.current_size = sprintf( name.data, dbSource );
	errc = MissionSys_modANOther_container__open_lcldbm__request_sync( context, &name, 1024, flags, &mode, dbHndl, &dbmStatus );
	if( dbmStatus == 9 /* "Can't be reader" */ ){
		return 0;  // busy, try again
	}else if( errc != ECOA__return_status_OK || dbmStatus != 0 ){
		msg.current_size = sprintf( msg.data, "MissionSys:ANOther: opening dbm '%s' returned code %d, status code %d (%s)", name.data, errc, dbmStatus, gdbm_errlist[dbmStatus] );
//		MissionSys_modANOther_container__log_warning( context, msg );
		return !0; // failed, don't try again
	}
	return !0; // OK, don't try again
}

static int tryOpenRemDB( MissionSys_modANOther__context *context, char *dbSource, ECOA__DBM__ConnectionId* dbHndl )
{
	ECOA__log msg;
	ECOA__return_status errc;
	ECOA__DBM__String name;
	ECOA__int32 dbmStatus = 0;
    ECOA__DBM__OpenFlags flags = ECOA__DBM__OpenFlag_READER;
    ECOA__DBM__Permissions mode = { ECOA__DBM__Permission_ALL, ECOA__DBM__Permission_ALL, ECOA__DBM__Permission_ALL };
    //
	name.current_size = sprintf( name.data, dbSource );
	errc = MissionSys_modANOther_container__open_remdbm__request_sync( context, &name, 1024, flags, &mode, dbHndl, &dbmStatus );
	if( dbmStatus == 9 /* "Can't be reader" */ ){
		nanosleep( &A_MO, NULL );
		return 0;  // busy, try again
	}else if( errc != ECOA__return_status_OK || dbmStatus != 0 ){
		msg.current_size = sprintf( msg.data, "MissionSys:ANOther: opening dbm '%s' returned code %d, status code %d (%s)", name.data, errc, dbmStatus, gdbm_errlist[dbmStatus] );
//		MissionSys_modANOther_container__log_warning( context, msg );
		return !0; // failed, don't try again
	}
	return !0; // OK, don't try again
}

void MissionSys_modANOther__Tick__received(MissionSys_modANOther__context *context)
{
	ECOA__log msg;
	ECOA__return_status errc;
    ECOA__int32 dbmStatus, i;
	VSSValue vssVal;
	demo1B__T_Tactical_Item ti;
	//
	msg.current_size = sprintf( msg.data, "MissionSys_modANOther__Tick__received" );
//	MissionSys_modANOther_container__log_trace( context, msg );
	//
	// ******************************* Design Failure Alert *******************************
	// The underlying gdbm dbs lock when being written, so there can be no read-while-write.
	// I should therefore have designed this to use an ECOA equivalent of a semaphore, or
	// a different underlying db implementation.!
	// But I didn't, so I'll bodge it by hoping to catch between open-write-close cycles...
	// ************************************************************************************
	//
	//*************************************************************************************
	// Vehicle State (Scenario 4.4 part 2):
	//*************************************************************************************
	for( i = 0; i < 2 && context->user.vssDBHandle < 1; i++ ){
		if( tryOpenLclDB( context, "./resource/MissionSys_VSS.db", &(context->user.vssDBHandle) )) break;
	}
	if( context->user.vssDBHandle > 0){
		//
		VSSKey   vssKey;
		ECOA__DBM__Datum key, value;
		ECOA__hr_time ts;
		//
		MissionSys_modANOther_container__get_relative_local_time( context, &ts );
		vssKey.ts = (ECOA__hr_time){ 0, 0 };
		//
		memcpy( &(key.data), &vssKey, (key.current_size = sizeof( vssKey )));
		errc = MissionSys_modANOther_container__fetch_lcldbm__request_sync( context, context->user.vssDBHandle, &key, &value, &dbmStatus );
		if( errc != ECOA__return_status_OK ){
			msg.current_size = sprintf( msg.data, "MissionSys:ANOther: fetching Vehicle State returned code %d, status code %d (%s)", errc, dbmStatus, gdbm_errlist[dbmStatus] );
			MissionSys_modANOther_container__log_warning( context, msg );
			vssVal.IsValid = ECOA__FALSE;
		}else{
			memcpy( &vssVal, &(value.data), sizeof( vssVal )); // == value.current_size );
			//
//			printf( "MissionSys:ANOther: Using retrieved Vehicle State: %9.5f %9.5f %4.0f %4.0f %4.0f %d\n", vssVal.avs.Latitude, vssVal.avs.Longitude, vssVal.avs.Height, vssVal.avs.Speed, vssVal.avs.Heading, vssVal.IsValid );
		}
		// Close the VSS so that it can be written...
		errc = MissionSys_modANOther_container__close_lcldbm__request_sync( context, context->user.vssDBHandle );
		if( errc != ECOA__return_status_OK ){
			msg.current_size = sprintf( msg.data, "MissionSys:ANOther: closing Vehicle State returned code %d", errc );
			MissionSys_modANOther_container__log_warning( context, msg );
		}else
			context->user.vssDBHandle = 0;
	}else{
		vssVal.IsValid = ECOA__FALSE;
	}
	//*************************************************************************************
	// Track refresh (Scenario 4.6 part 2):
	//*************************************************************************************
	for( i = 0; i < 2 && context->user.tgtDBHandle < 1; i++ ){
		if( tryOpenRemDB( context, "./resource/MissionSys_Tgt.db", &(context->user.tgtDBHandle) )) break;
	}
	if( context->user.tgtDBHandle > 0){
		//
		ECOA__DBM__Datum key, value;
		//
		key.current_size = sprintf( key.data, "current" );
		errc = MissionSys_modANOther_container__fetch_remdbm__request_sync( context, context->user.tgtDBHandle, &key, &value, &dbmStatus );
		if( dbmStatus != 0 ){
			msg.current_size = sprintf( msg.data, "MissionSys:ANOther: fetching Current Target returned code %d, status code %d (%s)", errc, dbmStatus, gdbm_errlist[dbmStatus] );
			MissionSys_modANOther_container__log_warning( context, msg );
		}else{
			memcpy( &ti, &(value.data), value.current_size );
			//
//			printf( "MissionSys:ANOther: Using retrieved Target '%d' at ( %9.5f %9.5f %4.0f )\n", ti.Tactical_Item_ID, ti.Location.Latitude, ti.Location.Longitude, ti.Location.Height );
		}
		// Close the TargetStore so that it can be written...
		errc = MissionSys_modANOther_container__close_remdbm__request_sync( context, context->user.tgtDBHandle );
		if( errc != ECOA__return_status_OK ){
			msg.current_size = sprintf( msg.data, "MissionSys:ANOther: closing Target Store returned code %d", errc );
			MissionSys_modANOther_container__log_warning( context, msg );
		}else
			context->user.tgtDBHandle = 0;
	}else{
		ti.Is_Valid = ECOA__FALSE;
	}
	//
	//##########################################################################################################################
	// Compute range and heading to target
	// and display on the console
	//##########################################################################################################################
	if( ti.Is_Valid && vssVal.IsValid ){
		//
		ECOA__double64 dX, dY, range, hdng;
		DatagramPacket pkt = datagramPacketCreate();
		char buff[1024];
		//
		// Compute distance from where we are to where we want to be...
		dY = ( toRadians( ti.Location.Latitude  - vssVal.avs.Latitude  )) * METER_PER_RAD;
		dX = ( toRadians( ti.Location.Longitude - vssVal.avs.Longitude )) * METER_PER_RAD * cos( toRadians( vssVal.avs.Latitude ));
		range = sqrt( dX*dX + dY*dY );
		//
		// ...and from that, the direction in which we NEED to be headed in order to get to the target...
		if( dX == 0.0 && dY == 0.0 )
			hdng = 0.0;
		else
			hdng = atan2( dX, dY );
		if( hdng < 0.0 ){
			hdng += (2.0*M_PI);
		}
		//
		sprintf( buff, "=*CURRTRG:%g %g %g %g %g", ti.Location.Latitude, ti.Location.Longitude, ti.Location.Height, range, toDegrees( hdng ));
		pkt->buffer = (unsigned char *)buff;
		pkt->bufferSizeBytes = strlen( buff );
		pkt->address->value = context->user.dsplAddr->value;
		pkt->port    = 9008;
		//
		if( context->user.dsplSock ){
			datagramSocketSend( context->user.dsplSock, pkt );
		}else{
			msg.current_size = sprintf( msg.data, "MissionSys:ANOther: no HMI socket to display to...\n" );
			MissionSys_modANOther_container__log_warning( context, msg );
		}
		datagramPacketDestroy( pkt );
	}else{
		msg.current_size = sprintf( msg.data, "MissionSys:ANOther: ti is %svalid but vss is %svalid",
				(ti.Is_Valid ? "" : "not "), (vssVal.IsValid ? "" : "not "));
//		MissionSys_modANOther_container__log_warning( context, msg );
	}
}
