/* File MissionSys_modMain.c */
/*
 ##############################################################
 # (C) 2015 BAE SYSTEMS
 #
 # ECOA Data Servers Sample - Example Data Server Client
 #
 # $Id: MissionSys_modMain.c,v 2.4 2017/11/06 15:33:32 ksappleb Exp $
 #
 # Part of the BAE Systems, Rochester, ECOA Samples collection
 #
 # Created by K S Appleby, BAE Systems, Airport Works, Rochester, Kent
 #
 # This software is developed for and on behalf of BAE Systems (Operations)
 # Limited, Dassault Aviation, Bull SAS, Thales Systemes Aeroportes,
 # GE Aviation Systems Limited,
 # General Dynamics United Kingdom Limited and Leonardo MW Ltd,
 # and the copyright is owned by BAE Systems (Operations) Limited,
 # Dassault Aviation, Bull SAS, Thales Systemes Aeroportes, 
 # GE Aviation Systems Limited, General Dynamics United Kingdom
 # Limited and Leonardo MW Ltd
 #
 # This software is developed by BAE Systems (Operations)
 # Limited, Electronic Systems, and is the Intellectual
 # Property of BAE Systems (Operations) Limited, Electronic Systems.
 #
 # The information set out in this document is provided solely
 # on an 'as is' basis and the co-developers of this software make
 # no warranties expressed or implied, including no warranties
 # as to completeness, accuracy or fitness for purpose, with respect
 # to any of the information.
 ##############################################################
*/
#include "MissionSys_modMain.h"
#include "SQLIF.h"
#include "sktlib/inetAddress.h"
#include "sktlib/streamSocket.h"
#include <sktlib/datagramSocket.h>
#include <sktlib/datagramPacket.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <libxml/parser.h>
#include <libxml/tree.h>
#include <libxml/xpath.h>
#include <gdbm.h>
#include <time.h>
#include <demo1b.h>

typedef struct {
	ECOA__hr_time  ts;
} VSSKey;

typedef struct {
	ECOA__boolean8 IsValid;
	ECOA__hr_time  ts;
	AVS__AVSType   avs;
} VSSValue;

//####################################################################################################

static void assignItem( ECOA__SQL__ColumnValue src, void* dest, int destSize )
{
	double dblTemp;
	//
	switch( src.dataType ){
		case ECOA__SQL__DataType_CHAR:
		case ECOA__SQL__DataType_VARCHAR:
			if( destSize > src.dataSize ){
				strncpy( dest, (char*)(src.value), src.dataSize );
				((char*)dest)[src.dataSize] = '\000';
			}else{
				strncpy( dest, (char*)(src.value), destSize );
				((char*)dest)[destSize-1] = '\000';
			}
			break;
		case ECOA__SQL__DataType_CLOB:
		case ECOA__SQL__DataType_BLOB:
			memcpy( dest, src.value, src.dataSize );
			break;
		case ECOA__SQL__DataType_NUMERIC:
		case ECOA__SQL__DataType_DECIMAL:
		case ECOA__SQL__DataType_INTEGER:
			memcpy( dest, src.value, 4 );
			break;
		case ECOA__SQL__DataType_SMALLINT:
		case ECOA__SQL__DataType_FLOAT:
		case ECOA__SQL__DataType_REAL:
		case ECOA__SQL__DataType_DOUBLE:
			memcpy( &dblTemp, src.value, sizeof( double ));
			if( destSize == sizeof( float ))
				*((float*)dest) = (float)dblTemp;
			if( destSize == sizeof( double ))
				*((double*)dest) = (double)dblTemp;
			break;
		case ECOA__SQL__DataType_DATE://TODO
		case ECOA__SQL__DataType_TIME://TODO
		case ECOA__SQL__DataType_TIMESTAMP://TODO
			break;
	}
}

//####################################################################################################

static ECOA__log toLog( ECOA__SQL__String* s )
{
	ECOA__log msg;
    msg.current_size = s->current_size; memcpy( msg.data, s->data, s->current_size );
	return msg;
}

//####################################################################################################

static void dbConnect( MissionSys_modMain__context *context )
{
	ECOA__log msg;
	ECOA__SQL__String dataSource;
	ECOA__SQL__NameString userName, password;
	ECOA__SQL__StringZ dataSourceZ, userNameZ, passwordZ;
	int l;
	//
    MissionSys_modMain_container__get_CIMSDB_value( context, &dataSourceZ );
    MissionSys_modMain_container__get_UserName_value( context, &userNameZ );
    MissionSys_modMain_container__get_Password_value( context, &passwordZ );
	//
   	l = dataSource.current_size = strlen( dataSourceZ ); strncpy( dataSource.data, dataSourceZ, l );
   	l = userName.current_size = strlen( userNameZ );     strncpy( userName.data, userNameZ, l );
   	l = password.current_size = strlen( passwordZ );     strncpy( password.data, passwordZ,  l );
	//
	msg.current_size = sprintf( msg.data, "MissionSys:modMain: dbConnect(%s)", dataSource.data );
	MissionSys_modMain_container__log_info( context, msg );
	//
	MissionSys_modMain_container__getConnection__request_async( context, &(context->user.connectionRequestID), &dataSource, &userName, &password );
}

//####################################################################################################

static int tilToXML( demo1B__T_Tactical_List til, char* buff, int bufMax )
{
	int i;
	char lbuf[128];
	//
	if( buff != NULL ){
		strcat( buff, "urn:ecoatargs:" );
		strcat( buff, "<ECOA>" );
		strcat( buff, "<TARGETS>" );
		for( i = 0; i < til.current_size; i++ ){
			if( til.data[i].Is_Valid ){
				strcat( buff, "<TARGET>" );
				sprintf( lbuf, "<ID>%d</ID>", til.data[i].Tactical_Item_ID );
					strcat( buff, lbuf );
				sprintf( lbuf, "<POSITION>" );
					strcat( buff, lbuf );
				sprintf( lbuf, "<LAT>%g</LAT>", til.data[i].Location.Latitude );
					strcat( buff, lbuf );
				sprintf( lbuf, "<LONG>%g</LONG>", til.data[i].Location.Longitude );
					strcat( buff, lbuf );
				sprintf( lbuf, "</POSITION>" );
					strcat( buff, lbuf );
				sprintf( lbuf, "<HEIGHT>%g</HEIGHT>", til.data[i].Location.Height );
					strcat( buff, lbuf );
				sprintf( lbuf, "<DESCRIPTION>(none)</DESCRIPTION>" );
					strcat( buff, lbuf );
				strcat( buff, "</TARGET>" );
			}
		}
		strcat( buff, "</TARGETS>" );
		strcat( buff, "</ECOA>" );
	}
	return strlen( buff );
}

//####################################################################################################

#define dsplPort 9008;

static void sendTIList( DatagramSocket sock, InetAddress dsplAddr, demo1B__T_Tactical_List til )
{
	char tilXML[10240];
	DatagramPacket pkt = datagramPacketCreate();
	int  xmlSize = tilToXML( til, tilXML, 10240 );
	//
	pkt->buffer = (unsigned char *)tilXML;
	pkt->bufferSizeBytes = xmlSize;
	pkt->address->value = dsplAddr->value;
	pkt->port    = dsplPort;
	//
	datagramSocketSend( sock, pkt );
	//
	datagramPacketDestroy( pkt );
}

//####################################################################################################

static ECOA__boolean8 loadTIDB( MissionSys_modMain__context* context )
{
	ECOA__return_status errc;
	ECOA__log msg;
	ECOA__SQL__String SQLString;
	ECOA__SQL__ResultSetId transactionId;
	ECOA__SQL__Error retcode;
	ECOA__SQL__ColumnValue tiID, tiLat, tiLong, tiHeight, tiRCS, tiRadSpd;
	ECOA__SQL__ScrollSet scroll = { ECOA__SQL__ScrollOrntn_NEXT, 0 };
	ECOA__DBM__Datum key, value;
    ECOA__int32 dbmStatus = 0;
	ECOA__int32 itemCount = 0;
	demo1B__T_Tactical_Item ti;
	demo1B__T_Tactical_List til;
	//
	til.current_size = 0;
	msg.current_size = sprintf( msg.data, "MissionSys:modMain:loadTIDB: loading Tactical Items List..." );
	MissionSys_modMain_container__log_warning( context, msg );
	//
	//*************************************************************************************
	// Target Data (Scenario 4.6 (part 1)):
	//*************************************************************************************
	// load the "initial" target (tactical item) list in accordance with the mission plan
	// copy the current target (according to the mission plan) into the target short-term store
	//
	SQLString.current_size = sprintf( SQLString.data, "select * from TacticalItems;" );
	errc = MissionSys_modMain_container__SQLExecute__request_sync( context, context->user.tidbConnection, &SQLString, &transactionId );
	if( errc != ECOA__return_status_OK ){
		msg.current_size = sprintf( msg.data, "MissionSys:modMain:loadTIDB: loading Tactical Items List returned code %d", errc );
		MissionSys_modMain_container__log_warning( context, msg );
	}
	if( transactionId == 0 ){
		msg.current_size = sprintf( msg.data, "MissionSys:modMain:loadTIDB: loading Tactical Items List failed" );
		MissionSys_modMain_container__log_warning( context, msg );
		return ECOA__FALSE;
	}
	// If not connected to the TargetStore (dbm data server), do so...
	if( context->user.tgtDBHandle < 1 ){
		ECOA__DBM__String name;
	    ECOA__DBM__OpenFlags flags = ECOA__DBM__OpenFlag_WRCREAT|ECOA__DBM__OpenFlag_SYNC;
	    ECOA__DBM__Permissions mode = { ECOA__DBM__Permission_ALL, ECOA__DBM__Permission_ALL, ECOA__DBM__Permission_ALL };
	    //
	    unlink( "./resource/MissionSys_Tgt.db" );
		name.current_size = sprintf( name.data, "./resource/MissionSys_Tgt.db" );
		errc = MissionSys_modMain_container__open_remdbm__request_sync( context, &name, 1024, flags, &mode, &(context->user.tgtDBHandle), &dbmStatus );
		if( errc != ECOA__return_status_OK || ( context->user.tgtDBHandle == 0 && dbmStatus != 10 )){
			msg.current_size = sprintf( msg.data, "MissionSys:modMain: opening dbm '%s' returned code %d, status code %d (%s)", name.data, errc, dbmStatus, gdbm_errlist[dbmStatus] );
			MissionSys_modMain_container__log_warning( context, msg );
//			return ECOA__FALSE;
		}
	}
	if( context->user.tgtDBHandle > 0 ){
		for(;;){
			errc = MissionSys_modMain_container__getResult__request_sync( context, context->user.tidbConnection, transactionId, &scroll, &retcode );
			itemCount++;
			if( retcode == ECOA__SQL__Error_SQL_NO_DATA ){ // end-of-list
				break;
			}
			if( retcode != ECOA__SQL__Error_NO_ERROR ){
				msg.current_size = sprintf( msg.data, "MissionSys:modMain: MissionSys_modMain_container__getResult__request_sync() returned %d", retcode );
				MissionSys_modMain_container__log_warning( context, msg );
				break;
			}
			ti.Is_Valid = ECOA__TRUE;
			//
			errc = MissionSys_modMain_container__getResultData__request_sync( context, context->user.tidbConnection, transactionId, /*column =*/ 1, &tiID, &retcode );
			if( retcode != ECOA__SQL__Error_NO_ERROR ){
				msg.current_size = sprintf( msg.data, "MissionSys:modMain: get(tactical item Id) returned code %d", retcode );
				MissionSys_modMain_container__log_warning( context, msg );
				ti.Is_Valid = ECOA__FALSE;
			}else{
				assignItem( tiID, &(ti.Tactical_Item_ID), sizeof( demo1B__T_Tactical_Item_ID ));
			}
			// Is this a target/tactical item we want?  Check ID in list from mission plan.
			// Well, not in this demo.  We load 'em all...
			//
			if( ti.Tactical_Item_ID <= 6 ) { //isWantedTacticalItem( ti.Tactical_Item_ID )) {
				errc = MissionSys_modMain_container__getResultData__request_sync( context, context->user.tidbConnection, transactionId, /*column =*/ 2, &tiLat, &retcode );
				if( retcode != ECOA__SQL__Error_NO_ERROR ){
					msg.current_size = sprintf( msg.data, "MissionSys:modMain: get(tactical item latitude) returned code %d", retcode );
					MissionSys_modMain_container__log_warning( context, msg );
					ti.Is_Valid = ECOA__FALSE;
				}else{
					assignItem( tiLat, &(ti.Location.Latitude), sizeof( demo1B__T_Angle ));
				}
				errc = MissionSys_modMain_container__getResultData__request_sync( context, context->user.tidbConnection, transactionId, /*column =*/ 3, &tiLong, &retcode );
				if( retcode != ECOA__SQL__Error_NO_ERROR ){
					msg.current_size = sprintf( msg.data, "MissionSys:modMain: get(tactical item longitude) returned code %d", retcode );
					MissionSys_modMain_container__log_warning( context, msg );
					ti.Is_Valid = ECOA__FALSE;
				}else{
					assignItem( tiLong, &(ti.Location.Longitude), sizeof( demo1B__T_Angle ));
				}
				errc = MissionSys_modMain_container__getResultData__request_sync( context, context->user.tidbConnection, transactionId, /*column =*/ 4, &tiHeight, &retcode );
				if( retcode != ECOA__SQL__Error_NO_ERROR ){
					msg.current_size = sprintf( msg.data, "MissionSys:modMain: get(tactical item height) returned code %d", retcode );
					MissionSys_modMain_container__log_warning( context, msg );
					ti.Is_Valid = ECOA__FALSE;
				}else{
					assignItem( tiHeight, &(ti.Location.Height), sizeof( demo1B__T_Length ));
				}
				errc = MissionSys_modMain_container__getResultData__request_sync( context, context->user.tidbConnection, transactionId, /*column =*/ 6, &tiRCS, &retcode );
				if( retcode != ECOA__SQL__Error_NO_ERROR ){
					msg.current_size = sprintf( msg.data, "MissionSys:modMain: get(tactical item rcs) returned code %d", retcode );
					MissionSys_modMain_container__log_warning( context, msg );
					ti.Is_Valid = ECOA__FALSE;
				}else{
					assignItem( tiRCS, &(ti.RCS), sizeof( demo1B__T_Surface ));
				}
				errc = MissionSys_modMain_container__getResultData__request_sync( context, context->user.tidbConnection, transactionId, /*column =*/ 5, &tiRadSpd, &retcode );
				if( retcode != ECOA__SQL__Error_NO_ERROR ){
					msg.current_size = sprintf( msg.data, "MissionSys:modMain: get(tactical item rad. speed) returned code %d", retcode );
					MissionSys_modMain_container__log_warning( context, msg );
					ti.Is_Valid = ECOA__FALSE;
				}else{
					assignItem( tiRadSpd, &(ti.Radial_Speed), sizeof( demo1B__T_Speed ));
				}
//				printf( "*** Found TIDB item %d @ (%g %g %g)\n", ti.Tactical_Item_ID, ti.Location.Latitude, ti.Location.Longitude, ti.Location.Height );
				//
				// Add each (wanted) tactical item to the rapid-access db...
				memcpy( &(key.data),   &(ti.Tactical_Item_ID), (key.current_size = sizeof(ti.Tactical_Item_ID)));
				memcpy( &(value.data), &(ti), (value.current_size = sizeof(ti)));
				errc = MissionSys_modMain_container__store_remdbm__request_sync( context, context->user.tgtDBHandle, &key, &value, ECOA__DBM__StoreFlag_REPLACE, &dbmStatus );
				if( dbmStatus != 0 ){
					msg.current_size = sprintf( msg.data, "MissionSys:modMain: storing ti %d to dbm returned code %d (%s)", ti.Tactical_Item_ID, dbmStatus, gdbm_errlist[dbmStatus] );
					MissionSys_modMain_container__log_warning( context, msg );
				}
				// ...and to a temporary list...
				til.data[ til.current_size++ ] = ti;
			}
		}
		// Send the selected tactical items list to the HMI display
		if( context->user.dsplSock != NULL ){
			sendTIList( context->user.dsplSock, context->user.dsplAddr, til );
		}
		// Get current target from mission plan
		//    But not in this demo. Here, curTgt = tactical item with ID = 2...
		// Store in the rapid access Target Store db
		ti.Tactical_Item_ID = (demo1B__T_Tactical_Item_ID)2;
		memcpy( &(key.data), &(ti.Tactical_Item_ID), (key.current_size = sizeof(ti.Tactical_Item_ID)));
		errc = MissionSys_modMain_container__fetch_remdbm__request_sync( context, context->user.tgtDBHandle, &key, &value, &dbmStatus );
		if( dbmStatus != 0 ){
			msg.current_size = sprintf( msg.data, "MissionSys:modMain: fetching Target %d returned code %d, status code %d (%s)", ti.Tactical_Item_ID, errc, dbmStatus, gdbm_errlist[dbmStatus] );
			MissionSys_modMain_container__log_warning( context, msg );
		}
		//
		// We don't need to unpack the retrieved value, we just change the key and re-store it as "current"...
		key.current_size = sprintf( key.data, "current" );
		errc = MissionSys_modMain_container__store_remdbm__request_sync( context, context->user.tgtDBHandle, &key, &value, ECOA__DBM__StoreFlag_REPLACE, &dbmStatus );
		if( dbmStatus != 0 ){
			msg.current_size = sprintf( msg.data, "MissionSys:modMain: storing 'current' Target returned code %d, status code %d (%s)", errc, dbmStatus, gdbm_errlist[dbmStatus] );
			MissionSys_modMain_container__log_warning( context, msg );
		}
		//
		// Close the TargetStore so that it can be read...
		errc = MissionSys_modMain_container__close_remdbm__request_sync( context, context->user.tgtDBHandle );
		if( errc != ECOA__return_status_OK ){
			msg.current_size = sprintf( msg.data, "MissionSys:modMain: closing Target Store returned code %d", errc );
			MissionSys_modMain_container__log_warning( context, msg );
		}else
			context->user.tgtDBHandle = 0;
	}
	return ECOA__TRUE;
}

//####################################################################################################

static void addTarget(MissionSys_modMain__context *context, demo1B__T_Tactical_Item ti)
{
	ECOA__return_status errc;
	ECOA__log msg;
	ECOA__DBM__Datum key, value;
    ECOA__int32 dbmStatus = 0;
	//
	// If not connected to the TargetStore (dbm data server), do so...
	if( context->user.tgtDBHandle < 1 ){
		ECOA__DBM__String name;
	    ECOA__DBM__OpenFlags flags = ECOA__DBM__OpenFlag_WRCREAT|ECOA__DBM__OpenFlag_SYNC;
	    ECOA__DBM__Permissions mode = { ECOA__DBM__Permission_ALL, ECOA__DBM__Permission_ALL, ECOA__DBM__Permission_ALL };
	    //
		name.current_size = sprintf( name.data, "./resource/MissionSys_Tgt.db" );
		errc = MissionSys_modMain_container__open_remdbm__request_sync( context, &name, 1024, flags, &mode, &(context->user.tgtDBHandle), &dbmStatus );
		if( errc != ECOA__return_status_OK || context->user.tgtDBHandle == 0 ){
			msg.current_size = sprintf( msg.data, "MissionSys:modMain: opening dbm '%s' returned code %d, status code %d (%s)", name.data, errc, dbmStatus, gdbm_errlist[dbmStatus] );
			MissionSys_modMain_container__log_warning( context, msg );
		}
	}
	if( context->user.tgtDBHandle > 0 ){
		memcpy( &(key.data), &(ti.Tactical_Item_ID), (key.current_size = sizeof(ti.Tactical_Item_ID)));
		memcpy( &(value.data), &(ti), (value.current_size = sizeof(ti)));
		errc = MissionSys_modMain_container__store_remdbm__request_sync( context, context->user.tgtDBHandle, &key, &value, ECOA__DBM__StoreFlag_REPLACE, &dbmStatus );
		if( errc != ECOA__return_status_OK ){
			msg.current_size = sprintf( msg.data, "MissionSys:modMain:addTarget failed with status code %d (%s)", dbmStatus, gdbm_errlist[dbmStatus] );
		}
		// Close the TargetStore so that it can be read...
		errc = MissionSys_modMain_container__close_remdbm__request_sync( context, context->user.tgtDBHandle );
		if( errc != ECOA__return_status_OK ){
			msg.current_size = sprintf( msg.data, "MissionSys:modMain: closing Target Store returned code %d", errc );
			MissionSys_modMain_container__log_warning( context, msg );
		}else
			context->user.tgtDBHandle = 0;
	}
}

//####################################################################################################

void MissionSys_modMain__INITIALIZE__received(MissionSys_modMain__context *context)
{
	ECOA__log msg;
	//
	msg.current_size = sprintf( msg.data, "MissionSys_modMain__INITIALIZE__received" );
//	MissionSys_modMain_container__log_trace( context, msg );
	//
	context->user.tilLoaded = ECOA__FALSE;
	context->user.tidbConnection = 0;
	context->user.vssDBHandle = 0;
	context->user.tgtDBHandle = 0;
	context->user.connectionRequestID = 0;
	context->user.svrSock = NULL;
	context->user.dsplSock = NULL;
	context->user.prompt = ECOA__TRUE;
}

void MissionSys_modMain__START__received(MissionSys_modMain__context *context)
{
    ECOA__int32 dbmStatus = 0;
	ECOA__return_status errc;
	ECOA__log msg;
    //
	msg.current_size = sprintf( msg.data, "MissionSys_modMain__START__received" );
//	MissionSys_modMain_container__log_trace( context, msg );
}

void MissionSys_modMain__STOP__received(MissionSys_modMain__context *context)
{
   /* User Code Here */
}

void MissionSys_modMain__SHUTDOWN__received(MissionSys_modMain__context *context)
{
   /* User Code Here */
}

//####################################################################################################

void MissionSys_modMain__getConnection__response_received
   (MissionSys_modMain__context* context,    const ECOA__uint32 ID, const ECOA__return_status status,
    const ECOA__SQL__ConnectionId connection,
    const ECOA__SQL__Error requestStatus)
{
	ECOA__log msg;
	//
	if( requestStatus != ECOA__SQL__Error_NO_ERROR ){
		msg.current_size = sprintf( msg.data, "MissionSys:modMain: connecting to datasource returned code %d", requestStatus );
		MissionSys_modMain_container__log_warning( context, msg );
		context->user.tidbConnection = 0;
	}else{
		context->user.tidbConnection = (ECOA__SQL__ConnectionId)connection;
		msg.current_size = sprintf( msg.data, "MissionSys_modMain__getConnection__response_received connectionID = 0x%08x", (unsigned int)connection );
//		MissionSys_modMain_container__log_trace( context, msg );
	}
}

void MissionSys_modMain__tableUpdateNotification__received(MissionSys_modMain__context *context,
    const ECOA__SQL__NameString *tableName)
{
   /* User Code Here */
}

//####################################################################################################

void MissionSys_modMain__AVSUpdate__received(MissionSys_modMain__context *context,
    const AVS__AVS *avs)
{
	ECOA__return_status errc;
	ECOA__log msg;
    ECOA__int32 dbmStatus = 0;
	VSSValue vssVal;
	xmlDocPtr  pDoc = NULL;
	xmlXPathContextPtr xpathCtx;
	xmlXPathObjectPtr xpathObj;
	xmlChar* p;
	//
	vssVal.IsValid = ECOA__FALSE;
	if(( pDoc = xmlParseMemory( (char*)avs->data, avs->current_size )) == NULL ){
		msg.current_size = sprintf( msg.data, "Invalid AVS %s", avs->data );
		MissionSys_modMain_container__log_warning( context, msg );
		return;
	}
	if(( xpathCtx = xmlXPathNewContext(pDoc)) == NULL ){
		msg.current_size = sprintf( msg.data, "Invalid AVS content" );
		MissionSys_modMain_container__log_warning( context, msg );
        xmlFreeDoc(pDoc);
		return;
	}
	if(( xpathObj = xmlXPathEval((xmlChar*)"/AVS/Latitude", xpathCtx)) == NULL ){
		msg.current_size = sprintf( msg.data, "Invalid AVS content: missing Latitude" );
		MissionSys_modMain_container__log_warning( context, msg );
        xmlXPathFreeContext(xpathCtx);
        xmlFreeDoc(pDoc);
		return;
	}else{
		vssVal.avs.Latitude = atof( (char*)(p = xmlNodeGetContent( xpathObj->nodesetval->nodeTab[0] )));
		xmlFree(p);
		xmlXPathFreeObject( xpathObj );
	}
	if(( xpathObj = xmlXPathEval((xmlChar*)"/AVS/Longitude", xpathCtx)) == NULL ){
		msg.current_size = sprintf( msg.data, "Invalid AVS content: missing Longitude" );
		MissionSys_modMain_container__log_warning( context, msg );
        xmlXPathFreeContext(xpathCtx);
        xmlFreeDoc(pDoc);
		return;
	}else{
		vssVal.avs.Longitude = atof( (char*)(p = xmlNodeGetContent( xpathObj->nodesetval->nodeTab[0] )));
		xmlFree(p);
		xmlXPathFreeObject( xpathObj );
	}
	if(( xpathObj = xmlXPathEval((xmlChar*)"/AVS/Height", xpathCtx)) == NULL ){
		msg.current_size = sprintf( msg.data, "Invalid AVS content: missing Height" );
		MissionSys_modMain_container__log_warning( context, msg );
        xmlXPathFreeContext(xpathCtx);
        xmlFreeDoc(pDoc);
		return;
	}else{
		vssVal.avs.Height = atof( (char*)(p = xmlNodeGetContent( xpathObj->nodesetval->nodeTab[0] )));
		xmlFree(p);
		xmlXPathFreeObject( xpathObj );
	}
	if(( xpathObj = xmlXPathEval((xmlChar*)"/AVS/Speed", xpathCtx)) == NULL ){
		msg.current_size = sprintf( msg.data, "Invalid AVS content: missing Speed" );
		MissionSys_modMain_container__log_warning( context, msg );
        xmlXPathFreeContext(xpathCtx);
        xmlFreeDoc(pDoc);
		return;
	}else{
		vssVal.avs.Speed = atof( (char*)(p = xmlNodeGetContent( xpathObj->nodesetval->nodeTab[0] )));
		xmlFree(p);
		xmlXPathFreeObject( xpathObj );
	}
	if(( xpathObj = xmlXPathEval((xmlChar*)"/AVS/Heading", xpathCtx)) == NULL ){
		msg.current_size = sprintf( msg.data, "Invalid AVS content: missing Heading" );
		MissionSys_modMain_container__log_warning( context, msg );
        xmlXPathFreeContext(xpathCtx);
        xmlFreeDoc(pDoc);
		return;
	}else{
		vssVal.avs.Heading = atof( (char*)(p = xmlNodeGetContent( xpathObj->nodesetval->nodeTab[0] )));
		xmlFree(p);
		xmlXPathFreeObject( xpathObj );
	}
    xmlXPathFreeContext(xpathCtx);
    xmlFreeDoc(pDoc);
	vssVal.IsValid = ECOA__TRUE;
	//
#if 0
	msg.current_size = sprintf( msg.data, "AVS = %9.5f %9.5f %4.0f %4.0f %4.0f %d",
			vssVal.avs.Latitude, vssVal.avs.Longitude,
			vssVal.avs.Height,
			vssVal.avs.Speed,
			vssVal.avs.Heading,
			vssVal.IsValid );
	MissionSys_modMain_container__log_trace( context, msg );
#endif
	//
	//*************************************************************************************
	// Vehicle State (Scenario 4.4 part 1):
	//*************************************************************************************
	// If not connected to the Vehicle State Store (dbm data server), do so...
	if( context->user.vssDBHandle < 1 ){
		ECOA__DBM__String name;
	    ECOA__DBM__OpenFlags flags = ECOA__DBM__OpenFlag_WRCREAT|ECOA__DBM__OpenFlag_SYNC;
	    ECOA__DBM__Permissions mode = { ECOA__DBM__Permission_ALL, ECOA__DBM__Permission_ALL, ECOA__DBM__Permission_ALL };
	    //
		name.current_size = sprintf( name.data, "./resource/MissionSys_VSS.db" );
		errc = MissionSys_modMain_container__open_lcldbm__request_sync( context, &name, 1024, flags, &mode, &(context->user.vssDBHandle), &dbmStatus );
		if( errc != ECOA__return_status_OK  || ( context->user.vssDBHandle == 0 && dbmStatus != 10 )){
			msg.current_size = sprintf( msg.data, "MissionSys:modMain: opening dbm '%s' returned code %d, status code %d (%s)", name.data, errc, dbmStatus, gdbm_errlist[dbmStatus] );
			MissionSys_modMain_container__log_warning( context, msg );
		}
	}
	if( context->user.vssDBHandle > 0){
		//
		VSSKey   vssKey;
		ECOA__DBM__Datum key, value;
		ECOA__hr_time ts;
		//
		MissionSys_modMain_container__get_relative_local_time( context, &ts );
		vssVal.ts = vssKey.ts = ts;
//		vssVal.avs  is already set...
		//
		// Save the VSS once for the history...
		memcpy( &(key.data), &vssKey, (key.current_size = sizeof( vssKey )));
		memcpy( &(value.data), &vssVal, (value.current_size = sizeof( vssVal )));
		errc = MissionSys_modMain_container__store_lcldbm__request_sync( context, context->user.vssDBHandle, &key, &value, ECOA__DBM__StoreFlag_INSERT, &dbmStatus );
		if( errc != ECOA__return_status_OK ){
			msg.current_size = sprintf( msg.data, "MissionSys:modMain: saving Vehicle State returned code %d, status code %d (%s)", errc, dbmStatus, gdbm_errlist[dbmStatus] );
			MissionSys_modMain_container__log_warning( context, msg );
		}
		// ...and once for immediate use (key timestamp = 0.0).
		vssKey.ts = (ECOA__hr_time){ 0, 0 };
		memcpy( &(key.data), &vssKey, (key.current_size = sizeof( vssKey )));
		errc = MissionSys_modMain_container__store_lcldbm__request_sync( context, context->user.vssDBHandle, &key, &value, ECOA__DBM__StoreFlag_REPLACE, &dbmStatus );
		if( errc != ECOA__return_status_OK ){
			msg.current_size = sprintf( msg.data, "MissionSys:modMain: saving Vehicle State returned code %d, status code %d (%s)", errc, dbmStatus, gdbm_errlist[dbmStatus] );
			MissionSys_modMain_container__log_warning( context, msg );
		}
		// Close the VSS so that others can read it...
		errc = MissionSys_modMain_container__close_lcldbm__request_sync( context, context->user.vssDBHandle );
		if( errc != ECOA__return_status_OK ){
			msg.current_size = sprintf( msg.data, "MissionSys:modMain: closing Vehicle State returned code %d", errc );
			MissionSys_modMain_container__log_warning( context, msg );
		}else
			context->user.vssDBHandle = 0;
	}
}

//####################################################################################################

static char* tsText( ECOA__global_time gts )
{
	static char buff[1024];
	time_t ts = (time_t)(gts.seconds);
	struct tm *lts = localtime( &ts );
	//
	sprintf( buff, "%02d:%02d %02d/%02d/%04d", lts->tm_hour, lts->tm_min, lts->tm_mday, lts->tm_mon, lts->tm_year+1900 );
	return buff;
}

void MissionSys_modMain__getWeather__response_received
   (MissionSys_modMain__context* context,
    const ECOA__uint32 ID, const ECOA__return_status status,
    const Weather__WeatherRecd* weather)
{
	ECOA__int32 bytes;
	char response[1024];
	DatagramPacket pkt = datagramPacketCreate();
	//
	if( context->user.msgSock ){
		sprintf( response, "Weather report for %s at %s:\n  %g C; %.1f MPH %s; %g MB; %s\n", weather->Location, tsText(weather->Timestamp), weather->TemperatureC, weather->WindSpeedMPH, weather->WindDir, weather->PressureMB, weather->Synopsis );
//		printf( "*** MissionSys_modMain__getWeather__response_received sending '%s'\n", response );
		bytes = streamSocketSend( context->user.msgSock, response, strlen(response) );
		bytes = streamSocketSend( context->user.msgSock, (void*)"CIMSDB> ", 8 );
		//
		// Echo the response to the info display
		sprintf( response, "=*INFO:Weather report for %s at %s:\n  %g C; %.1f MPH %s; %g MB; %s\n", weather->Location, tsText(weather->Timestamp), weather->TemperatureC, weather->WindSpeedMPH, weather->WindDir, weather->PressureMB, weather->Synopsis );
		pkt->buffer = response;
		pkt->bufferSizeBytes = strlen( response );
		pkt->address->value = context->user.dsplAddr->value;
		pkt->port    = dsplPort;
		//
		datagramSocketSend( context->user.dsplSock, pkt );
		//
		datagramPacketDestroy( pkt );
	}
	context->user.prompt = ECOA__TRUE;
}

//####################################################################################################

void MissionSys_modMain__getForecastWeather__response_received
   (MissionSys_modMain__context* context,
    const ECOA__uint32 ID, const ECOA__return_status status,
    const Weather__WeatherRecd* weather)
{
	// TODO
}

//####################################################################################################

#define CTRL_D '\004'
#define CTRL_Z '\032'

static ECOA__char8* trim( ECOA__char8* s, ECOA__char8* buff )
// strips whitespace from beginning and end of s, using buff as temp store
{
	ECOA__char8 *ps = s, *pb = buff, *pl = s + strlen(s) -1;
	while( isspace( (int)*ps ) && ps <= pl ) ps++;
	while( isspace( (int)*pl ) && pl > ps ) pl--;
	while( ps <= pl ) *pb++ = *ps++;
	*pb = '\0';
	return buff;
}

//####################################################################################################

#define RESPONSEMAX 100000

void MissionSys_modMain__Tick__received(MissionSys_modMain__context *context)
{
	ECOA__log msg;
	ECOA__char8 command[ECOA__SQL__String_MAXSIZE], response[RESPONSEMAX];
	ECOA__int32 bytes, res;
	ECOA__uint32 ID;
	//
	fflush(stdout); // Make sure that any piped IO is completed
	//
	// Once we're connected, load the tactical items database
	if( context->user.tidbConnection > 0 && !context->user. tilLoaded ){
		context->user.tilLoaded = loadTIDB( context );
	}
	//
	msg.current_size = sprintf( msg.data, "MissionSys_modMain__Tick__received" );
//	MissionSys_modMain_container__log_trace( context, msg );
	//
	// If the HMI (console) socket is not open, do so...
	if( context->user.svrSock == NULL ){
		InetAddress self = inetAddressGetLocalHost();
		// Open a SERVER (i.e. unconnected) socket
		context->user.svrSock = streamSocketBind( self, 9009 );
		msg.current_size = sprintf( msg.data, "MissionSys:modMain: Server socket on %d", context->user.svrSock->descriptor );
//		MissionSys_modMain_container__log_trace( context, msg );
		streamSocketSetBlockingMode( context->user.svrSock, 0 );
		streamSocketSetTimeout( context->user.svrSock, 0.1 );
	}
	// If the HMI (display) socket is not open, do so...
	if( context->user.dsplSock == NULL ){
		context->user.dsplSock = datagramSocketCreate( 0, NULL );
		context->user.dsplAddr = inetAddressGetLocalHost();
	}
	// If the tactical items database is not connected, make it so...
	if( context->user.tidbConnection < 1 ){
		dbConnect( context );
	}
	//*******************************************************************************************************************
	// Scenario 4.7 (Tactical Data) is implemented by opening a port to a console.  An operator can then enter queries
	// which will be processed here and the results returned.
	//*******************************************************************************************************************
	if( context->user.msgSock == NULL ){
		// If not, do we have a pending connection?
		if(( context->user.msgSock = streamSocketAccept( context->user.svrSock )) != NULL ){
			streamSocketSetTimeout( context->user.msgSock, 0.1 );
			msg.current_size = sprintf( msg.data, "MissionSys:modMain: Connection accepted on %d", context->user.msgSock->descriptor );
//			MissionSys_modMain_container__log_trace( context, msg );
			//
			// Prompt the user
			sprintf( response,
					"****************************************\n"
					"* Tactical Items SQL command processor *\n"
					"****************************************\n"
					"\n"
					"CIMSDB> " );
			bytes= streamSocketSend( context->user.msgSock, response, strlen( response ));
		}
	}
	if( context->user.msgSock != NULL ){
		// With a connection, read in any message(s) in and process it/them (Scenario 4.7)
		for(;;){
			memset( command, 0, sizeof( command ));
			bytes = streamSocketReceive( context->user.msgSock, command, sizeof( command ));
			if( bytes > 0 ){ // => message received
				//
				if(( strncasecmp( command, "exit", 4 ) == 0 ) ||
						( strncasecmp( command, "quit", 4 ) == 0 ) ||
						command[0] == CTRL_D || command[0] == CTRL_Z ){
					msg.current_size = sprintf( msg.data, "MissionSys:modMain: connection closed." );
//					MissionSys_modMain_container__log_trace( context, msg );
					streamSocketDestroy( context->user.msgSock );
					context->user.msgSock = NULL;
					break;
				}
				if( strncasecmp( command, "help", 4 ) == 0 ){
					char buff[1024];
					sprintf( response, "This SQL command interface allows the following commands:\n"
							"  help           obviously...\n"
							"  exit           terminates the interface\n" );
					strcat( response, SQLIF_Help( buff ));
					strcat( response,
							"  get weather for location;\n"
							"                 get the weather for location\n"
							"  get forecast for location;\n"
							"                 get the weather for location (not implemented yet)\n"
							"  new target     change the current target (not implemented yet)\n"
							"  add target     add target to the TIDB (not implemented yet)\n"
							);
					bytes = streamSocketSend( context->user.msgSock, response, strlen(response) );
				}else
				if( strncasecmp( command, "new", 3 ) == 0 ){
					// Override the "current" target
					;//TODO
				}else
				if( strncasecmp( command, "add", 3 ) == 0 ){
					// Add a new target
					;//TODO
				}else
				if( strncasecmp( command, "get", 3 ) == 0 ){
					// Get the weather for a location
					// e.g.:  get weather for ME8;             // by postcode
					//        get weather for Macklesfield;    // by placename
					//        get weather for 53.156,-1.959;   // by lat,long
					if( strncasecmp( command, "get weather for ", 16 ) == 0 ){
						char location[32];
						int i;
						//
						for( i = 16; i < strlen( command ) && command[i] != ';'; i++ ){
							location[i-16] = command[i];
						}
						location[i-15] = '\0';
//						printf( "*** MissionSys_modMain_container__getWeather__request_async called for %s\n", location );
						MissionSys_modMain_container__getWeather__request_async( context, &ID, (Weather__wxLocationString*)location );
						//
						// Stall the prompt until the asynchronous response is received...
						context->user.prompt = ECOA__FALSE;
					}else
					// Get a forecast for a location
					if( strncasecmp( command, "get forecast for ", 17 ) == 0 ){
						;//TODO
					}
				}else
				if( strlen( trim( command, response )) == 0 ){
					;// do nothing...
				}else
				{
					// Process received command as SQL...
					res = SQLIF_processCommand( context, command, response );
					// Return any response
					if( !res && strlen( response ) > 0 ){
						bytes = streamSocketSend( context->user.msgSock, response, strlen(response) );
					}
				}
				if( context->user.prompt ){
					// Prompt the user again
					bytes= streamSocketSend( context->user.msgSock, (void*)"CIMSDB> ", 8 );
				}
				//
			}else
			if( bytes < 0 ){ // => connection closed
				msg.current_size = sprintf( msg.data, "MissionSys:modMain: Connection closed." );
				MissionSys_modMain_container__log_trace( context, msg );
				streamSocketDestroy( context->user.msgSock );
				context->user.msgSock = NULL;
				break;
			}
			else{
			//  bytes == 0 // => timedout.  Go and try for more input...
				break;
			}
		}
	}
}

