/*
 ##############################################################
 # (C) 2015 BAE SYSTEMS
 #
 # ECOA Data Servers Sample - Example Data Server Client command-line interface
 #
 # $Id: SQLIF.c,v 2.1 2017/11/06 15:33:32 ksappleb Exp $
 #
 # Part of the BAE Systems, Rochester, ECOA Samples collection
 #
 # Created by K S Appleby, BAE Systems, Airport Works, Rochester, Kent
 #
 # This software is developed for and on behalf of BAE Systems (Operations)
 # Limited, Dassault Aviation, Bull SAS, Thales Systemes Aeroportes,
 # GE Aviation Systems Limited,
 # General Dynamics United Kingdom Limited and Leonardo MW Ltd,
 # and the copyright is owned by BAE Systems (Operations) Limited,
 # Dassault Aviation, Bull SAS, Thales Systemes Aeroportes, 
 # GE Aviation Systems Limited, General Dynamics United Kingdom
 # Limited and Leonardo MW Ltd
 #
 # This software is developed by BAE Systems (Operations)
 # Limited, Electronic Systems, and is the Intellectual
 # Property of BAE Systems (Operations) Limited, Electronic Systems.
 #
 # The information set out in this document is provided solely
 # on an 'as is' basis and the co-developers of this software make
 # no warranties expressed or implied, including no warranties
 # as to completeness, accuracy or fitness for purpose, with respect
 # to any of the information.
 ##############################################################
*/
#include "SQLIF.h"
#include <ECOA__SQL.h>
#include <sktlib/inetAddress.h>
#include <sktlib/streamSocket.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sqltypes.h>

//***********************************************************************************************

typedef struct {

	ECOA__int32 rowCount;
	ECOA__SQL__Error status;
	ECOA__SQL__ConnectionId connection;
	ECOA__SQL__ResultSetId  transactionId;
	ECOA__SQL__Buffer resultBuff;

} sqlResultSet;

//***********************************************************************************************

static void listTables( MissionSys_modMain__context *context, ECOA__char8* resultBuff )
{
	ECOA__return_status errc;
	ECOA__log msg;
	ECOA__SQL__TableNameList tableList;
	ECOA__SQL__Error retcode;
	ECOA__int32 i;
	ECOA__char8 buff[1024];
	//
	errc = MissionSys_modMain_container__listTables__request_sync( context, context->user.tidbConnection, &tableList, &retcode );
		if( errc != ECOA__return_status_OK ){
			msg.current_size = sprintf( msg.data, "MissionSys: SQLIF: listTables returned code %d", errc );
			MissionSys_modMain_container__log_warning( context, msg );
			return;
		}
	resultBuff[0] = '\000';
	for( i = 0; i < tableList.current_size; i++ ){
		ECOA__SQL__NameString tableName = tableList.data[i];
		tableName.data[tableName.current_size] = '\000';
		sprintf( buff, "%s\n", tableName.data );
		strcat( resultBuff, buff );
	}
}

//***********************************************************************************************

static char ttRes[1024];

static char* date_toText( ECOA__SQL__Buffer value )
{
	sprintf( ttRes, "#%4d-%02d-%02d#", ((DATE_STRUCT*)value)->year, ((DATE_STRUCT*)value)->month, ((DATE_STRUCT*)value)->day );
	return ttRes;
}

static char* time_toText( ECOA__SQL__Buffer value )
{
	sprintf( ttRes, "#%02d:%02d:%02d#", ((TIME_STRUCT*)value)->hour, ((TIME_STRUCT*)value)->minute, ((TIME_STRUCT*)value)->second );
	return ttRes;
}

static char* stamp_toText( ECOA__SQL__Buffer value )
{
	sprintf( ttRes, "#%4d-%02d-%02d %02d:%02d:%02d.%d#",
			((TIMESTAMP_STRUCT*)value)->year, ((TIMESTAMP_STRUCT*)value)->month, ((TIMESTAMP_STRUCT*)value)->day,
			((TIMESTAMP_STRUCT*)value)->hour, ((TIMESTAMP_STRUCT*)value)->minute, ((TIMESTAMP_STRUCT*)value)->second,
			((TIMESTAMP_STRUCT*)value)->fraction );
	return ttRes;
}

static char* slice( const char* chars, size_t numb )
{
	strncpy( ttRes, chars, numb );
	ttRes[numb] = '\000';
	return ttRes;
}

//***********************************************************************************************

static sqlResultSet* sqlParse( MissionSys_modMain__context *context, char *sql )
{
	ECOA__return_status errc;
	ECOA__log msg;
	ECOA__SQL__String SQLString;
	ECOA__SQL__ResultSetId transactionId;
	ECOA__SQL__Error retcode;
	ECOA__SQL__ScrollSet scroll = { ECOA__SQL__ScrollOrntn_NEXT, 0 };
	ECOA__SQL__ColumnInfo columnInfo;
	ECOA__SQL__ColumnValue columnValue;
	ECOA__uint16 columnCount = 0, column;
	char buff[1024];
	sqlResultSet* rslt = (sqlResultSet*)malloc( sizeof(sqlResultSet));
	//
	memset( rslt->resultBuff, 0, sizeof( ECOA__SQL__Buffer ));
	//
	SQLString.current_size = sprintf( SQLString.data, sql );
	errc = MissionSys_modMain_container__SQLExecute__request_sync( context, context->user.tidbConnection, &SQLString, &transactionId );
		if( errc != ECOA__return_status_OK ){
			msg.current_size = sprintf( msg.data, "MissionSys: SQLIF: '%s' returned code %d", sql, errc );
			MissionSys_modMain_container__log_warning( context, msg );
			//
			rslt->rowCount = 0;
			rslt->status = ECOA__SQL__Error_SQL_ERROR;
			rslt->resultBuff[0] = '\000';
			//
			return rslt;
		}
		if( transactionId == 0 ){
			msg.current_size = sprintf( msg.data, "MissionSys: SQLIF: nil transaction return" );
			MissionSys_modMain_container__log_warning( context, msg );
			//
			rslt->rowCount = 0;
			rslt->status = ECOA__SQL__Error_SQL_ERROR;
			rslt->resultBuff[0] = '\000';
			//
			return rslt;
		}
	// All's well so far, so populate the resultSet object.
	rslt->connection = context->user.tidbConnection;
	rslt->transactionId = transactionId;
	//
	// Textify the result set
	errc = MissionSys_modMain_container__getResultColumnCount__request_sync( context, rslt->connection, rslt->transactionId, &columnCount, &retcode );
	// Writes the column names to the resultBuff
	for( column = 1; column <= columnCount; column++ ){
		errc = MissionSys_modMain_container__getResultColumnInfo__request_sync( context, rslt->connection, rslt->transactionId, column, &columnInfo, &retcode );
		if( column == 1 ){
			sprintf( buff, "\"%s\"", slice( columnInfo.name.data, columnInfo.name.current_size ));
			strcpy( ((char*)rslt->resultBuff), buff );
		}else{
			sprintf( buff, ", \"%s\"", slice( columnInfo.name.data, columnInfo.name.current_size ));
			strcat( ((char*)rslt->resultBuff), buff );
		}
	}
	strcat( ((char*)rslt->resultBuff), "\n" );
	rslt->rowCount = 0;
	for(;;){
		errc = MissionSys_modMain_container__getResult__request_sync( context, context->user.tidbConnection, transactionId, &scroll, &retcode );
		if( retcode == ECOA__SQL__Error_SQL_NO_DATA ){ // end-of-list
			break;
		}
		if( retcode != ECOA__SQL__Error_NO_ERROR ){
			msg.current_size = sprintf( msg.data, "MissionSys: SQLIF: getResult returned %d for row %d", retcode, rslt->rowCount );
			MissionSys_modMain_container__log_warning( context, msg );
			rslt->status = retcode;
			break;
		}
		rslt->rowCount++;
		for( column = 1; column <= columnCount; column++ ){
			errc = MissionSys_modMain_container__getResultColumnInfo__request_sync( context, rslt->connection, rslt->transactionId, column, &columnInfo, &retcode );
			switch( columnInfo.dataType ){
				case ECOA__SQL__DataType_CHAR:
					errc = MissionSys_modMain_container__getResultData__request_sync( context, rslt->connection, rslt->transactionId, column, &columnValue, &retcode );
					sprintf( buff, "\"%s\"", slice( (char*)columnValue.value, columnValue.dataSize ));
					break;
				case ECOA__SQL__DataType_VARCHAR:
					errc = MissionSys_modMain_container__getResultData__request_sync( context, rslt->connection, rslt->transactionId, column, &columnValue, &retcode );
					sprintf( buff, "\"%s\"", slice( (char*)columnValue.value, columnValue.dataSize ));
					break;
				case ECOA__SQL__DataType_NUMERIC:
					errc = MissionSys_modMain_container__getResultData__request_sync( context, rslt->connection, rslt->transactionId, column, &columnValue, &retcode );
					sprintf( buff, "%.7g", *(double*)columnValue.value );
					break;
				case ECOA__SQL__DataType_DECIMAL:
					errc = MissionSys_modMain_container__getResultData__request_sync( context, rslt->connection, rslt->transactionId, column, &columnValue, &retcode );
					sprintf( buff, "%.7g", *(double*)columnValue.value );
					break;
				case ECOA__SQL__DataType_INTEGER:
					errc = MissionSys_modMain_container__getResultData__request_sync( context, rslt->connection, rslt->transactionId, column, &columnValue, &retcode );
					sprintf( buff, "%d", *(int*)columnValue.value );
					break;
				case ECOA__SQL__DataType_SMALLINT:
					errc = MissionSys_modMain_container__getResultData__request_sync( context, rslt->connection, rslt->transactionId, column, &columnValue, &retcode );
					sprintf( buff, "%d", *(int*)columnValue.value );
					break;
				case ECOA__SQL__DataType_FLOAT:
					errc = MissionSys_modMain_container__getResultData__request_sync( context, rslt->connection, rslt->transactionId, column, &columnValue, &retcode );
					sprintf( buff, "%.7g", *(double*)columnValue.value );
					break;
				case ECOA__SQL__DataType_REAL:
					errc = MissionSys_modMain_container__getResultData__request_sync( context, rslt->connection, rslt->transactionId, column, &columnValue, &retcode );
					sprintf( buff, "%.7g", *(double*)columnValue.value );
					break;
				case ECOA__SQL__DataType_DOUBLE:
					errc = MissionSys_modMain_container__getResultData__request_sync( context, rslt->connection, rslt->transactionId, column, &columnValue, &retcode );
					sprintf( buff, "%.7g", *(double*)columnValue.value );
					break;
				case ECOA__SQL__DataType_DATE:
					errc = MissionSys_modMain_container__getResultData__request_sync( context, rslt->connection, rslt->transactionId, column, &columnValue, &retcode );
					sprintf( buff, "%s", date_toText( columnValue.value ));
					break;
				case ECOA__SQL__DataType_TIME:
					errc = MissionSys_modMain_container__getResultData__request_sync( context, rslt->connection, rslt->transactionId, column, &columnValue, &retcode );
					sprintf( buff, "%s", time_toText( columnValue.value ));
					break;
				case ECOA__SQL__DataType_TIMESTAMP:
					errc = MissionSys_modMain_container__getResultData__request_sync( context, rslt->connection, rslt->transactionId, column, &columnValue, &retcode );
					sprintf( buff, "%s", stamp_toText( columnValue.value ));
					break;
				case ECOA__SQL__DataType_CLOB:
					errc = MissionSys_modMain_container__getResultData__request_sync( context, rslt->connection, rslt->transactionId, column, &columnValue, &retcode );
					sprintf( buff, (char*)columnValue.value );
					break;
				case ECOA__SQL__DataType_BLOB:
					errc = MissionSys_modMain_container__getResultData__request_sync( context, rslt->connection, rslt->transactionId, column, &columnValue, &retcode );
					sprintf( buff, (char*)columnValue.value );
					break;
				default:
					errc = MissionSys_modMain_container__getResultData__request_sync( context, rslt->connection, rslt->transactionId, column, &columnValue, &retcode );
					sprintf( buff, "(unknown type code %d size %d) %s", columnInfo.dataType, columnValue.dataSize, slice( (char*)columnValue.value, columnValue.dataSize ));
			}
			if( column == 1 ){
				strcat( ((char*)rslt->resultBuff), buff );
			}else{
				strcat( ((char*)rslt->resultBuff), ", " );
				strcat( ((char*)rslt->resultBuff), buff );
			}
			rslt->status = retcode;
			if( retcode != ECOA__SQL__Error_NO_ERROR ) break;
		}
		strcat( ((char*)rslt->resultBuff), "\n" );
	}
	//
	msg.current_size = sprintf( msg.data, "MissionSys: SQLIF: parsed '%s' got %d rows", sql, rslt->rowCount );
	MissionSys_modMain_container__log_trace( context, msg );
	//
	return rslt;
}

//***********************************************************************************************

char* SQLIF_Help( ECOA__char8* resultBuff )
{
	sprintf( resultBuff,
			"  show tables    lists the tables that you may query\n"
			"  take file      reads SQL from file and parses it\n"
			"  dump file      dumps the entire database to file\n"
			"  xmltake file   reads XML from file and loads the database accordingly\n"
			"  xmldump file   dumps the entire databse as XML to file\n"
			"  SQL statement  any valid SQL statement (e.g. 'SELECT * FROM myTable;')\n"
			"                 SQL statements must be terminated with a semi-colon\n" );
	return resultBuff;
}

//***********************************************************************************************

int SQLIF_processCommand(MissionSys_modMain__context *context, ECOA__char8* stringToParse, ECOA__char8* resultBuff )
{
	ECOA__SQL__Error retcode;
	ECOA__return_status errc;
	ECOA__log msg;
	ECOA__SQL__String fname;
	sqlResultSet* resultSet;
	char buff[128];
	//
	// Sort out SQLIF commands...
	if( strncasecmp( stringToParse, "show tables", 11 ) == 0 ){
		listTables( context, resultBuff );
		return 0;
	}else
	if( strncasecmp( stringToParse, "take", 4 ) == 0 ){
		fname.current_size = sprintf( fname.data, strtok( &(stringToParse[5]), "; \n" ));
		errc = MissionSys_modMain_container__take__request_sync( context, context->user.tidbConnection, &fname, ECOA__SQL__FileType_SQL, &retcode );
		if( errc != ECOA__return_status_OK ){
			msg.current_size = sprintf( msg.data, "MissionSys: SQLIF: '%s' returned code %d", stringToParse, errc );
			MissionSys_modMain_container__log_warning( context, msg );
			sprintf( resultBuff, "Failed\n" );
			return -1;
		}else{
			sprintf( resultBuff, "OK\n" );
		}
		return 0;
	}else
	if( strncasecmp( stringToParse, "dump", 4 ) == 0 ){
		fname.current_size = sprintf( fname.data, strtok( &(stringToParse[5]), "; \n" ));
		errc = MissionSys_modMain_container__dump__request_sync( context, context->user.tidbConnection, &fname, ECOA__SQL__FileType_SQL, &retcode );
		if( errc != ECOA__return_status_OK ){
			msg.current_size = sprintf( msg.data, "MissionSys: SQLIF: '%s' returned code %d", stringToParse, errc );
			MissionSys_modMain_container__log_warning( context, msg );
			sprintf( resultBuff, "Failed\n" );
			return -1;
		}else{
			sprintf( resultBuff, "OK\n" );
		}
		return 0;
	}else
	if( strncasecmp( stringToParse, "xmltake", 7 ) == 0 ){
		fname.current_size = sprintf( fname.data, strtok( &(stringToParse[5]), "; \n" ));
		errc = MissionSys_modMain_container__take__request_sync( context, context->user.tidbConnection, &fname, ECOA__SQL__FileType_XML, &retcode );
		if( errc != ECOA__return_status_OK ){
			msg.current_size = sprintf( msg.data, "MissionSys: SQLIF: '%s' returned code %d", stringToParse, errc );
			MissionSys_modMain_container__log_warning( context, msg );
			sprintf( resultBuff, "Failed\n" );
			return -1;
		}else{
			sprintf( resultBuff, "OK\n" );
		}
		return 0;
	}else
	if( strncasecmp( stringToParse, "xmldump", 7 ) == 0 ){
		fname.current_size = sprintf( fname.data, strtok( &(stringToParse[5]), "; \n" ));
		errc = MissionSys_modMain_container__dump__request_sync( context, context->user.tidbConnection, &fname, ECOA__SQL__FileType_XML, &retcode );
		if( errc != ECOA__return_status_OK ){
			msg.current_size = sprintf( msg.data, "MissionSys: SQLIF: '%s' returned code %d", stringToParse, errc );
			MissionSys_modMain_container__log_warning( context, msg );
			sprintf( resultBuff, "Failed\n" );
			return -1;
		}else{
			sprintf( resultBuff, "OK\n" );
		}
		return 0;
	}
	// If it wasn't one of them, parse it as SQL.
	resultSet = sqlParse( context, stringToParse );
	//
	if( resultSet->status == ECOA__SQL__Error_NO_ERROR ){
		if( resultSet->rowCount >= 0 ){
			if( resultSet->rowCount > 0 ){
				strcpy( resultBuff, ((char*)resultSet->resultBuff) );
			}else
				resultBuff[0] = '\000'; // No result to return...
			sprintf( buff, "%d rows in set.\n", resultSet->rowCount );
			strcat( resultBuff, buff );
		}else{
			sprintf( resultBuff, "Query OK.\n" );
		}
	}else{
		ECOA__SQL__String errText;
		// handle errors here...
		MissionSys_modMain_container__getErrorText__request_sync( context, resultSet->status, &errText );
		sprintf( resultBuff, "Query Failed. (Code %d) %s\n", resultSet->status, errText.data );
	}
	if( resultSet )free( resultSet );
	return 0;
}
