/* File NavSys_modMain.c */
/*
 ##############################################################
 # (C) 2015 BAE SYSTEMS
 #
 # ECOA Data Servers Sample - Example Data Server Client
 #
 # $Id: NavSys_modMain.c,v 2.4 2017/11/06 15:33:32 ksappleb Exp $
 #
 # Part of the BAE Systems, Rochester, ECOA Samples collection
 #
 # Created by K S Appleby, BAE Systems, Airport Works, Rochester, Kent
 #
 # This software is developed for and on behalf of BAE Systems (Operations)
 # Limited, Dassault Aviation, Bull SAS, Thales Systemes Aeroportes,
 # GE Aviation Systems Limited,
 # General Dynamics United Kingdom Limited and Leonardo MW Ltd,
 # and the copyright is owned by BAE Systems (Operations) Limited,
 # Dassault Aviation, Bull SAS, Thales Systemes Aeroportes, 
 # GE Aviation Systems Limited, General Dynamics United Kingdom
 # Limited and Leonardo MW Ltd
 #
 # This software is developed by BAE Systems (Operations)
 # Limited, Electronic Systems, and is the Intellectual
 # Property of BAE Systems (Operations) Limited, Electronic Systems.
 #
 # The information set out in this document is provided solely
 # on an 'as is' basis and the co-developers of this software make
 # no warranties expressed or implied, including no warranties
 # as to completeness, accuracy or fitness for purpose, with respect
 # to any of the information.
 ##############################################################
*/
#include "NavSys_modMain.h"
#include <stdio.h>
#include <string.h>
#include <libxml/parser.h>
#include <libxml/tree.h>
#include <libxml/xpath.h>
#include <sktlib/inetAddress.h>
#include <sktlib/datagramPacket.h>
#include <sktlib/datagramSocket.h>
#include <time.h>
#include <math.h>
#ifndef M_PI
#  define M_PI 3.14159265358979
#endif

//*************************************************************************************

// Mean Earth circumference based on mean radius of 6371km (WGS84 ellipsoid)
#define METER_PER_RAD (40030174.0/(2.0*M_PI))

#define toDegrees(x) ((x)*180.0/M_PI)
#define toRadians(x) ((x)*M_PI/180.0)

//*************************************************************************************
void NavSys_modMain__INITIALIZE__received(NavSys_modMain__context *context)
{
	context->user.wxConnection = 0;
	context->user.curTile = 0;
	context->user.wxLatitude = context->user.wxLongitude = 0.0;
	xmlInitParser();
}

void NavSys_modMain__START__received(NavSys_modMain__context *context)
{
   /* User Code Here */
}

void NavSys_modMain__STOP__received(NavSys_modMain__context *context)
{
   /* User Code Here */
}

void NavSys_modMain__SHUTDOWN__received(NavSys_modMain__context *context)
{
   /* User Code Here */
}

//*************************************************************************************
static void dbConnect( NavSys_modMain__context *context )
{
	ECOA__SQL__String dataSource;
	ECOA__SQL__NameString userName, password;
	ECOA__SQL__StringZ dataSourceZ, userNameZ, passwordZ;
	int l;
	//
    NavSys_modMain_container__get_CIMSDB_value( context, &dataSourceZ );
    NavSys_modMain_container__get_UserName_value( context, &userNameZ );
    NavSys_modMain_container__get_Password_value( context, &passwordZ );
	//
   	l = dataSource.current_size = strlen( dataSourceZ ); strncpy( dataSource.data, dataSourceZ, l );
   	l = userName.current_size = strlen( userNameZ );     strncpy( userName.data, userNameZ, l );
   	l = password.current_size = strlen( passwordZ );     strncpy( password.data, passwordZ,  l );
	//
	NavSys_modMain_container__getConnection__request_async( context, &(context->user.connectionRequestID), &dataSource, &userName, &password );
}

//*************************************************************************************
static char* sqlTime( const struct tm *time )
{
	static char buff[64];
	//
	sprintf( buff, "#%04d-%02d-%02d %02d:%02d:%02d#", 1900+time->tm_year, 1+time->tm_mon, time->tm_mday, time->tm_hour, time->tm_min, time->tm_sec );
	return buff;
}

//*************************************************************************************
void sendToHMIDisplay( NavSys_modMain__context* context, ECOA__log msg )
{
	DatagramPacket pkt = datagramPacketCreate();
	char buf[2048];
	//
	memset( buf, 0, 2048 );
	sprintf( buf, "=*INFO:" );
	strncpy( &(buf[7]), msg.data, msg.current_size );
	strcat( buf, "\n" );
	pkt->buffer = (unsigned char*)buf;
	pkt->bufferSizeBytes = strlen( buf );
	pkt->address->value = context->user.dsplAddr->value;
	pkt->port    = 9008;
	//
	datagramSocketSend( context->user.dsplSock, pkt );
	//
	datagramPacketDestroy( pkt );
}

//*************************************************************************************
void NavSys_modMain__getConnection__response_received
   (NavSys_modMain__context* context,    const ECOA__uint32 ID, const ECOA__return_status status,
    const ECOA__SQL__ConnectionId connection,
    const ECOA__SQL__Error requestStatus)
{
	ECOA__log msg;
	//
	if( requestStatus != ECOA__SQL__Error_NO_ERROR ){
		msg.current_size = sprintf( msg.data, "NavSys: connecting to datasource returned code %d", requestStatus );
		NavSys_modMain_container__raise_error( context, msg );
		context->user.wxConnection = 0;
	}else{
		context->user.wxConnection = connection;
//		printf( "NavSys_modMain__getConnection__response_received connectionID = 0x%08x\n", connection );
	}
}

//*************************************************************************************
void NavSys_modMain__tableUpdateNotification__received(NavSys_modMain__context *context,
    const ECOA__SQL__NameString *tableName)
{
   /* User Code Here */
}

//*************************************************************************************
void NavSys_modMain__getWeather__response_received
   (NavSys_modMain__context* context,
    const ECOA__uint32 ID, const ECOA__return_status status,
    const Weather__WeatherRecd* weather)
{
   /* User Code Here */
}

//*************************************************************************************
void NavSys_modMain__getForecastWeather__response_received
   (NavSys_modMain__context* context,
    const ECOA__uint32 ID, const ECOA__return_status status,
    const Weather__WeatherRecd* weather)
{
   /* User Code Here */
}

//*************************************************************************************
void NavSys_modMain__AVSUpdate__received(NavSys_modMain__context *context,
    const AVS__AVS *avs)
{
	ECOA__return_status errc;
	ECOA__log msg;
	ECOA__FileIO__FileName Name;
	ECOA__FileIO__ModeFlag    Flags       = ECOA__FileIO__ModeFlag_READ;
	ECOA__FileIO__Permissions Permissions = (ECOA__FileIO__Permissions){ECOA__FileIO__Permission_ALL,ECOA__FileIO__Permission_ALL,ECOA__FileIO__Permission_ALL};
	ECOA__int32 res;
	ECOA__int32 tx, ty, tile;
	ECOA__FileIO__string readBuff;
	ECOA__SQL__String SQLString;
	ECOA__SQL__ResultSetId transactionId;
	NavSys_modMain_container__weather_handle wxh;
	Weather__WeatherRecd wx;
	xmlDocPtr  pDoc = NULL;
	xmlXPathContextPtr xpathCtx;
	xmlXPathObjectPtr xpathObj;
	xmlChar* p;
	//
	msg.current_size = sprintf( msg.data, "NavSys_modMain__AVSUpdate__received" );
//	NavSys_modMain_container__log_trace( context, msg );
	//
	if(( pDoc = xmlParseMemory( (char*)avs->data, avs->current_size )) == NULL ){
		msg.current_size = sprintf( msg.data, "Invalid AVS %s", avs->data );
		NavSys_modMain_container__log_warning( context, msg );
		return;
	}
	if(( xpathCtx = xmlXPathNewContext(pDoc)) == NULL ){
		msg.current_size = sprintf( msg.data, "Invalid AVS content" );
		NavSys_modMain_container__log_warning( context, msg );
        xmlFreeDoc(pDoc);
		return;
	}
	if(( xpathObj = xmlXPathEval((xmlChar*)"/AVS/Latitude", xpathCtx)) == NULL ){
		msg.current_size = sprintf( msg.data, "Invalid AVS content: missing Latitude" );
		NavSys_modMain_container__log_warning( context, msg );
        xmlXPathFreeContext(xpathCtx);
        xmlFreeDoc(pDoc);
		return;
	}else{
		context->user.Latitude = atof( (char*)(p = xmlNodeGetContent( xpathObj->nodesetval->nodeTab[0] )));
		xmlFree(p);
		xmlXPathFreeObject( xpathObj );
	}
	if(( xpathObj = xmlXPathEval((xmlChar*)"/AVS/Longitude", xpathCtx)) == NULL ){
		msg.current_size = sprintf( msg.data, "Invalid AVS content: missing Longitude" );
		NavSys_modMain_container__log_warning( context, msg );
        xmlXPathFreeContext(xpathCtx);
        xmlFreeDoc(pDoc);
		return;
	}else{
		context->user.Longitude = atof( (char*)(p = xmlNodeGetContent( xpathObj->nodesetval->nodeTab[0] )));
		xmlFree(p);
		xmlXPathFreeObject( xpathObj );
	}
	if(( xpathObj = xmlXPathEval((xmlChar*)"/AVS/Height", xpathCtx)) == NULL ){
		msg.current_size = sprintf( msg.data, "Invalid AVS content: missing Height" );
		NavSys_modMain_container__log_warning( context, msg );
        xmlXPathFreeContext(xpathCtx);
        xmlFreeDoc(pDoc);
		return;
	}else{
		context->user.Height = atof( (char*)(p = xmlNodeGetContent( xpathObj->nodesetval->nodeTab[0] )));
		xmlFree(p);
		xmlXPathFreeObject( xpathObj );
	}
	if(( xpathObj = xmlXPathEval((xmlChar*)"/AVS/Speed", xpathCtx)) == NULL ){
		msg.current_size = sprintf( msg.data, "Invalid AVS content: missing Speed" );
		NavSys_modMain_container__log_warning( context, msg );
        xmlXPathFreeContext(xpathCtx);
        xmlFreeDoc(pDoc);
		return;
	}else{
		context->user.Speed = atof( (char*)(p = xmlNodeGetContent( xpathObj->nodesetval->nodeTab[0] )));
		xmlFree(p);
		xmlXPathFreeObject( xpathObj );
	}
	if(( xpathObj = xmlXPathEval((xmlChar*)"/AVS/Heading", xpathCtx)) == NULL ){
		msg.current_size = sprintf( msg.data, "Invalid AVS content: missing Heading" );
		NavSys_modMain_container__log_warning( context, msg );
        xmlXPathFreeContext(xpathCtx);
        xmlFreeDoc(pDoc);
		return;
	}else{
		context->user.Heading = atof( (char*)(p = xmlNodeGetContent( xpathObj->nodesetval->nodeTab[0] )));
		xmlFree(p);
		xmlXPathFreeObject( xpathObj );
	}
    xmlXPathFreeContext(xpathCtx);
    xmlFreeDoc(pDoc);
	//
#if 0
	msg.current_size = sprintf( msg.data, "AVS = %9.5f %9.5f %4.0f %4.0f %4.0f",
			context->user.Latitude, context->user.Longitude,
			context->user.Height,
			context->user.Speed,
			context->user.Heading );
	NavSys_modMain_container__log_info( context, msg );
#endif
	//
	//*************************************************************************************
	// Update the Weather Store (Scenario 4.3)
	//*************************************************************************************
	{
		Weather__wxLocationString location;
		ECOA__double64 dy = toRadians( context->user.Latitude - context->user.wxLatitude ) * METER_PER_RAD,
					   dx = toRadians( context->user.Longitude - context->user.wxLongitude ) * METER_PER_RAD * cos( toRadians( context->user.Latitude )),
					   delta = sqrt( dx * dx + dy * dy );
		//
		// To reduce network traffic, we only want to get the weather (by setting the location) if we've moved significantly from
		// where we were when we last asked...
		if( delta >= 5000.0 ){
			context->user.wxLatitude  = context->user.Latitude;
			context->user.wxLongitude = context->user.Longitude;
			sprintf( (char*)location, "%g,%g", context->user.wxLatitude, context->user.wxLongitude );
			NavSys_modMain_container__setLocation__send( context, &location );
			//
			// Now read the current weather back from the (metServer) webportal
			errc = NavSys_modMain_container__weather__get_read_access( context, &wxh );
			if( errc != ECOA__return_status_OK ){
				msg.current_size = sprintf( msg.data, "NavSys_modMain_container__weather__get_read_access returned %d", errc );
				NavSys_modMain_container__log_info( context, msg );
			}else{
				time_t wx_Timestamp_seconds;
				//
				wx = (*wxh.data);
				errc = NavSys_modMain_container__weather__release_read_access( context, &wxh );
				wx_Timestamp_seconds = wx.Timestamp.seconds; // need to convert unsigned -> signed
				if( context->user.wxConnection > 0 ){
					struct tm *lclTS;
					SQLString.current_size = sprintf( SQLString.data, "insert into Weather values( %s, '%s', %g, %.1f, '%s', %g, '%s' );",
														sqlTime( lclTS = localtime( &(wx_Timestamp_seconds) )),
														wx.Location, wx.TemperatureC, wx.WindSpeedMPH, wx.WindDir, wx.PressureMB, wx.Synopsis );
//					printf( "*** NavSys storing weather with (%d chars) '%s'\n", SQLString.current_size, SQLString.data );
					errc = NavSys_modMain_container__SQLExecute__request_sync( context, context->user.wxConnection, &SQLString, &transactionId );
						if( errc != ECOA__return_status_OK ){
							msg.current_size = sprintf( msg.data, "NavSys: storing local weather returned code %d", errc );
							NavSys_modMain_container__log_warning( context, msg );
						}
					msg.current_size = sprintf( msg.data, "Wx update at %02d:%02d:  %g C; %.1f MPH %s; %g MB; %s",
															lclTS->tm_hour, lclTS->tm_min, wx.TemperatureC, wx.WindSpeedMPH, wx.WindDir, wx.PressureMB, wx.Synopsis );
					NavSys_modMain_container__log_info( context, msg );
					sendToHMIDisplay( context, msg );
				}
			}
		}
	}
	//
	//*************************************************************************************
	// Update the console display so we can see what's going on...
	//*************************************************************************************
	{
		char buff[1024];
		int  bytesSent;
		DatagramPacket pkt = datagramPacketCreate();
		//
		if( context->user.mapSock == NULL ){
			if(( context->user.mapSock = datagramSocketCreate( 0, /*localhost*/NULL )) == NULL ){
				perror( "mapSock" );
			}
		}
		if( context->user.mapSock ){
			//
			pkt->address->value = inetAddressGetLocalHost()->value;
			pkt->port    = 9008;
			//
			sprintf( buff, "=*AVS:%.9lg %.9lg %lg %lg %lg",
					  context->user.Latitude, context->user.Longitude, context->user.Heading,
					  context->user.Height, context->user.Speed );
			pkt->buffer = (unsigned char *)buff;
			pkt->bufferSizeBytes = strlen( buff )+1;
			bytesSent = datagramSocketSend( context->user.mapSock, pkt );
		}
		datagramPacketDestroy( pkt );
	}
    //
	//*************************************************************************************
	// Digital Map Tiles (Scenario 4.5)
	//*************************************************************************************
	// Which map tile do we need? (Scenario 4.5)
	tx = (int)(( context->user.Longitude +  2.01 ) * 100.0 ); // Map (0,0) reference point is arbitrary, and dependent on
	ty = (int)(( context->user.Latitude  - 53.395 ) * 100.0 ); // the aircraft model...
	sprintf( Name, "mapData/tile_%03d%03d.dat", tx, ty );
	tile = tx*1000 + ty;
	//
	// Load a new tile if necessary...
	if( tile != context->user.curTile ){
		//
		msg.current_size = sprintf( msg.data, "NavSys: Loading tile_%03d%03d.dat", tx, ty );
		NavSys_modMain_container__log_info( context, msg );
		//
		errc = NavSys_modMain_container__open__request_sync( context, &Name, Flags, &Permissions, &(context->user.fHndl) );
			if( errc != ECOA__return_status_OK ){
				msg.current_size = sprintf( msg.data, "NavSys: opening tile '%s' returned code %d", Name, errc );
				NavSys_modMain_container__log_warning( context, msg );
			}
		if( context->user.fHndl > 0 ){
			errc = NavSys_modMain_container__read__request_sync( context, context->user.fHndl, ECOA__FileIO__string_MAXSIZE, &readBuff, &res );
				if( errc != ECOA__return_status_OK ){
					msg.current_size = sprintf( msg.data, "NavSys: reading tile '%s' returned code %d", Name, errc );
					NavSys_modMain_container__log_warning( context, msg );
				}
			//******************************************************************************
			// Here would go actually doing something with the data read in...
			//******************************************************************************
//			printf( "  read %d bytes\n", res );
			if( res > 0 ) context->user.curTile = tile;
			errc = NavSys_modMain_container__close__request_sync( context, context->user.fHndl, &res );
				if( errc != ECOA__return_status_OK ){
					msg.current_size = sprintf( msg.data, "NavSys: closing tile '%s' returned code %d", Name, errc );
					NavSys_modMain_container__log_warning( context, msg );
				}
		}else{
			msg.current_size = sprintf( msg.data, "NavSys: tile '%s' not found", Name );
			NavSys_modMain_container__log_warning( context, msg );
			context->user.curTile = tile; // prevent multiple shouts...
		}
	}
	//*************************************************************************************
	// Whilst we demonstrate using a file server for "loading map tiles", we have available
	// a complete digital map app external to the demo.
	//
	// We'll use it for display purposes...
	//*************************************************************************************
	{
		char buff[1024];
		int  bytesSent;
		DatagramPacket pkt = datagramPacketCreate();
		//
		if( context->user.mapSock == NULL ){
			if(( context->user.mapSock = datagramSocketCreate( 0, /*localhost*/NULL )) == NULL ){
				perror( "mapSock" );
			}
		}
		if( context->user.mapSock ){
			//
			pkt->address->value = inetAddressGetLocalHost()->value;
			pkt->port    = 9001;
			//
			sprintf( buff, "=*AVS:%.9lg %.9lg %lg %lg %lg",
					  context->user.Latitude, context->user.Longitude, context->user.Heading,
					  context->user.Height, context->user.Speed );
			pkt->buffer = (unsigned char *)buff;
			pkt->bufferSizeBytes = strlen( buff )+1;
			bytesSent = datagramSocketSend( context->user.mapSock, pkt );
		}
		datagramPacketDestroy( pkt );
	}
}

//*************************************************************************************
void NavSys_modMain__Tick__received(NavSys_modMain__context *context)
{
	ECOA__log msg;
	//
	// If the HMI (display) socket is not open, do so...
	if( context->user.dsplSock == NULL ){
		context->user.dsplSock = datagramSocketCreate( 0, NULL );
		context->user.dsplAddr = inetAddressGetLocalHost();
	}
	msg.current_size = sprintf( msg.data, "NavSys_modMain__Tick__received" );
//	NavSys_modMain_container__log_trace( context, msg );
	//
	fflush(stdout); // Make sure that any piped IO is completed
	//
	// If not connected to the weather database, do so...
	if( context->user.wxConnection < 1 ){
		dbConnect( context );
	}
}
