/* File dbmServer_modMain.c */
/*
 ##############################################################
 # (C) 2015 BAE SYSTEMS
 #
 # ECOA Data Servers Sample - Rapid Access Database Server
 #
 # $Id: dbmServer_modMain.c,v 2.3 2017/11/06 15:33:32 ksappleb Exp $
 #
 # Part of the BAE Systems, Rochester, ECOA Samples collection
 #
 # Created by K S Appleby, BAE Systems, Airport Works, Rochester, Kent
 #
 # This software is developed for and on behalf of BAE Systems (Operations)
 # Limited, Dassault Aviation, Bull SAS, Thales Systemes Aeroportes,
 # GE Aviation Systems Limited,
 # General Dynamics United Kingdom Limited and Leonardo MW Ltd,
 # and the copyright is owned by BAE Systems (Operations) Limited,
 # Dassault Aviation, Bull SAS, Thales Systemes Aeroportes, 
 # GE Aviation Systems Limited, General Dynamics United Kingdom
 # Limited and Leonardo MW Ltd
 #
 # This software is developed by BAE Systems (Operations)
 # Limited, Electronic Systems, and is the Intellectual
 # Property of BAE Systems (Operations) Limited, Electronic Systems.
 #
 # The information set out in this document is provided solely
 # on an 'as is' basis and the co-developers of this software make
 # no warranties expressed or implied, including no warranties
 # as to completeness, accuracy or fitness for purpose, with respect
 # to any of the information.
 ##############################################################
*/
#include "dbmServer_modMain.h"
#include <string.h>
#include <gdbm.h>

void dbmServer_modMain__INITIALIZE__received(dbmServer_modMain__context *context)
{
	int i;
	//
	for( i = 0; i < DBMSERVER_MAXHANDLES; i++ )
		context->user.dbFs[i] = NULL;
}

void dbmServer_modMain__START__received(dbmServer_modMain__context *context)
{
   /* User Code Here */
}

void dbmServer_modMain__STOP__received(dbmServer_modMain__context *context)
{
   /* User Code Here */
}

void dbmServer_modMain__SHUTDOWN__received(dbmServer_modMain__context *context)
{
   /* User Code Here */
}


void dbmServer_modMain__open__request_received
   (dbmServer_modMain__context* context,
    const ECOA__uint32 ID,
    const ECOA__DBM__String* database,
    const ECOA__int32 size,
    const ECOA__DBM__OpenFlags flags,
    const ECOA__int32 mode)
{
	ECOA__DBM__ConnectionId dbHndl;
	ECOA__log msg;
	ECOA__return_status errc;
	ECOA__int32 dbmStatus = 0;
	char nameBuff[database->current_size+1];
	//
	strncpy( nameBuff, database->data, database->current_size ); nameBuff[database->current_size] = '\00';
	//
	for( dbHndl = 1; dbHndl < DBMSERVER_MAXHANDLES; dbHndl++ ){
		if( context->user.dbFs[dbHndl] == NULL ){
			msg.current_size = sprintf( msg.data, "dbmServer: opening '%s' as %d", nameBuff, dbHndl );
//			dbmServer_modMain_container__log_trace( context, msg );
			context->user.dbFs[dbHndl] = gdbm_open( nameBuff, (int)size, (int)flags, (int)mode, NULL );
			if( context->user.dbFs[dbHndl] == NULL ){
				msg.current_size = sprintf( msg.data, "dbmServer: open '%s' failed with %d (%s)", nameBuff, gdbm_errno, gdbm_errlist[gdbm_errno] );
//				dbmServer_modMain_container__log_warning( context, msg );
				dbmStatus = gdbm_errno;
				dbHndl = 0;
			}
			break;
		}
	}
	if( dbHndl >= DBMSERVER_MAXHANDLES ){
		msg.current_size = sprintf( msg.data, "dbmServer: no more handles for '%s'", nameBuff );
		dbmServer_modMain_container__log_warning( context, msg );
		dbHndl = 0; dbmStatus = -1;
		// We'll set this, but it's meaningless as it doesn't propagate...
		errc = ECOA__return_status_RESOURCE_NOT_AVAILABLE;
	}
	//
	if(( errc = dbmServer_modMain_container__open__response_send( context, ID, dbHndl, dbmStatus )) != ECOA__return_status_OK ){
		msg.current_size = sprintf( msg.data, "dbmServer_modMain_container__open__response_send returned %d", errc );
		dbmServer_modMain_container__log_warning( context, msg );
	}
	return;
}

void dbmServer_modMain__close__request_received
   (dbmServer_modMain__context* context,
    const ECOA__uint32 ID,
    const ECOA__DBM__ConnectionId connection)
{
	ECOA__log           msg;
	ECOA__return_status errc;
	//
	gdbm_close( context->user.dbFs[connection] );
	context->user.dbFs[connection] = NULL;
	//
	if(( errc = dbmServer_modMain_container__close__response_send( context, ID )) != ECOA__return_status_OK ){
		msg.current_size = sprintf( msg.data, "dbmServer_modMain_container__close__response_send returned %d", errc );
		dbmServer_modMain_container__log_warning( context, msg );
	}
//	printf( "dbmServer_modMain__close__request_received( %d ) --> %d\n", connection, res );
}

void dbmServer_modMain__store__request_received
   (dbmServer_modMain__context* context,
    const ECOA__uint32 ID,
    const ECOA__DBM__ConnectionId connection,
    const ECOA__DBM__Datum* key,
    const ECOA__DBM__Datum* content,
    const ECOA__DBM__StoreFlag flag)
{
	ECOA__int32 res = 0;
	ECOA__return_status errc;
	datum gkey, gcontent;
	//
	gkey.dsize = (int)key->current_size;
	gkey.dptr  = (char*)&(key->data);
	//
	gcontent.dsize = (int)content->current_size;
	gcontent.dptr  = (char*)&(content->data);
	//
	res = gdbm_store( context->user.dbFs[ connection ], gkey, gcontent, flag );
	//
	errc = dbmServer_modMain_container__store__response_send( context, ID, res );
}

void dbmServer_modMain__fetch__request_received
   (dbmServer_modMain__context* context,
    const ECOA__uint32 ID,
    const ECOA__DBM__ConnectionId connection,
    const ECOA__DBM__Datum* key)
{
	ECOA__int32 res = 0;
	ECOA__return_status errc;
	datum gkey, gcontent;
	ECOA__DBM__Datum content;
	//
	gkey.dsize = (int)key->current_size;
	gkey.dptr  = (char*)&(key->data);
	//
	gcontent = gdbm_fetch( context->user.dbFs[ connection ], gkey );
	//
	if( gcontent.dptr != NULL ){
		content.current_size = gcontent.dsize;
		memcpy( content.data, gcontent.dptr, gcontent.dsize );
		free( gcontent.dptr );
	}else{
		content.current_size = 0;
		content.data[0] = '\000';
	}
	errc = dbmServer_modMain_container__fetch__response_send( context, ID, &content, res );
}

void dbmServer_modMain__exists__request_received
   (dbmServer_modMain__context* context,
    const ECOA__uint32 ID,
    const ECOA__DBM__ConnectionId connection,
    const ECOA__DBM__Datum* key)
{
	ECOA__int32 res = 0;
	ECOA__return_status errc;
	datum gkey;
	//
	gkey.dsize = (int)key->current_size;
	gkey.dptr  = (char*)&(key->data);
	//
	res = gdbm_exists( context->user.dbFs[ connection ], gkey );
	//
	errc = dbmServer_modMain_container__exists__response_send( context, ID, (res == 0 ? ECOA__FALSE : ECOA__TRUE ));
}

void dbmServer_modMain__delete__request_received
   (dbmServer_modMain__context* context,
    const ECOA__uint32 ID,
    const ECOA__DBM__ConnectionId connection,
    const ECOA__DBM__Datum* key)
{
	ECOA__int32 res = 0;
	ECOA__return_status errc;
	datum gkey;
	//
	gkey.dsize = (int)key->current_size;
	gkey.dptr  = (char*)&(key->data);
	//
	res = gdbm_delete( context->user.dbFs[ connection ], gkey );
	//
	errc = dbmServer_modMain_container__delete__response_send( context, ID, res );
}

void dbmServer_modMain__firstkey__request_received
   (dbmServer_modMain__context* context,
    const ECOA__uint32 ID,
    const ECOA__DBM__ConnectionId connection)
{
   /* User Code Here */
}

void dbmServer_modMain__nexttkey__request_received
   (dbmServer_modMain__context* context,
    const ECOA__uint32 ID,
    const ECOA__DBM__ConnectionId connection,
    const ECOA__DBM__Datum* key)
{
   /* User Code Here */
}

void dbmServer_modMain__reorganize__request_received
   (dbmServer_modMain__context* context,
    const ECOA__uint32 ID,
    const ECOA__DBM__ConnectionId connection)
{
   /* User Code Here */
}

void dbmServer_modMain__sync__request_received
   (dbmServer_modMain__context* context,
    const ECOA__uint32 ID,
    const ECOA__DBM__ConnectionId connection)
{
	ECOA__int32 res = 0;
	ECOA__return_status errc;
	//
	gdbm_sync( context->user.dbFs[ connection ] );
	//
	errc = dbmServer_modMain_container__sync__response_send( context, ID, res );
}

void dbmServer_modMain__lasterror__request_received
   (dbmServer_modMain__context* context,
    const ECOA__uint32 ID)
{
   /* User Code Here */
}

void dbmServer_modMain__setopt__request_received
   (dbmServer_modMain__context* context,
    const ECOA__uint32 ID,
    const ECOA__DBM__ConnectionId connection,
    const ECOA__DBM__Option option,
    const ECOA__int32 value)
{
   /* User Code Here */
}

void dbmServer_modMain__dump__request_received
   (dbmServer_modMain__context* context,
    const ECOA__uint32 ID,
    const ECOA__DBM__ConnectionId connection,
    const ECOA__DBM__String* fileName)
{
   /* User Code Here */
}

void dbmServer_modMain__take__request_received
   (dbmServer_modMain__context* context,
    const ECOA__uint32 ID,
    const ECOA__DBM__ConnectionId connection,
    const ECOA__DBM__String* fileName)
{
   /* User Code Here */
}

void dbmServer_modMain__getHelp__request_received
   (dbmServer_modMain__context* context,
    const ECOA__uint32 ID,
    const ECOA__DBM__ConnectionId connection)
{
   /* User Code Here */
}

void dbmServer_modMain__getLimits__request_received
   (dbmServer_modMain__context* context,
    const ECOA__uint32 ID,
    const ECOA__DBM__ConnectionId connection)
{
   /* User Code Here */
}

void dbmServer_modMain__getCounts__request_received
   (dbmServer_modMain__context* context,
    const ECOA__uint32 ID,
    const ECOA__DBM__ConnectionId connection)
{
   /* User Code Here */
}

