/* File fileServer_modMain.c */
/*
 ##############################################################
 # (C) 2015 BAE SYSTEMS
 #
 # ECOA Data Servers Sample - File Server
 #
 # $Id: fileServer_modMain.c,v 2.3 2017/11/06 15:33:32 ksappleb Exp $
 #
 # Part of the BAE Systems, Rochester, ECOA Samples collection
 #
 # Created by K S Appleby, BAE Systems, Airport Works, Rochester, Kent
 #
 # This software is developed for and on behalf of BAE Systems (Operations)
 # Limited, Dassault Aviation, Bull SAS, Thales Systemes Aeroportes,
 # GE Aviation Systems Limited,
 # General Dynamics United Kingdom Limited and Leonardo MW Ltd,
 # and the copyright is owned by BAE Systems (Operations) Limited,
 # Dassault Aviation, Bull SAS, Thales Systemes Aeroportes, 
 # GE Aviation Systems Limited, General Dynamics United Kingdom
 # Limited and Leonardo MW Ltd
 #
 # This software is developed by BAE Systems (Operations)
 # Limited, Electronic Systems, and is the Intellectual
 # Property of BAE Systems (Operations) Limited, Electronic Systems.
 #
 # The information set out in this document is provided solely
 # on an 'as is' basis and the co-developers of this software make
 # no warranties expressed or implied, including no warranties
 # as to completeness, accuracy or fitness for purpose, with respect
 # to any of the information.
 ##############################################################
*/
#include "fileServer_modMain.h"
#include <string.h>
#include <unistd.h>
#include <math.h>
#include <fcntl.h>
#include <dirent.h>
#include <sys/stat.h>

#ifndef min
#define min(a,b) ((a)<(b)?(a):(b))
#endif

void fileServer_modMain__INITIALIZE__received(fileServer_modMain__context *context)
{
	int i;
	//
	context->user.nextHandle = 0;
	for( i = 0; i < DIR_MAXHANDLES; i++ )
		context->user.dirHandles[i] = NULL;
}

void fileServer_modMain__START__received(fileServer_modMain__context *context)
{
   /* User Code Here */
}

void fileServer_modMain__STOP__received(fileServer_modMain__context *context)
{
   /* User Code Here */
}

void fileServer_modMain__SHUTDOWN__received(fileServer_modMain__context *context)
{
   /* User Code Here */
}


void fileServer_modMain__open__request_received
   (fileServer_modMain__context* context,
    const ECOA__uint32 ID,
    const ECOA__FileIO__FileName* name,
    const ECOA__FileIO__ModeFlag flags,
    const ECOA__FileIO__Permissions* permissions)
{
	ECOA__return_status errc;
	ECOA__int32 handle;
	ECOA__log   msg;
	int   fileFlags = O_RDONLY, fileMode = 0;
	//
	// Sort out file flags.
	// We CANNOT assume that our numbering matches the toolchain/operating system...
	if( flags & ECOA__FileIO__ModeFlag_WRITE )     fileFlags |= O_WRONLY;
	if( flags & ECOA__FileIO__ModeFlag_READWRITE ) fileFlags |= O_RDWR;
	if( flags & ECOA__FileIO__ModeFlag_APPEND )    fileFlags |= O_APPEND;
	if( flags & ECOA__FileIO__ModeFlag_NONBLOCK )  fileFlags |= O_NONBLOCK;
	if( flags & ECOA__FileIO__ModeFlag_CREATE )    fileFlags |= O_CREAT;
	if( flags & ECOA__FileIO__ModeFlag_TRUNCATE )  fileFlags |= O_TRUNC;
	if( flags & ECOA__FileIO__ModeFlag_EXCLUSIVE ) fileFlags |= O_EXCL;
	if( flags & ECOA__FileIO__ModeFlag_NOCTTY )    fileFlags |= O_NOCTTY;
#ifdef O_PRIV
	if( flags & ECOA__FileIO__ModeFlag_PRIVATE )   fileFlags |= O_PRIV;
#endif
#ifdef O_LARGEFILE
	if( flags & ECOA__FileIO__ModeFlag_LARGEFILE ) fileFlags |= O_LARGEFILE;
#endif
#ifdef O_BINARY
	if( flags & ECOA__FileIO__ModeFlag_BINARY )    fileFlags |= O_BINARY;
#endif
	//
	// Sort out mode (permission) bits.
	// We CANNOT assume that our numbering matches the toolchain/operating system...
#ifdef S_IROTH
	if( permissions->Others & ECOA__FileIO__Permission_READ )    fileMode |= S_IROTH;
	if( permissions->Others & ECOA__FileIO__Permission_WRITE )   fileMode |= S_IWOTH;
	if( permissions->Others & ECOA__FileIO__Permission_EXECUTE ) fileMode |= S_IXOTH;
	if( permissions->Group  & ECOA__FileIO__Permission_READ )    fileMode |= S_IRGRP;
	if( permissions->Group  & ECOA__FileIO__Permission_WRITE )   fileMode |= S_IWGRP;
	if( permissions->Group  & ECOA__FileIO__Permission_EXECUTE ) fileMode |= S_IXGRP;
	if( permissions->Owner  & ECOA__FileIO__Permission_READ )    fileMode |= S_IRUSR;
	if( permissions->Owner  & ECOA__FileIO__Permission_WRITE )   fileMode |= S_IWUSR;
	if( permissions->Owner  & ECOA__FileIO__Permission_EXECUTE ) fileMode |= S_IXUSR;
#else
	if( permissions->Others & ECOA__FileIO__Permission_READ )    fileMode |= 0004;
	if( permissions->Others & ECOA__FileIO__Permission_WRITE )   fileMode |= 0002;
	if( permissions->Others & ECOA__FileIO__Permission_EXECUTE ) fileMode |= 0001;
	if( permissions->Group  & ECOA__FileIO__Permission_READ )    fileMode |= 0040;
	if( permissions->Group  & ECOA__FileIO__Permission_WRITE )   fileMode |= 0020;
	if( permissions->Group  & ECOA__FileIO__Permission_EXECUTE ) fileMode |= 0010;
	if( permissions->Owner  & ECOA__FileIO__Permission_READ )    fileMode |= 0400;
	if( permissions->Owner  & ECOA__FileIO__Permission_WRITE )   fileMode |= 0200;
	if( permissions->Owner  & ECOA__FileIO__Permission_EXECUTE ) fileMode |= 0100;
#endif
//	printf( "fileServer_modMain__open__request_received( %s, 0%o, 0%o )", *name, fileFlags, fileMode );
	handle = (ECOA__int32)open( *name, fileFlags, fileMode );
//	printf( " --> handle %d\n", handle );
	if(( errc = fileServer_modMain_container__open__response_send( context, ID, handle )) != ECOA__return_status_OK ){
		msg.current_size = sprintf( msg.data, "fileServer_modMain_container__open__response_send returned %d", errc );
		fileServer_modMain_container__log_warning( context, msg );
	}
	return;
}

void fileServer_modMain__close__request_received
   (fileServer_modMain__context* context,
    const ECOA__uint32 ID,
    const ECOA__int32 handle)
{
	ECOA__log           msg;
	ECOA__return_status errc;
	ECOA__int32         res  = (ECOA__int32)close( handle );
	if(( errc = fileServer_modMain_container__close__response_send( context, ID, res )) != ECOA__return_status_OK ){
		msg.current_size = sprintf( msg.data, "fileServer_modMain_container__close__response_send returned %d", errc );
		fileServer_modMain_container__log_warning( context, msg );
	}
//	printf( "fileServer_modMain__close__request_received( %d ) --> %d\n", Handle, res );
}

void fileServer_modMain__read__request_received
   (fileServer_modMain__context* context,
    const ECOA__uint32 ID,
    const ECOA__int32 handle,
    const ECOA__int32 maxCount)
{
	ECOA__FileIO__string buffer;
	ECOA__int32 actualCount;
	ECOA__return_status errc;
	ECOA__log           msg;
	//
//	printf( "fileServer_modMain__read__request_received( %d, %d )\n", Handle, MaxCount );
	buffer.current_size = actualCount = read( handle, (void*)&buffer.data, min((size_t)maxCount, ECOA__FileIO__string_MAXSIZE) );
	errc = fileServer_modMain_container__read__response_send( context, ID, &buffer, actualCount);
	if( errc != ECOA__return_status_OK ){
		msg.current_size = sprintf( msg.data, "fileServer_modMain_container__read__response_send returned %d", errc );
		fileServer_modMain_container__log_warning( context, msg );
	}
}

void fileServer_modMain__write__request_received
   (fileServer_modMain__context* context,
    const ECOA__uint32 ID,
    const ECOA__int32 handle,
    const ECOA__FileIO__string* buffer,
    const ECOA__int32 maxCount)
{
	ECOA__int32 ActualCount;
	ECOA__return_status errc;
	ECOA__log           msg;
	//
//	printf( "fileServer_modMain__write__request_received( %d, 0x%x, %d )\n", Handle, Buffer->data, MaxCount );
	ActualCount = write( handle, (void*)&buffer->data, (size_t)maxCount );
	errc = fileServer_modMain_container__write__response_send( context, ID, ActualCount);
	if( errc != ECOA__return_status_OK ){
		msg.current_size = sprintf( msg.data, "fileServer_modMain_container__write__response_send returned %d", errc );
		fileServer_modMain_container__log_warning( context, msg );
	}
}

/*
 * To get the current position we can use:
 *     cur_posn = seek( fd, 0, SEEK_CUR );
 */
void fileServer_modMain__seek__request_received
   (fileServer_modMain__context* context,
    const ECOA__uint32 ID,
    const ECOA__int32 handle,
    const ECOA__int64 position,
    const ECOA__FileIO__SeekBase base)
{
	int whence;
	ECOA__int64 actualPosn;
	ECOA__return_status errc;
	ECOA__log           msg;
	//
	// Sort out the seek base.
	// We CANNOT assume that our numbering matches the toolchain/operating system...
	if( base == ECOA__FileIO__SeekBase_SEEK_SET ) whence = SEEK_SET;
	if( base == ECOA__FileIO__SeekBase_SEEK_CUR ) whence = SEEK_CUR;
	if( base == ECOA__FileIO__SeekBase_SEEK_END ) whence = SEEK_END;
	//
	actualPosn = (ECOA__int64)lseek( handle, (off_t)position, whence );
	errc = fileServer_modMain_container__seek__response_send( context, ID, actualPosn );
	if( errc != ECOA__return_status_OK ){
		msg.current_size = sprintf( msg.data, "fileServer_modMain_container__seek__response_send returned %d", errc );
		fileServer_modMain_container__log_warning( context, msg );
	}
}

void fileServer_modMain__delete__request_received
   (fileServer_modMain__context* context,
    const ECOA__uint32 ID,
    const ECOA__FileIO__FileName* name)
{
	ECOA__return_status errc;
	ECOA__log           msg;
	ECOA__int32         res;
	//
	res = unlink( (char*)name );
	errc = fileServer_modMain_container__delete__response_send( context, ID, res );
	if( errc != ECOA__return_status_OK ){
		msg.current_size = sprintf( msg.data, "fileServer_modMain_container__delete__response_send returned %d", errc );
		fileServer_modMain_container__log_warning( context, msg );
	}
}

void fileServer_modMain__opendir__request_received
   (fileServer_modMain__context* context,
    const ECOA__uint32 ID,
    const ECOA__FileIO__FileName* name)
{
	ECOA__return_status errc;
	ECOA__int32         handle;
	ECOA__log           msg;
	DIR                *d = NULL;
	//
	if(( d = opendir( (char*)name )) != NULL ){
		if( context->user.nextHandle < DIR_MAXHANDLES ){
			handle = ++context->user.nextHandle;
		}else{
			// Do we have any closed handles that we can reuse?
			for( handle = 1; handle < DIR_MAXHANDLES; handle++ ){
				if( context->user.dirHandles[handle] == NULL )
					break;
			}
			// If not, then there's nothing else we can do...
			if( handle == DIR_MAXHANDLES )
				handle = 0;
		}
	}
	if( handle == 0 ){
		msg.current_size = sprintf( msg.data, "fileServer_modMain__opendir__request_received failed : %d", ECOA__return_status_RESOURCE_NOT_AVAILABLE );
		fileServer_modMain_container__log_warning( context, msg );
	}
	errc = fileServer_modMain_container__opendir__response_send( context, ID, handle );
	if( errc != ECOA__return_status_OK ){
		msg.current_size = sprintf( msg.data, "fileServer_modMain_container__closedir__response_send returned %d", errc );
		fileServer_modMain_container__log_warning( context, msg );
	}
//	printf( "fileServer_modMain__opendir__request_received( %s ) --> %d\n", Name, Handle );
}

void fileServer_modMain__readdir__request_received
   (fileServer_modMain__context* context,
    const ECOA__uint32 ID,
    const ECOA__int32 handle)
{
	ECOA__return_status errc;
	ECOA__log           msg;
	ECOA__FileIO__DirEntry dirEnt;
	struct dirent      *de;
	struct stat         s;
	//
	dirEnt.Name[0] = '\0';
	dirEnt.EntryType = ECOA__FileIO__EntryType_NONE;
	//
	if( handle > 0 && handle < DIR_MAXHANDLES && context->user.dirHandles[handle] != NULL ){
		//
		// Read an entry.
		if(( de = readdir( context->user.dirHandles[handle] )) != NULL ){
			//
			// Save the name
			strcpy( dirEnt.Name, de->d_name );
			//
			// Figure the entry's type...
			stat( de->d_name, &s );
			dirEnt.EntryType = ECOA__FileIO__EntryType_NONE;
			if( S_ISCHR(s.st_mode) )
				dirEnt.EntryType += ECOA__FileIO__EntryType_CHARACTER;
			if( S_ISDIR(s.st_mode) )
				dirEnt.EntryType += ECOA__FileIO__EntryType_DIRECTORY;
			if( S_ISBLK(s.st_mode) )
				dirEnt.EntryType += ECOA__FileIO__EntryType_BLOCK;
			if( S_ISREG(s.st_mode) )
				dirEnt.EntryType += ECOA__FileIO__EntryType_REGULAR;
			if( S_ISLNK(s.st_mode) )
				dirEnt.EntryType += ECOA__FileIO__EntryType_LINK;
			  /*
			   *  OK. That covers the interesting types.
			   *  If we haven't found it yet, it must be something else that we're not interested in (e.g. a FIFO or a SOCKET)...
			   */
			if( dirEnt.EntryType == ECOA__FileIO__EntryType_NONE )
				dirEnt.EntryType = ECOA__FileIO__EntryType_OTHER;
		}else{
			// No more entries to return...
			dirEnt.Name[0] = '\0';
			dirEnt.EntryType = ECOA__FileIO__EntryType_NONE;
		}
		errc = fileServer_modMain_container__readdir__response_send( context, ID, &dirEnt, 0 );
	} else {
		msg.current_size = sprintf( msg.data, "fileServer_modMain__readdir__request_received failed : %d", ECOA__return_status_RESOURCE_NOT_AVAILABLE );
		fileServer_modMain_container__log_warning( context, msg );
		errc = fileServer_modMain_container__readdir__response_send( context, ID, &dirEnt, -1 );
	}
	if( errc != ECOA__return_status_OK ){
		msg.current_size = sprintf( msg.data, "fileServer_modMain_container__readdir__response_send returned %d", errc );
		fileServer_modMain_container__log_warning( context, msg );
	}
	printf( "fileServer_modMain__readdir__request_received( %d ) --> '%s' (type %d)\n", handle, dirEnt.Name, dirEnt.EntryType );
}

void fileServer_modMain__closedir__request_received
   (fileServer_modMain__context* context,
    const ECOA__uint32 ID,
    const ECOA__int32 handle)
{
	ECOA__return_status errc;
	ECOA__int32         res = -1;
	ECOA__log           msg;
	//
	if( handle > 0 && handle < DIR_MAXHANDLES && context->user.dirHandles[handle] != NULL ){
		res = closedir( context->user.dirHandles[handle] );
		errc = fileServer_modMain_container__closedir__response_send( context, ID, res );
		context->user.dirHandles[handle] = NULL;
	} else {
		msg.current_size = sprintf( msg.data, "fileServer_modMain__closedir__request_received failed : %d", ECOA__return_status_RESOURCE_NOT_AVAILABLE );
		fileServer_modMain_container__log_warning( context, msg );
		errc = fileServer_modMain_container__closedir__response_send( context, ID, -1 );
	}
	if( errc != ECOA__return_status_OK ){
		msg.current_size = sprintf( msg.data, "fileServer_modMain_container__closedir__response_send returned %d", errc );
		fileServer_modMain_container__log_warning( context, msg );
	}
//	printf( "fileServer_modMain__closedir__request_received( %d ) --> %d\n", Handle, res );
}

