/* File sqlServer_modMain.c */
/*
 ##############################################################
 # (C) 2015 BAE SYSTEMS
 #
 # ECOA Data Servers Sample - SQL Database Server
 #
 # $Id: sqlServer_modMain.c,v 2.3 2017/11/06 15:33:31 ksappleb Exp $
 #
 # Part of the BAE Systems, Rochester, ECOA Samples collection
 #
 # Created by K S Appleby, BAE Systems, Airport Works, Rochester, Kent
 #
 # This software is developed for and on behalf of BAE Systems (Operations)
 # Limited, Dassault Aviation, Bull SAS, Thales Systemes Aeroportes,
 # GE Aviation Systems Limited,
 # General Dynamics United Kingdom Limited and Leonardo MW Ltd,
 # and the copyright is owned by BAE Systems (Operations) Limited,
 # Dassault Aviation, Bull SAS, Thales Systemes Aeroportes, 
 # GE Aviation Systems Limited, General Dynamics United Kingdom
 # Limited and Leonardo MW Ltd
 #
 # This software is developed by BAE Systems (Operations)
 # Limited, Electronic Systems, and is the Intellectual
 # Property of BAE Systems (Operations) Limited, Electronic Systems.
 #
 # The information set out in this document is provided solely
 # on an 'as is' basis and the co-developers of this software make
 # no warranties expressed or implied, including no warranties
 # as to completeness, accuracy or fitness for purpose, with respect
 # to any of the information.
 ##############################################################
*/
#include "sqlServer_modMain.h"

# if defined(_WIN32 ) || defined( WIN32 )
#   include <windows.h>
# endif
# include <sql.h>
# include <sqlext.h>
# include "sqlerc.h"

# include <stdio.h>
# include <stdlib.h>
# include <string.h>

/************************************************************************************************************************/

SQLHSTMT dbExecute( sqlServer_modMain__context* context, SQLHDBC hdbc, char *SQLString);
ECOA__SQL__Error dbDumpSQL( sqlServer_modMain__context* context, SQLHDBC hdbc, char* fileName );
ECOA__SQL__Error dbTakeSQL( sqlServer_modMain__context* context, SQLHDBC hdbc, char* fileName );
ECOA__SQL__Error dbDumpXML( sqlServer_modMain__context* context, SQLHDBC hdbc, char* fileName );
ECOA__SQL__Error dbTakeXML( sqlServer_modMain__context* context, SQLHDBC hdbc, char* fileName );

/************************************************************************************************************************/

static char* toStrz( const ECOA__SQL__String* s, char* buff ) // ECOA string slice
{
	strncpy( buff, s->data, s->current_size );
	buff[s->current_size] = '\000';
	return buff;
}

/************************************************************************************************************************/

void sqlServer_modMain__INITIALIZE__received(sqlServer_modMain__context *context)
{
   /* User Code Here */
}

void sqlServer_modMain__START__received(sqlServer_modMain__context *context)
{
   /* User Code Here */
}

void sqlServer_modMain__STOP__received(sqlServer_modMain__context *context)
{
   /* User Code Here */
}

void sqlServer_modMain__SHUTDOWN__received(sqlServer_modMain__context *context)
{
   /* User Code Here */
}

/************************************************************************************************************************/

void sqlServer_modMain__getConnection__request_received
   (sqlServer_modMain__context* context,
    const ECOA__uint32 ID,
    const ECOA__SQL__String* dataSource,
    const ECOA__SQL__NameString* userName,
    const ECOA__SQL__NameString* password)
{
	SQLHDBC   hdbc;
	SQLHENV   henv;
	SQLRETURN retcode;
	ECOA__SQL__ConnectionId connectionId = 0;
	ECOA__SQL__String dataSourceC;
	ECOA__return_status errc;
	ECOA__log msg;
	/*
     *  Allocate env handle... */
# if( ODBCVER >= 0x0300 )
	retcode = SQLAllocHandle( SQL_HANDLE_ENV, (SQLHANDLE)SQL_NULL_HANDLE, &henv );
# else
	retcode = SQLAllocEnv( &henv );
# endif
	if( retcode != SQL_SUCCESS && retcode != SQL_SUCCESS_WITH_INFO ){
		SQL_ERROR_WARN("sqlServer:SQLAllocHandle(ENV)", retcode);
		errc = sqlServer_modMain_container__getConnection__response_send( context, ID, connectionId, toECOACode( retcode ));
		return;
	}
    /**/
# if( ODBCVER >= 0x0300 )
    /*
     *  Set ODBC version attribute. */
    retcode = SQLSetEnvAttr( henv, SQL_ATTR_ODBC_VERSION, (void*)SQL_OV_ODBC3, 0 );
	if( retcode != SQL_SUCCESS && retcode != SQL_SUCCESS_WITH_INFO ){
		SQL_ERROR_WARN("sqlServer:SQLSetEnvAttr", retcode);
		errc = sqlServer_modMain_container__getConnection__response_send( context, ID, connectionId, toECOACode( retcode ));
	}
# endif
    /*
     *  Allocate connection handle... */
# if( ODBCVER >= 0x0300 )
    retcode = SQLAllocHandle( SQL_HANDLE_DBC, henv, &hdbc );
# else
    retcode = SQLAllocConnect( henv, &hdbc );
# endif
	if( retcode != SQL_SUCCESS && retcode != SQL_SUCCESS_WITH_INFO ){
		SQL_ERROR_WARN("sqlServer:SQLAllocHandle(DBC)", retcode);
		errc = sqlServer_modMain_container__getConnection__response_send( context, ID, connectionId, toECOACode( retcode ));
	}
    /**/
# if( ODBCVER >= 0x0300 )
    SQLFreeHandle( SQL_HANDLE_ENV, henv );
# else
    SQLFreeEnv( henv );
# endif
    /**/
    strncpy( dataSourceC.data, dataSource->data, ( dataSourceC.current_size = dataSource->current_size ));
	msg.current_size = sprintf( msg.data, "sqlServer:Connecting to data source \"%s\"", dataSourceC.data );
	sqlServer_modMain_container__log_info( context, msg );
	/**/
  	retcode = SQLConnect( hdbc, (SQLCHAR*)dataSourceC.data, dataSourceC.current_size,
                        		(SQLCHAR*)userName->data,   userName->current_size,
								(SQLCHAR*)password->data,   password->current_size );
	if( retcode != SQL_SUCCESS && retcode != SQL_SUCCESS_WITH_INFO ){
		SQL_ERROR_WARN("sqlServer:SQLConnect", retcode);
		errc = sqlServer_modMain_container__getConnection__response_send( context, ID, connectionId, toECOACode( retcode ));
	}
	/*
	 * We now have a valid connection handle (hdbc) which we can return...
	 */
	connectionId = (ECOA__SQL__ConnectionId)hdbc;
	errc = sqlServer_modMain_container__getConnection__response_send( context, ID, connectionId, toECOACode( retcode ));
}

/************************************************************************************************************************/

void sqlServer_modMain__endConnection__request_received
   (sqlServer_modMain__context* context,
    const ECOA__uint32 ID,
    const ECOA__SQL__ConnectionId connection)
{
	ECOA__SQL__Error retcode;
	SQLHDBC hdbc;
	//
	if( connection > 0 ){
		hdbc = (SQLHDBC)connection;
		/*
		 *  Close out the ODBC connection. */
		retcode = SQLDisconnect( hdbc );
		if( retcode == SQL_SUCCESS ){
# if( ODBCVER >= 0x0300 )
			retcode = SQLFreeHandle( SQL_HANDLE_DBC, hdbc );
# else
			retcode = SQLFreeConnect( hdbc );
# endif
		}else
			retcode = SQL_INVALID_HANDLE;
	}
	sqlServer_modMain_container__endConnection__response_send( context, ID, toECOACode( retcode ));
}

/************************************************************************************************************************/

void sqlServer_modMain__SQLExecute__request_received
   (sqlServer_modMain__context* context,
    const ECOA__uint32 ID,
    const ECOA__SQL__ConnectionId connection,
    const ECOA__SQL__String* SQLString)
{
	ECOA__SQL__ResultSetId transaction = 0;
	ECOA__log msg;
	SQLHSTMT  hstmt;
	SQLRETURN retcode;
	SQLHDBC hdbc;
	//
	if( connection > 0 ){
		hdbc = (SQLHDBC)connection;
		/*
		 *  Create an SQL statement ... */
# if( ODBCVER >= 0x0300 )
		retcode = SQLAllocHandle( SQL_HANDLE_STMT, hdbc, &hstmt );
# else
		retcode = SQLAllocStmt( hdbc, &hstmt );
# endif
		if( retcode != SQL_SUCCESS && retcode != SQL_SUCCESS_WITH_INFO ){
			SQL_ERROR_WARN("sqlServer:SQLAllocHandle(STMT)", retcode);
			sqlServer_modMain_container__SQLExecute__response_send( context, ID, transaction );
			return;
		}
		transaction = (ECOA__SQL__ResultSetId)hstmt;
		/*
		 *  Add the SQLString to the SQL statement ... */
		retcode = SQLPrepare( hstmt, (SQLCHAR*)(SQLString->data), SQLString->current_size );
		if( retcode != SQL_SUCCESS && retcode != SQL_SUCCESS_WITH_INFO ){
			SQL_ERROR_WARN("sqlServer:SQLPrepare", retcode);
		}
		/*
		 *  Send it */
		retcode = SQLExecute( hstmt );
		if( retcode != SQL_SUCCESS && retcode != SQL_SUCCESS_WITH_INFO ){
			char buff[ECOA__SQL__String_MAXSIZE];
			msg.current_size = sprintf( msg.data, "sqlServer:SQLExecute: SQLError code %d (%s)\nSQLExecute( '%s' )", retcode, SQLErc(retcode), toStrz( SQLString, buff ));
			sqlServer_modMain_container__log_warning( context, msg );
			//
			sqlServer_modMain_container__SQLExecute__response_send( context, ID, transaction );
			return;
		}
	}
	sqlServer_modMain_container__SQLExecute__response_send( context, ID, transaction );
}

/************************************************************************************************************************/

void sqlServer_modMain__getResult__request_received
   (sqlServer_modMain__context* context,
    const ECOA__uint32 ID,
    const ECOA__SQL__ConnectionId connection,
    const ECOA__SQL__ResultSetId transaction,
    const ECOA__SQL__ScrollSet* scroll)
{
	SQLRETURN retcode;
	SQLSMALLINT orientn;
	//
	switch( scroll->orientation ){
		case ECOA__SQL__ScrollOrntn_NEXT:
			orientn = SQL_FETCH_NEXT; break;
		case ECOA__SQL__ScrollOrntn_PRIOR:
			orientn = SQL_FETCH_PRIOR; break;
		case ECOA__SQL__ScrollOrntn_FIRST:
			orientn = SQL_FETCH_FIRST; break;
		case ECOA__SQL__ScrollOrntn_LAST:
			orientn = SQL_FETCH_LAST; break;
		case ECOA__SQL__ScrollOrntn_ABSOLUTE:
			orientn = SQL_FETCH_ABSOLUTE; break;
		case ECOA__SQL__ScrollOrntn_RELATIVE:
			orientn = SQL_FETCH_RELATIVE; break;
		case ECOA__SQL__ScrollOrntn_BOOKMARK:
			orientn = SQL_FETCH_BOOKMARK; break;
		default:
			orientn = SQL_FETCH_NEXT;
	}
	if( connection > 0 && transaction > 0 ){
		retcode = SQLFetchScroll( (SQLHSTMT)transaction, orientn, (SQLLEN)(scroll->offset) );
		if( retcode != SQL_SUCCESS && retcode != SQL_SUCCESS_WITH_INFO && retcode != SQL_NO_DATA ){
			SQL_ERROR_WARN("sqlServer:SQLFetchScroll", retcode);
		}
	}else{
		retcode = SQL_INVALID_HANDLE;
	}
	sqlServer_modMain_container__getResult__response_send( context, ID, toECOACode( retcode ));
}

/************************************************************************************************************************/

void sqlServer_modMain__getResultData__request_received
   (sqlServer_modMain__context* context,
    const ECOA__uint32 ID,
    const ECOA__SQL__ConnectionId connection,
    const ECOA__SQL__ResultSetId transaction,
    const ECOA__uint16 columnNo)
{
	ECOA__SQL__ColumnValue columnValue;
	SQLRETURN   retcode;
	SQLCHAR     columnName[128];
	SQLSMALLINT colNameActualLen;
	SQLSMALLINT dataType;
	SQLULEN     colSize;
	SQLSMALLINT decimalDigits;
	SQLSMALLINT nullable;
	SQLLEN      actualLen;
	//
	columnValue.dataType = 0;
	memset( columnValue.value, 0, ECOA__SQL__Buffer_MAXSIZE );
	//
	if( connection > 0 && transaction > 0 ){
		dataType = colNameActualLen = colSize = decimalDigits = nullable = 0;
		retcode = SQLDescribeCol( (SQLHSTMT)transaction, (SQLSMALLINT)columnNo,
				columnName, 128, &colNameActualLen,
				&dataType, &colSize, &decimalDigits,
				&nullable );
		if( retcode != SQL_SUCCESS && retcode != SQL_SUCCESS_WITH_INFO ){
			SQL_ERROR_WARN("sqlServer:SQLDescribeCol", retcode);
		}
		//
		retcode = SQLGetData( (SQLHSTMT)transaction, (SQLSMALLINT)columnNo, SQL_C_DEFAULT, columnValue.value, (SQLLEN)ECOA__SQL__Buffer_MAXSIZE, NULL );
		if( retcode != SQL_SUCCESS && retcode != SQL_SUCCESS_WITH_INFO ){
			SQL_ERROR_WARN("sqlServer:SQLGetData", retcode);
		}else{
			columnValue.dataType = dataType;
			columnValue.dataSize = colSize;
		}
	}else{
		retcode = SQL_INVALID_HANDLE;
	}
	sqlServer_modMain_container__getResultData__response_send( context, ID, &columnValue, toECOACode( retcode ));
}

/************************************************************************************************************************/

void sqlServer_modMain__getResultColumnCount__request_received
   (sqlServer_modMain__context* context,
    const ECOA__uint32 ID,
    const ECOA__SQL__ConnectionId connection,
    const ECOA__SQL__ResultSetId transaction)
{
	ECOA__uint16 columnCount = 0;
	SQLRETURN retcode;
	SQLHDBC hdbc;
	//
	if( connection > 0 && transaction > 0 ){
		retcode = SQLNumResultCols( (SQLHSTMT)transaction, (SQLSMALLINT*)&columnCount );
		if( retcode != SQL_SUCCESS && retcode != SQL_SUCCESS_WITH_INFO ){
			SQL_ERROR_WARN("sqlServer:SQLNumResultCols", retcode);
			columnCount = 0;
		}
	}else{
		retcode = SQL_INVALID_HANDLE;
	}
	sqlServer_modMain_container__getResultColumnCount__response_send( context, ID, columnCount, toECOACode( retcode ));
}

/************************************************************************************************************************/

void sqlServer_modMain__getResultColumnInfo__request_received
   (sqlServer_modMain__context* context,
    const ECOA__uint32 ID,
    const ECOA__SQL__ConnectionId connection,
    const ECOA__SQL__ResultSetId transaction,
    const ECOA__uint16 columnNo)
{
	ECOA__SQL__ColumnInfo columnInfo = (ECOA__SQL__ColumnInfo){{0,""},0,0,0,ECOA__FALSE};
	SQLRETURN retcode;
	SQLHDBC hdbc;
	char columnName[128];
	short colNameActualLen;
	short dataType;
	unsigned int colSize;
	short decimalDigits;
	short nullable;
	//
	if( connection > 0 && transaction > 0 ){
		dataType = colNameActualLen = colSize = decimalDigits = nullable = 0;
		retcode = SQLDescribeCol( (SQLHSTMT)transaction, (SQLSMALLINT)columnNo,
				(SQLCHAR*)&columnName, 128, (SQLSMALLINT*)&colNameActualLen,
				(SQLSMALLINT*)&dataType, (SQLULEN*)&colSize, (SQLSMALLINT*)&decimalDigits,
				(SQLSMALLINT*)&nullable );
		if( retcode != SQL_SUCCESS && retcode != SQL_SUCCESS_WITH_INFO ){
			SQL_ERROR_WARN("sqlServer:SQLDescribeCol", retcode);
		}else{
			columnInfo.name.current_size = colNameActualLen;
			strncpy( columnInfo.name.data, columnName, colNameActualLen );
			switch( dataType ){
				case SQL_CHAR:      columnInfo.dataType = ECOA__SQL__DataType_CHAR;      break;
				case SQL_VARCHAR:   columnInfo.dataType = ECOA__SQL__DataType_VARCHAR;   break;
				case SQL_NUMERIC:   columnInfo.dataType = ECOA__SQL__DataType_NUMERIC;   break;
				case SQL_DECIMAL:   columnInfo.dataType = ECOA__SQL__DataType_DECIMAL;   break;
				case SQL_INTEGER:   columnInfo.dataType = ECOA__SQL__DataType_INTEGER;   break;
				case SQL_SMALLINT:  columnInfo.dataType = ECOA__SQL__DataType_SMALLINT;  break;
				case SQL_FLOAT:     columnInfo.dataType = ECOA__SQL__DataType_FLOAT;     break;
				case SQL_REAL:      columnInfo.dataType = ECOA__SQL__DataType_REAL;      break;
				case SQL_DOUBLE:    columnInfo.dataType = ECOA__SQL__DataType_DOUBLE;    break;
				case SQL_DATE:      columnInfo.dataType = ECOA__SQL__DataType_DATE;      break;
				case SQL_TIME:      columnInfo.dataType = ECOA__SQL__DataType_TIME;      break;
				case SQL_TIMESTAMP: columnInfo.dataType = ECOA__SQL__DataType_TIMESTAMP; break;
				case SQL_TYPE_DATE: columnInfo.dataType = ECOA__SQL__DataType_DATE;      break;
				case SQL_TYPE_TIME: columnInfo.dataType = ECOA__SQL__DataType_TIME;      break;
				case SQL_TYPE_TIMESTAMP: columnInfo.dataType = ECOA__SQL__DataType_TIMESTAMP; break;
				default:            columnInfo.dataType = dataType;
			}

			columnInfo.columnSize = colSize;
			columnInfo.decimalDigits = decimalDigits;
			columnInfo.nullable = ( nullable != 0 ? ECOA__TRUE : ECOA__FALSE );
		}
	}else{
		retcode = SQL_INVALID_HANDLE;
	}
	sqlServer_modMain_container__getResultColumnInfo__response_send( context, ID, &columnInfo, toECOACode( retcode ));
}

/************************************************************************************************************************/

void sqlServer_modMain__getStatus__request_received
   (sqlServer_modMain__context* context,
    const ECOA__uint32 ID,
    const ECOA__SQL__ConnectionId connection,
    const ECOA__SQL__ResultSetId transaction,
    const ECOA__int16 recordNo)
{
	ECOA__SQL__ResultStatus resultStatus;
	SQLRETURN   retcode;
	SQLHDBC     hdbc;
	SQLCHAR     SQLState[5];
	SQLINTEGER  NativeError;
	SQLCHAR     MessageText[1024];
	SQLSMALLINT TextLength;
	     //
	if( connection > 0 && transaction > 0 ){
		retcode = SQLGetDiagRec( SQL_HANDLE_STMT, (SQLHSTMT)transaction, (SQLSMALLINT)recordNo, SQLState, &NativeError, MessageText, 1024, &TextLength );
		if( retcode != SQL_SUCCESS && retcode != SQL_SUCCESS_WITH_INFO ){
			SQL_ERROR_WARN("sqlServer:SQLNumResultCols", retcode);
		}else{
			memcpy( resultStatus.SQLState, SQLState, 5 );
			resultStatus.NativeError = NativeError;
			strncpy( (char*)(resultStatus.MessageText), (char*)MessageText, TextLength );
			resultStatus.MessageLength = TextLength;
		}
	}else{
		retcode = SQL_INVALID_HANDLE;
	}
	sqlServer_modMain_container__getStatus__response_send( context, ID, &resultStatus, toECOACode( retcode ));
}

/************************************************************************************************************************/

void sqlServer_modMain__setUpdateTable__request_received
   (sqlServer_modMain__context* context,
    const ECOA__uint32 ID,
    const ECOA__SQL__ConnectionId connection,
    const ECOA__SQL__NameString* tableName)
{
	/* TODO */
}

/************************************************************************************************************************/

void sqlServer_modMain__dump__request_received
   (sqlServer_modMain__context* context,
    const ECOA__uint32 ID,
    const ECOA__SQL__ConnectionId connection,
    const ECOA__SQL__String* fileName,
    const ECOA__SQL__FileType format)
{
	SQLRETURN retcode;
	char buff[ECOA__SQL__String_MAXSIZE];
	//
	if( connection > 0 ){
		switch( format ){
			case ECOA__SQL__FileType_XML:
				retcode = dbDumpXML( context, (SQLHDBC)connection, toStrz( fileName, buff ));
				break;
				/**/
			case ECOA__SQL__FileType_SQL:
			default:
				retcode = dbDumpSQL( context, (SQLHDBC)connection, toStrz( fileName, buff ));
				break;
		}
	}else{
		retcode = SQL_INVALID_HANDLE;
	}
	sqlServer_modMain_container__dump__response_send( context, ID, toECOACode( retcode ));
}

/************************************************************************************************************************/

void sqlServer_modMain__take__request_received
   (sqlServer_modMain__context* context,
    const ECOA__uint32 ID,
    const ECOA__SQL__ConnectionId connection,
    const ECOA__SQL__String* fileName,
    const ECOA__SQL__FileType format)
{
	SQLRETURN retcode;
	char buff[ECOA__SQL__String_MAXSIZE];
	//
	if( connection > 0 ){
		switch( format ){
			case ECOA__SQL__FileType_XML:
				retcode = dbTakeXML( context, (SQLHDBC)connection, toStrz( fileName, buff ));
				break;
				/**/
			case ECOA__SQL__FileType_SQL:
			default:
				retcode = dbTakeSQL( context, (SQLHDBC)connection, toStrz( fileName, buff ));
				break;
		}
	}else{
		retcode = SQL_INVALID_HANDLE;
	}
	sqlServer_modMain_container__take__response_send( context, ID, toECOACode( retcode ));
}

/************************************************************************************************************************/

void sqlServer_modMain__getHelp__request_received
   (sqlServer_modMain__context* context,
    const ECOA__uint32 ID,
    const ECOA__SQL__ConnectionId connection)
{
   /* User Code Here */
}

/************************************************************************************************************************/

void sqlServer_modMain__listTables__request_received
   (sqlServer_modMain__context* context,
    const ECOA__uint32 ID,
    const ECOA__SQL__ConnectionId connection)
{
	SQLHSTMT  hstmt;
	SQLRETURN retcode = 0;
	ECOA__SQL__TableNameList tableList;
	//
	tableList.current_size = 0;
    /*
     *  Create an SQL statement ... */
# if( ODBCVER >= 0x0300 )
    retcode = SQLAllocHandle( SQL_HANDLE_STMT, (SQLHDBC)connection, &hstmt );
# else
    retcode = SQLAllocStmt( (SQLHDBC)connection, &hstmt );
# endif
		if( retcode != SQL_SUCCESS && retcode != SQL_SUCCESS_WITH_INFO ){
			SQL_ERROR_REPORT("sqlServer:SQLAllocHandle(STMT)", retcode);
			return;
		}
    /**/
	retcode = SQLTables( hstmt, NULL, 0, NULL, 0, NULL, 0, (SQLCHAR*)"TABLE", SQL_NTS );
		if( retcode != SQL_SUCCESS && retcode != SQL_SUCCESS_WITH_INFO ){
			SQL_ERROR_REPORT("sqlServer:SQLTables(STMT)", retcode);
			return;
		}
    /**/
	while(( retcode = SQLFetch( hstmt )) == 0 ){
		char tableName[1024];
		SQLGetData( hstmt, 3, SQL_C_CHAR, tableName, sizeof(tableName), NULL );
		tableList.data[tableList.current_size].current_size = sprintf( tableList.data[tableList.current_size].data, tableName );
		tableList.current_size++;
	}
	//
	sqlServer_modMain_container__listTables__response_send( context, ID, &tableList, toECOACode( retcode ));
}

/************************************************************************************************************************/

void sqlServer_modMain__getLimits__request_received
   (sqlServer_modMain__context* context,
    const ECOA__uint32 ID,
    const ECOA__SQL__ConnectionId connection)
{
	ECOA__SQL__IntegerList limits = (ECOA__SQL__IntegerList){ 9,
		{	ECOA__SQL__String_MAXSIZE,
			ECOA__SQL__NameString_MAXSIZE,
			ECOA__SQL__CLOB_MAXSIZE,
			ECOA__SQL__BLOB_MAXSIZE,
			ECOA__SQL__Buffer_MAXSIZE,
			ECOA__SQL__IntegerList_MAXSIZE,
			ECOA__SQL__UpdateTableList_MAXSIZE,
			ECOA__SQL__TableNameList_MAXSIZE,
			ECOA__SQL__SQLStatusCode_MAXSIZE
		}
	};
	SQLRETURN retcode = 0;
	//
	sqlServer_modMain_container__getLimits__response_send( context, ID, &limits, toECOACode( retcode ));
}

/************************************************************************************************************************/

void sqlServer_modMain__getErrorText__request_received
   (sqlServer_modMain__context* context,
    const ECOA__uint32 ID,
    const ECOA__SQL__Error requestStatus)
{
	ECOA__SQL__String res;
	/**/
	switch( requestStatus ){
		case ECOA__SQL__Error_NO_ERROR:
			res.current_size = sprintf( res.data, "ECOA.SQL: No Error" ); break;
		case ECOA__SQL__Error_NOT_CONNECTED:
			res.current_size = sprintf( res.data, "ECOA.SQL: Not Connected" ); break;
		case ECOA__SQL__Error_END_OF_RESULTS:
			res.current_size = sprintf( res.data, "ECOA.SQL: End of Results" ); break;
		case ECOA__SQL__Error_UNEXPECTED_MESSAGE:
			res.current_size = sprintf( res.data, "ECOA.SQL: Unexpected Message" ); break;
		case ECOA__SQL__Error_CLIENT_NOT_VALID:
			res.current_size = sprintf( res.data, "ECOA.SQL: Client Not Valid" ); break;
		case ECOA__SQL__Error_SQL_ERROR:
			res.current_size = sprintf( res.data, "ECOA.SQL: SQL Error" ); break;
		case ECOA__SQL__Error_SQL_NO_DATA:
			res.current_size = sprintf( res.data, "ECOA.SQL: Query Returned No Data" ); break;
		case ECOA__SQL__Error_SQL_NEED_DATA:
			res.current_size = sprintf( res.data, "ECOA.SQL: Query Needs Data" ); break;
		case ECOA__SQL__Error_SQL_NOT_READY:
			res.current_size = sprintf( res.data, "ECOA.SQL: Data Source Not Ready" ); break;
		default :
			res.current_size = sprintf( res.data, "(unknown)" ); break;
	}
	sqlServer_modMain_container__getErrorText__response_send( context, ID, &res );
}

/************************************************************************************************************************/

void sqlServer_modMain__getCounts__request_received
   (sqlServer_modMain__context* context,
    const ECOA__uint32 ID,
    const ECOA__SQL__ConnectionId connection)
{
   /* User Code Here */
}

/************************************************************************************************************************/

SQLHSTMT dbExecute( sqlServer_modMain__context* context, SQLHDBC hdbc, char *SQLString)
{
	SQLHSTMT  hstmt;
	SQLRETURN retcode;
	/*
	 *  Create an SQL statement ... */
# if( ODBCVER >= 0x0300 )
	retcode = SQLAllocHandle( SQL_HANDLE_STMT, (SQLHDBC)hdbc, &hstmt );
# else
	retcode = SQLAllocStmt( (SQLHDBC)hdbc, &hstmt );
# endif
	if( retcode != SQL_SUCCESS && retcode != SQL_SUCCESS_WITH_INFO ){
		SQL_ERROR_WARN("sqlServer:SQLAllocHandle(STMT)", retcode);
		return 0;
	}
	/*
	 *  Add the SQLString to the SQL statement ... */
	retcode = SQLPrepare( hstmt, (SQLCHAR*)SQLString, SQL_NTS );
	if( retcode != SQL_SUCCESS && retcode != SQL_SUCCESS_WITH_INFO ){
		SQL_ERROR_WARN("sqlServer:SQLPrepare", retcode);
//		return 0;  // NO...
	}
	/*
	 *  Send it */
	retcode = SQLExecute( hstmt );
	if( retcode != SQL_SUCCESS && retcode != SQL_SUCCESS_WITH_INFO ){
		SQL_ERROR_WARN("sqlServer:SQLExecute", retcode);
		return 0;
	}
	return hstmt;
}

/************************************************************************************************************************/

ECOA__SQL__Error dbDumpSQL( sqlServer_modMain__context* context, SQLHDBC hdbc, char* fileName )
{
	return ECOA__SQL__Error_NO_ERROR;
}

/************************************************************************************************************************/

ECOA__SQL__Error dbTakeSQL( sqlServer_modMain__context* context, SQLHDBC hdbc, char* fileName )
{
	return ECOA__SQL__Error_NO_ERROR;
}

/************************************************************************************************************************/

ECOA__SQL__Error dbDumpXML( sqlServer_modMain__context* context, SQLHDBC hdbc, char* fileName )
{
	return ECOA__SQL__Error_NO_ERROR;
}

/************************************************************************************************************************/

ECOA__SQL__Error dbTakeXML( sqlServer_modMain__context* context, SQLHDBC hdbc, char* fileName )
{
	return ECOA__SQL__Error_NO_ERROR;
}

