/*
 ##############################################################
 # (C) 2015 BAE SYSTEMS
 #
 # ECOA Data Servers Sample - ODBC/SQL Error Utils.
 #
 # $Id: sqlerc.h,v 2.1 2017/11/06 15:33:31 ksappleb Exp $
 #
 # Part of the BAE Systems, Rochester, ECOA Samples collection
 #
 # Created by K S Appleby, BAE Systems, Airport Works, Rochester, Kent
 #
 # This software is developed for and on behalf of BAE Systems (Operations)
 # Limited, Dassault Aviation, Bull SAS, Thales Systemes Aeroportes,
 # GE Aviation Systems Limited,
 # General Dynamics United Kingdom Limited and Leonardo MW Ltd,
 # and the copyright is owned by BAE Systems (Operations) Limited,
 # Dassault Aviation, Bull SAS, Thales Systemes Aeroportes, 
 # GE Aviation Systems Limited, General Dynamics United Kingdom
 # Limited and Leonardo MW Ltd
 #
 # This software is developed by BAE Systems (Operations)
 # Limited, Electronic Systems, and is the Intellectual
 # Property of BAE Systems (Operations) Limited, Electronic Systems.
 #
 # The information set out in this document is provided solely
 # on an 'as is' basis and the co-developers of this software make
 # no warranties expressed or implied, including no warranties
 # as to completeness, accuracy or fitness for purpose, with respect
 # to any of the information.
 ##############################################################
*/
#ifndef _SQLERC_H_
#define _SQLERC_H_

  // In case they're not already included...
# include <sql.h>
# include <sqlext.h>
# include <stdio.h>
# include <ECOA__SQL.h>

# ifdef __GNUC__
#   define INLINE __inline__
# else
#   define INLINE inline
# endif

static INLINE
char *SQLErc( int e )
  // Return a string version of error code e
{
  static char buf[16];
  switch( e ){
    case SQL_SUCCESS:
      return "SQL_SUCCESS";
      break;
    case SQL_SUCCESS_WITH_INFO:
      return "SQL_SUCCESS_WITH_INFO";
      break;
#   if( ODBCVER >= 0x0300 )
      case SQL_NO_DATA:
        return "SQL_NO_DATA";
        break;
#   endif
    case SQL_ERROR:
      return "SQL_ERROR";
      break;
    case SQL_INVALID_HANDLE:
      return "SQL_INVALID_HANDLE";
      break;
    case SQL_NEED_DATA:
      return "SQL_NEED_DATA";
      break;
    case SQL_STILL_EXECUTING:
      return "SQL_STILL_EXECUTING";
      break;
    default:
      sprintf( buf, "%d (unknown)", e );
      return buf;
  }
}

  // Report errors, and abort if need be.
# define SQL_ERROR_ABORT( s, e )\
  {\
	ECOA__log msg;\
    msg.current_size = sprintf( msg.data, "%s: SQLError code %d (%s)", s, e, SQLErc(e) );\
    sqlServer_modMain_container__raise_fatal_error( context, msg );\
    /*exit( 4 );*/ \
  }

// Report errors.
# define SQL_ERROR_REPORT( s, e )\
  {\
	ECOA__log msg;\
	msg.current_size = sprintf( msg.data, "%s: SQLError code %d (%s)", s, e, SQLErc(e) );\
    sqlServer_modMain_container__raise_error( context, msg );\
  }

// Report warnings.
# define SQL_ERROR_WARN( s, e )\
  {\
	ECOA__log msg;\
	msg.current_size = sprintf( msg.data, "%s: SQLError code %d (%s)", s, e, SQLErc(e) );\
	sqlServer_modMain_container__log_warning( context, msg );\
  }

static
ECOA__SQL__Error toECOACode( SQLRETURN erc )
{
	  switch( erc ){
	    case SQL_SUCCESS:
	      return ECOA__SQL__Error_NO_ERROR;
	      break;
	    case SQL_SUCCESS_WITH_INFO:
	      return ECOA__SQL__Error_NO_ERROR;
	      break;
	#   if( ODBCVER >= 0x0300 )
	      case SQL_NO_DATA:
	        return ECOA__SQL__Error_SQL_NO_DATA;
	        break;
	#   endif
	    case SQL_ERROR:
	      return ECOA__SQL__Error_SQL_ERROR;
	      break;
	    case SQL_INVALID_HANDLE:
	      return ECOA__SQL__Error_NOT_CONNECTED;
	      break;
	    case SQL_NEED_DATA:
	      return ECOA__SQL__Error_SQL_NEED_DATA;
	      break;
	    case SQL_STILL_EXECUTING:
	      return ECOA__SQL__Error_SQL_NOT_READY;
	      break;
	    default:
	      return ECOA__SQL__Error_UNEXPECTED_MESSAGE;
	  }
}

#endif
