-- File ECOA-DBM.ads

-- Standard ECOA Types
with ECOA;


package ECOA.DBM is

-- Simple type, min is 0, max is 4294967295
type ConnectionId is new ECOA.Unsigned_32_Type range 0 .. 4294967295;

-- Enum type
type Option is new ECOA.Signed_16_Type;
Option_SETCACHESIZE : constant Option := 1;
Option_SETFASTMODE : constant Option := 2;
Option_SETSYNCMODE : constant Option := 3;
Option_SETCENTFREE : constant Option := 4;
Option_SETCOALESCEBLKS : constant Option := 5;
Option_SETMAXMAPSIZE : constant Option := 6;
Option_SETMMAP : constant Option := 7;
Option_GETFLAGS : constant Option := 8;
Option_GETMMAP : constant Option := 9;
Option_GETCACHESIZE : constant Option := 10;
Option_GETSYNMODE : constant Option := 11;
Option_GETCENTFREE : constant Option := 12;
Option_GETCOALESCEBLKS : constant Option := 13;
Option_GETMAXMAPSIZE : constant Option := 14;
Option_GETDBNAME : constant Option := 15;

-- Enum type
type OpenFlag is new ECOA.Signed_16_Type;
OpenFlag_READER : constant OpenFlag := 0;
OpenFlag_WRITER : constant OpenFlag := 1;
OpenFlag_WRCREAT : constant OpenFlag := 2;
OpenFlag_NEWDB : constant OpenFlag := 3;
OpenFlag_FAST : constant OpenFlag := 16;
OpenFlag_SYNC : constant OpenFlag := 32;
OpenFlag_NOLOCK : constant OpenFlag := 64;
OpenFlag_NOMMAP : constant OpenFlag := 128;
OpenFlag_CLOEXEC : constant OpenFlag := 256;

-- Simple type, min is -2147483647, max is 2147483647
type OpenFlags is new ECOA.Signed_32_Type range -2147483647 .. 2147483647;

-- Enum type
type StoreFlag is new ECOA.Signed_16_Type;
StoreFlag_INSERT : constant StoreFlag := 0;
StoreFlag_REPLACE : constant StoreFlag := 1;

-- Enum type
type Permission is new ECOA.Unsigned_8_Type;
Permission_READ : constant Permission := 4;
Permission_WRITE : constant Permission := 2;
Permission_EXECUTE : constant Permission := 1;
Permission_READEXEC : constant Permission := 5;
Permission_ALL : constant Permission := 7;

-- Record type
type Permissions is
   record
      Owner : Permission;
      Group : Permission;
      Others : Permission;
   end record;

-- Array type
type String_Size is new ECOA.Unsigned_32_Type range 0..65536;
subtype String_Index is String_Size range 0..65536-1;
type String_Data is array
    (String_Index) of ECOA.Character_8_Type;
type String is
   record
      Current_Size : String_Size;
      Data         : String_Data;
   end record;

-- Array type
type NameString_Size is new ECOA.Unsigned_32_Type range 0..128;
subtype NameString_Index is NameString_Size range 0..128-1;
type NameString_Data is array
    (NameString_Index) of ECOA.Character_8_Type;
type NameString is
   record
      Current_Size : NameString_Size;
      Data         : NameString_Data;
   end record;

-- Array type
type Datum_Size is new ECOA.Unsigned_32_Type range 0..65536;
subtype Datum_Index is Datum_Size range 0..65536-1;
type Datum_Data is array
    (Datum_Index) of ECOA.Character_8_Type;
type Datum is
   record
      Current_Size : Datum_Size;
      Data         : Datum_Data;
   end record;


end ECOA.DBM;
