-- File ECOA-FileIO.ads

-- Standard ECOA Types
with ECOA;


package ECOA.FileIO is

-- Array type
type string_Size is new ECOA.Unsigned_32_Type range 0..65536;
subtype string_Index is string_Size range 0..65536-1;
type string_Data is array
    (string_Index) of ECOA.Character_8_Type;
type string is
   record
      Current_Size : string_Size;
      Data         : string_Data;
   end record;

-- Fixed Array type
type FileName_Index is range 0..256-1;
type FileName is array
    (FileName_Index) of ECOA.Character_8_Type;

-- Enum type
type ModeFlag is new ECOA.Signed_16_Type;
ModeFlag_READ : constant ModeFlag := 0;
ModeFlag_WRITE : constant ModeFlag := 1;
ModeFlag_READWRITE : constant ModeFlag := 2;
ModeFlag_APPEND : constant ModeFlag := 8;
ModeFlag_NONBLOCK : constant ModeFlag := 128;
ModeFlag_CREATE : constant ModeFlag := 256;
ModeFlag_TRUNCATE : constant ModeFlag := 512;
ModeFlag_EXCLUSIVE : constant ModeFlag := 1024;
ModeFlag_NOCTTY : constant ModeFlag := 2048;
ModeFlag_PRIVATE : constant ModeFlag := 4096;
ModeFlag_LARGEFILE : constant ModeFlag := 8192;
ModeFlag_BINARY : constant ModeFlag := 16384;

-- Enum type
type SeekBase is new ECOA.Unsigned_8_Type;
SeekBase_SEEK_SET : constant SeekBase := 0;
SeekBase_SEEK_CUR : constant SeekBase := 1;
SeekBase_SEEK_END : constant SeekBase := 2;

-- Enum type
type Permission is new ECOA.Unsigned_8_Type;
Permission_READ : constant Permission := 4;
Permission_WRITE : constant Permission := 2;
Permission_EXECUTE : constant Permission := 1;
Permission_READEXEC : constant Permission := 5;
Permission_ALL : constant Permission := 7;

-- Enum type
type EntryType is new ECOA.Signed_32_Type;
EntryType_NONE : constant EntryType := 0;
EntryType_CHARACTER : constant EntryType := 131072;
EntryType_DIRECTORY : constant EntryType := 262144;
EntryType_BLOCK : constant EntryType := 393216;
EntryType_REGULAR : constant EntryType := 1048576;
EntryType_LINK : constant EntryType := 1179648;
EntryType_OTHER : constant EntryType := 8912896;

-- Record type
type Permissions is
   record
      Owner : Permission;
      Group : Permission;
      Others : Permission;
   end record;

-- Record type
type FileSpec is
   record
      FileName : FileName;
      Access : ModeFlag;
      CreatePermissions : Permissions;
   end record;

-- Record type
type DirEntry is
   record
      Name : FileName;
      EntryType : EntryType;
   end record;


end ECOA.FileIO;
