-- File ECOA-SQL.ads

-- Standard ECOA Types
with ECOA;


package ECOA.SQL is

-- Enum type
type DataType is new ECOA.Unsigned_8_Type;
DataType_CHAR : constant DataType := 0;
DataType_VARCHAR : constant DataType := 1;
DataType_NUMERIC : constant DataType := 2;
DataType_DECIMAL : constant DataType := 3;
DataType_INTEGER : constant DataType := 4;
DataType_SMALLINT : constant DataType := 5;
DataType_FLOAT : constant DataType := 6;
DataType_REAL : constant DataType := 7;
DataType_DOUBLE : constant DataType := 8;
DataType_DATE : constant DataType := 9;
DataType_TIME : constant DataType := 10;
DataType_TIMESTAMP : constant DataType := 11;
DataType_CLOB : constant DataType := 12;
DataType_BLOB : constant DataType := 13;

-- Enum type
type FileType is new ECOA.Unsigned_8_Type;
FileType_SQL : constant FileType := 0;
FileType_XML : constant FileType := 1;

-- Enum type
type Error is new ECOA.Unsigned_8_Type;
Error_NO_ERROR : constant Error := 0;
Error_NOT_CONNECTED : constant Error := 1;
Error_END_OF_RESULTS : constant Error := 2;
Error_UNEXPECTED_MESSAGE : constant Error := 3;
Error_CLIENT_NOT_VALID : constant Error := 4;
Error_SQL_ERROR : constant Error := 5;
Error_SQL_NO_DATA : constant Error := 6;
Error_SQL_NEED_DATA : constant Error := 7;
Error_SQL_NOT_READY : constant Error := 8;

-- Fixed Array type
type StringZ_Index is range 0..4096-1;
type StringZ is array
    (StringZ_Index) of ECOA.Character_8_Type;

-- Array type
type String_Size is new ECOA.Unsigned_32_Type range 0..4096;
subtype String_Index is String_Size range 0..4096-1;
type String_Data is array
    (String_Index) of ECOA.Character_8_Type;
type String is
   record
      Current_Size : String_Size;
      Data         : String_Data;
   end record;

-- Array type
type NameString_Size is new ECOA.Unsigned_32_Type range 0..128;
subtype NameString_Index is NameString_Size range 0..128-1;
type NameString_Data is array
    (NameString_Index) of ECOA.Character_8_Type;
type NameString is
   record
      Current_Size : NameString_Size;
      Data         : NameString_Data;
   end record;

-- Fixed Array type
type CLOB_Index is range 0..1024-1;
type CLOB is array
    (CLOB_Index) of ECOA.Character_8_Type;

-- Fixed Array type
type BLOB_Index is range 0..65536-1;
type BLOB is array
    (BLOB_Index) of ECOA.Byte_Type;

-- Simple type, min is 0, max is 18446744073709551615
type ConnectionId is new ECOA.uint64 range 0 .. 18446744073709551615;

-- Simple type, min is 0, max is 18446744073709551615
type ResultSetId is new ECOA.uint64 range 0 .. 18446744073709551615;

-- Fixed Array type
type Buffer_Index is range 0..65536-1;
type Buffer is array
    (Buffer_Index) of ECOA.Byte_Type;

-- Record type
type ColumnValue is
   record
      dataType : DataType;
      dataSize : ECOA.Unsigned_32_Type;
      value : Buffer;
   end record;

-- Record type
type ColumnInfo is
   record
      name : NameString;
      dataType : DataType;
      columnSize : ECOA.Unsigned_32_Type;
      decimalDigits : ECOA.Unsigned_16_Type;
      nullable : ECOA.Boolean_8_Type;
   end record;

-- Enum type
type ScrollOrntn is new ECOA.Unsigned_8_Type;
ScrollOrntn_NEXT : constant ScrollOrntn := 0;
ScrollOrntn_PRIOR : constant ScrollOrntn := 1;
ScrollOrntn_FIRST : constant ScrollOrntn := 2;
ScrollOrntn_LAST : constant ScrollOrntn := 3;
ScrollOrntn_ABSOLUTE : constant ScrollOrntn := 4;
ScrollOrntn_RELATIVE : constant ScrollOrntn := 5;
ScrollOrntn_BOOKMARK : constant ScrollOrntn := 6;

-- Record type
type ScrollSet is
   record
      orientation : ScrollOrntn;
      offset : ECOA.Signed_64_Type;
   end record;

-- Array type
type IntegerList_Size is new ECOA.Unsigned_32_Type range 0..128;
subtype IntegerList_Index is IntegerList_Size range 0..128-1;
type IntegerList_Data is array
    (IntegerList_Index) of ECOA.Signed_32_Type;
type IntegerList is
   record
      Current_Size : IntegerList_Size;
      Data         : IntegerList_Data;
   end record;

-- Array type
type UpdateTableList_Size is new ECOA.Unsigned_32_Type range 0..32;
subtype UpdateTableList_Index is UpdateTableList_Size range 0..32-1;
type UpdateTableList_Data is array
    (UpdateTableList_Index) of NameString;
type UpdateTableList is
   record
      Current_Size : UpdateTableList_Size;
      Data         : UpdateTableList_Data;
   end record;

-- Array type
type TableNameList_Size is new ECOA.Unsigned_32_Type range 0..32;
subtype TableNameList_Index is TableNameList_Size range 0..32-1;
type TableNameList_Data is array
    (TableNameList_Index) of NameString;
type TableNameList is
   record
      Current_Size : TableNameList_Size;
      Data         : TableNameList_Data;
   end record;

-- Fixed Array type
type SQLStatusCode_Index is range 0..5-1;
type SQLStatusCode is array
    (SQLStatusCode_Index) of ECOA.Character_8_Type;

-- Record type
type ResultStatus is
   record
      SQLState : SQLStatusCode;
      NativeError : ECOA.Signed_64_Type;
      MessageText : Buffer;
      MessageLength : ECOA.Signed_16_Type;
   end record;


end ECOA.SQL;
