/* File ECOA__DBM.hpp */

#if !defined(ECOA__DBM_HPP)
#define ECOA__DBM_HPP

#include "ECOA.hpp" /* Fundamental types declared for the platform */

namespace ECOA {

namespace DBM {


/* Simple type, min is 0, max is 4294967295 */
typedef ECOA::uint32 ConnectionId;
static const ECOA::uint32 ConnectionId_minRange = 0;
static const ECOA::uint32 ConnectionId_maxRange = 4294967295;

/* Enum type */
struct Option
{
   ECOA::int16 value;
   enum EnumValues {
      SETCACHESIZE = 1,
      SETFASTMODE = 2,
      SETSYNCMODE = 3,
      SETCENTFREE = 4,
      SETCOALESCEBLKS = 5,
      SETMAXMAPSIZE = 6,
      SETMMAP = 7,
      GETFLAGS = 8,
      GETMMAP = 9,
      GETCACHESIZE = 10,
      GETSYNMODE = 11,
      GETCENTFREE = 12,
      GETCOALESCEBLKS = 13,
      GETMAXMAPSIZE = 14,
      GETDBNAME = 15
   };
inline void operator = (ECOA::int16 i) { value = i; }
inline operator ECOA::int16() const { return value; }
};

/* Enum type */
struct OpenFlag
{
   ECOA::int16 value;
   enum EnumValues {
      READER = 0,
      WRITER = 1,
      WRCREAT = 2,
      NEWDB = 3,
      FAST = 16,
      SYNC = 32,
      NOLOCK = 64,
      NOMMAP = 128,
      CLOEXEC = 256
   };
inline void operator = (ECOA::int16 i) { value = i; }
inline operator ECOA::int16() const { return value; }
};

/* Simple type, min is -2147483647, max is 2147483647 */
typedef ECOA::int32 OpenFlags;
static const ECOA::int32 OpenFlags_minRange = -2147483647;
static const ECOA::int32 OpenFlags_maxRange = 2147483647;

/* Enum type */
struct StoreFlag
{
   ECOA::int16 value;
   enum EnumValues {
      INSERT = 0,
      REPLACE = 1
   };
inline void operator = (ECOA::int16 i) { value = i; }
inline operator ECOA::int16() const { return value; }
};

/* Enum type */
struct Permission
{
   ECOA::uint8 value;
   enum EnumValues {
      READ = 4,
      WRITE = 2,
      EXECUTE = 1,
      READEXEC = 5,
      ALL = 7
   };
inline void operator = (ECOA::uint8 i) { value = i; }
inline operator ECOA::uint8() const { return value; }
};

/* Record type */
typedef struct
{
   Permission Owner;
   Permission Group;
   Permission Others;
} Permissions;

/* Array type */
const ECOA::uint32 String_MAXSIZE  = 65536;
typedef struct
{
   ECOA::uint32 current_size;
   ECOA::char8 data[String_MAXSIZE];
} String;

/* Array type */
const ECOA::uint32 NameString_MAXSIZE  = 128;
typedef struct
{
   ECOA::uint32 current_size;
   ECOA::char8 data[NameString_MAXSIZE];
} NameString;

/* Array type */
const ECOA::uint32 Datum_MAXSIZE  = 65536;
typedef struct
{
   ECOA::uint32 current_size;
   ECOA::char8 data[Datum_MAXSIZE];
} Datum;


} /* DBM */

} /* ECOA */

#endif  /* ECOA__DBM_HPP */
