/* File ECOA__FileIO.h */
#if !defined(ECOA__FILEIO_H)
#define ECOA__FILEIO_H

#include "ECOA.h" /* Fundamental types declared for the platform */

#if defined(__cplusplus)
extern "C" {
#endif /* __cplusplus */

/* Array type */
#define ECOA__FileIO__string_MAXSIZE 65536
typedef struct
{
   ECOA__uint32 current_size;
   ECOA__char8 data[ECOA__FileIO__string_MAXSIZE];
} ECOA__FileIO__string;

/* Fixed array type */
#define ECOA__FileIO__FileName_MAXSIZE 256
typedef ECOA__char8 ECOA__FileIO__FileName[ECOA__FileIO__FileName_MAXSIZE];

/* Enum type */
typedef ECOA__int16 ECOA__FileIO__ModeFlag;
#define ECOA__FileIO__ModeFlag_READ (0)
#define ECOA__FileIO__ModeFlag_WRITE (1)
#define ECOA__FileIO__ModeFlag_READWRITE (2)
#define ECOA__FileIO__ModeFlag_APPEND (8)
#define ECOA__FileIO__ModeFlag_NONBLOCK (128)
#define ECOA__FileIO__ModeFlag_CREATE (256)
#define ECOA__FileIO__ModeFlag_TRUNCATE (512)
#define ECOA__FileIO__ModeFlag_EXCLUSIVE (1024)
#define ECOA__FileIO__ModeFlag_NOCTTY (2048)
#define ECOA__FileIO__ModeFlag_PRIVATE (4096)
#define ECOA__FileIO__ModeFlag_LARGEFILE (8192)
#define ECOA__FileIO__ModeFlag_BINARY (16384)

/* Enum type */
typedef ECOA__uint8 ECOA__FileIO__SeekBase;
#define ECOA__FileIO__SeekBase_SEEK_SET (0)
#define ECOA__FileIO__SeekBase_SEEK_CUR (1)
#define ECOA__FileIO__SeekBase_SEEK_END (2)

/* Enum type */
typedef ECOA__uint8 ECOA__FileIO__Permission;
#define ECOA__FileIO__Permission_READ (4)
#define ECOA__FileIO__Permission_WRITE (2)
#define ECOA__FileIO__Permission_EXECUTE (1)
#define ECOA__FileIO__Permission_READEXEC (5)
#define ECOA__FileIO__Permission_ALL (7)

/* Enum type */
typedef ECOA__int32 ECOA__FileIO__EntryType;
#define ECOA__FileIO__EntryType_NONE (0)
#define ECOA__FileIO__EntryType_CHARACTER (131072)
#define ECOA__FileIO__EntryType_DIRECTORY (262144)
#define ECOA__FileIO__EntryType_BLOCK (393216)
#define ECOA__FileIO__EntryType_REGULAR (1048576)
#define ECOA__FileIO__EntryType_LINK (1179648)
#define ECOA__FileIO__EntryType_OTHER (8912896)

/* Record type */
typedef struct
{
   ECOA__FileIO__Permission Owner;
   ECOA__FileIO__Permission Group;
   ECOA__FileIO__Permission Others;
} ECOA__FileIO__Permissions;

/* Record type */
typedef struct
{
   ECOA__FileIO__FileName FileName;
   ECOA__FileIO__ModeFlag Access;
   ECOA__FileIO__Permissions CreatePermissions;
} ECOA__FileIO__FileSpec;

/* Record type */
typedef struct
{
   ECOA__FileIO__FileName Name;
   ECOA__FileIO__EntryType EntryType;
} ECOA__FileIO__DirEntry;


#if defined(__cplusplus)
}
#endif /* __cplusplus */

#endif  /* ECOA__FILEIO_H */
