/* File ECOA__FileIO.hpp */

#if !defined(ECOA__FILEIO_HPP)
#define ECOA__FILEIO_HPP

#include "ECOA.hpp" /* Fundamental types declared for the platform */

namespace ECOA {

namespace FileIO {


/* Array type */
const ECOA::uint32 string_MAXSIZE  = 65536;
typedef struct
{
   ECOA::uint32 current_size;
   ECOA::char8 data[string_MAXSIZE];
} string;

/* Fixed array type */
const ECOA::uint32 FileName_MAXSIZE  = 256;
typedef ECOA::char8 FileName[FileName_MAXSIZE];

/* Enum type */
struct ModeFlag
{
   ECOA::int16 value;
   enum EnumValues {
      READ = 0,
      WRITE = 1,
      READWRITE = 2,
      APPEND = 8,
      NONBLOCK = 128,
      CREATE = 256,
      TRUNCATE = 512,
      EXCLUSIVE = 1024,
      NOCTTY = 2048,
      PRIVATE = 4096,
      LARGEFILE = 8192,
      BINARY = 16384
   };
inline void operator = (ECOA::int16 i) { value = i; }
inline operator ECOA::int16() const { return value; }
};

/* Enum type */
struct SeekBase
{
   ECOA::uint8 value;
   enum EnumValues {
      SEEK_SET = 0,
      SEEK_CUR = 1,
      SEEK_END = 2
   };
inline void operator = (ECOA::uint8 i) { value = i; }
inline operator ECOA::uint8() const { return value; }
};

/* Enum type */
struct Permission
{
   ECOA::uint8 value;
   enum EnumValues {
      READ = 4,
      WRITE = 2,
      EXECUTE = 1,
      READEXEC = 5,
      ALL = 7
   };
inline void operator = (ECOA::uint8 i) { value = i; }
inline operator ECOA::uint8() const { return value; }
};

/* Enum type */
struct EntryType
{
   ECOA::int32 value;
   enum EnumValues {
      NONE = 0,
      CHARACTER = 131072,
      DIRECTORY = 262144,
      BLOCK = 393216,
      REGULAR = 1048576,
      LINK = 1179648,
      OTHER = 8912896
   };
inline void operator = (ECOA::int32 i) { value = i; }
inline operator ECOA::int32() const { return value; }
};

/* Record type */
typedef struct
{
   Permission Owner;
   Permission Group;
   Permission Others;
} Permissions;

/* Record type */
typedef struct
{
   FileName FileName;
   ModeFlag Access;
   Permissions CreatePermissions;
} FileSpec;

/* Record type */
typedef struct
{
   FileName Name;
   EntryType EntryType;
} DirEntry;


} /* FileIO */

} /* ECOA */

#endif  /* ECOA__FILEIO_HPP */
