/* File ECOA__SQL.h */
#if !defined(ECOA__SQL_H)
#define ECOA__SQL_H

#include "ECOA.h" /* Fundamental types declared for the platform */

#if defined(__cplusplus)
extern "C" {
#endif /* __cplusplus */

/* Enum type */
typedef ECOA__uint8 ECOA__SQL__DataType;
#define ECOA__SQL__DataType_CHAR (0)
#define ECOA__SQL__DataType_VARCHAR (1)
#define ECOA__SQL__DataType_NUMERIC (2)
#define ECOA__SQL__DataType_DECIMAL (3)
#define ECOA__SQL__DataType_INTEGER (4)
#define ECOA__SQL__DataType_SMALLINT (5)
#define ECOA__SQL__DataType_FLOAT (6)
#define ECOA__SQL__DataType_REAL (7)
#define ECOA__SQL__DataType_DOUBLE (8)
#define ECOA__SQL__DataType_DATE (9)
#define ECOA__SQL__DataType_TIME (10)
#define ECOA__SQL__DataType_TIMESTAMP (11)
#define ECOA__SQL__DataType_CLOB (12)
#define ECOA__SQL__DataType_BLOB (13)

/* Enum type */
typedef ECOA__uint8 ECOA__SQL__FileType;
#define ECOA__SQL__FileType_SQL (0)
#define ECOA__SQL__FileType_XML (1)

/* Enum type */
typedef ECOA__uint8 ECOA__SQL__Error;
#define ECOA__SQL__Error_NO_ERROR (0)
#define ECOA__SQL__Error_NOT_CONNECTED (1)
#define ECOA__SQL__Error_END_OF_RESULTS (2)
#define ECOA__SQL__Error_UNEXPECTED_MESSAGE (3)
#define ECOA__SQL__Error_CLIENT_NOT_VALID (4)
#define ECOA__SQL__Error_SQL_ERROR (5)
#define ECOA__SQL__Error_SQL_NO_DATA (6)
#define ECOA__SQL__Error_SQL_NEED_DATA (7)
#define ECOA__SQL__Error_SQL_NOT_READY (8)

/* Fixed array type */
#define ECOA__SQL__StringZ_MAXSIZE 4096
typedef ECOA__char8 ECOA__SQL__StringZ[ECOA__SQL__StringZ_MAXSIZE];

/* Array type */
#define ECOA__SQL__String_MAXSIZE 4096
typedef struct
{
   ECOA__uint32 current_size;
   ECOA__char8 data[ECOA__SQL__String_MAXSIZE];
} ECOA__SQL__String;

/* Array type */
#define ECOA__SQL__NameString_MAXSIZE 128
typedef struct
{
   ECOA__uint32 current_size;
   ECOA__char8 data[ECOA__SQL__NameString_MAXSIZE];
} ECOA__SQL__NameString;

/* Fixed array type */
#define ECOA__SQL__CLOB_MAXSIZE 1024
typedef ECOA__char8 ECOA__SQL__CLOB[ECOA__SQL__CLOB_MAXSIZE];

/* Fixed array type */
#define ECOA__SQL__BLOB_MAXSIZE 65536
typedef ECOA__byte ECOA__SQL__BLOB[ECOA__SQL__BLOB_MAXSIZE];

/* Simple type, min is 0, max is 18446744073709551615 */
typedef ECOA__uint64 ECOA__SQL__ConnectionId;
#define ECOA__SQL__ConnectionId_minRange (0)
#define ECOA__SQL__ConnectionId_maxRange (18446744073709551615)

/* Simple type, min is 0, max is 18446744073709551615 */
typedef ECOA__uint64 ECOA__SQL__ResultSetId;
#define ECOA__SQL__ResultSetId_minRange (0)
#define ECOA__SQL__ResultSetId_maxRange (18446744073709551615)

/* Fixed array type */
#define ECOA__SQL__Buffer_MAXSIZE 65536
typedef ECOA__byte ECOA__SQL__Buffer[ECOA__SQL__Buffer_MAXSIZE];

/* Record type */
typedef struct
{
   ECOA__SQL__DataType dataType;
   ECOA__uint32 dataSize;
   ECOA__SQL__Buffer value;
} ECOA__SQL__ColumnValue;

/* Record type */
typedef struct
{
   ECOA__SQL__NameString name;
   ECOA__SQL__DataType dataType;
   ECOA__uint32 columnSize;
   ECOA__uint16 decimalDigits;
   ECOA__boolean8 nullable;
} ECOA__SQL__ColumnInfo;

/* Enum type */
typedef ECOA__uint8 ECOA__SQL__ScrollOrntn;
#define ECOA__SQL__ScrollOrntn_NEXT (0)
#define ECOA__SQL__ScrollOrntn_PRIOR (1)
#define ECOA__SQL__ScrollOrntn_FIRST (2)
#define ECOA__SQL__ScrollOrntn_LAST (3)
#define ECOA__SQL__ScrollOrntn_ABSOLUTE (4)
#define ECOA__SQL__ScrollOrntn_RELATIVE (5)
#define ECOA__SQL__ScrollOrntn_BOOKMARK (6)

/* Record type */
typedef struct
{
   ECOA__SQL__ScrollOrntn orientation;
   ECOA__int64 offset;
} ECOA__SQL__ScrollSet;

/* Array type */
#define ECOA__SQL__IntegerList_MAXSIZE 128
typedef struct
{
   ECOA__uint32 current_size;
   ECOA__int32 data[ECOA__SQL__IntegerList_MAXSIZE];
} ECOA__SQL__IntegerList;

/* Array type */
#define ECOA__SQL__UpdateTableList_MAXSIZE 32
typedef struct
{
   ECOA__uint32 current_size;
   ECOA__SQL__NameString data[ECOA__SQL__UpdateTableList_MAXSIZE];
} ECOA__SQL__UpdateTableList;

/* Array type */
#define ECOA__SQL__TableNameList_MAXSIZE 32
typedef struct
{
   ECOA__uint32 current_size;
   ECOA__SQL__NameString data[ECOA__SQL__TableNameList_MAXSIZE];
} ECOA__SQL__TableNameList;

/* Fixed array type */
#define ECOA__SQL__SQLStatusCode_MAXSIZE 5
typedef ECOA__char8 ECOA__SQL__SQLStatusCode[ECOA__SQL__SQLStatusCode_MAXSIZE];

/* Record type */
typedef struct
{
   ECOA__SQL__SQLStatusCode SQLState;
   ECOA__int64 NativeError;
   ECOA__SQL__Buffer MessageText;
   ECOA__int16 MessageLength;
} ECOA__SQL__ResultStatus;


#if defined(__cplusplus)
}
#endif /* __cplusplus */

#endif  /* ECOA__SQL_H */
