/* File ECOA__SQL.hpp */

#if !defined(ECOA__SQL_HPP)
#define ECOA__SQL_HPP

#include "ECOA.hpp" /* Fundamental types declared for the platform */

namespace ECOA {

namespace SQL {


/* Enum type */
struct DataType
{
   ECOA::uint8 value;
   enum EnumValues {
      CHAR = 0,
      VARCHAR = 1,
      NUMERIC = 2,
      DECIMAL = 3,
      INTEGER = 4,
      SMALLINT = 5,
      FLOAT = 6,
      REAL = 7,
      DOUBLE = 8,
      DATE = 9,
      TIME = 10,
      TIMESTAMP = 11,
      CLOB = 12,
      BLOB = 13
   };
inline void operator = (ECOA::uint8 i) { value = i; }
inline operator ECOA::uint8() const { return value; }
};

/* Enum type */
struct FileType
{
   ECOA::uint8 value;
   enum EnumValues {
      SQL = 0,
      XML = 1
   };
inline void operator = (ECOA::uint8 i) { value = i; }
inline operator ECOA::uint8() const { return value; }
};

/* Enum type */
struct Error
{
   ECOA::uint8 value;
   enum EnumValues {
      NO_ERROR = 0,
      NOT_CONNECTED = 1,
      END_OF_RESULTS = 2,
      UNEXPECTED_MESSAGE = 3,
      CLIENT_NOT_VALID = 4,
      SQL_ERROR = 5,
      SQL_NO_DATA = 6,
      SQL_NEED_DATA = 7,
      SQL_NOT_READY = 8
   };
inline void operator = (ECOA::uint8 i) { value = i; }
inline operator ECOA::uint8() const { return value; }
};

/* Fixed array type */
const ECOA::uint32 StringZ_MAXSIZE  = 4096;
typedef ECOA::char8 StringZ[StringZ_MAXSIZE];

/* Array type */
const ECOA::uint32 String_MAXSIZE  = 4096;
typedef struct
{
   ECOA::uint32 current_size;
   ECOA::char8 data[String_MAXSIZE];
} String;

/* Array type */
const ECOA::uint32 NameString_MAXSIZE  = 128;
typedef struct
{
   ECOA::uint32 current_size;
   ECOA::char8 data[NameString_MAXSIZE];
} NameString;

/* Fixed array type */
const ECOA::uint32 CLOB_MAXSIZE  = 1024;
typedef ECOA::char8 CLOB[CLOB_MAXSIZE];

/* Fixed array type */
const ECOA::uint32 BLOB_MAXSIZE  = 65536;
typedef ECOA::byte BLOB[BLOB_MAXSIZE];

/* Simple type, min is 0, max is 18446744073709551615 */
typedef ECOA::uint64 ConnectionId;
static const ECOA::uint64 ConnectionId_minRange = 0;
static const ECOA::uint64 ConnectionId_maxRange = 18446744073709551615;

/* Simple type, min is 0, max is 18446744073709551615 */
typedef ECOA::uint64 ResultSetId;
static const ECOA::uint64 ResultSetId_minRange = 0;
static const ECOA::uint64 ResultSetId_maxRange = 18446744073709551615;

/* Fixed array type */
const ECOA::uint32 Buffer_MAXSIZE  = 65536;
typedef ECOA::byte Buffer[Buffer_MAXSIZE];

/* Record type */
typedef struct
{
   DataType dataType;
   ECOA::uint32 dataSize;
   Buffer value;
} ColumnValue;

/* Record type */
typedef struct
{
   NameString name;
   DataType dataType;
   ECOA::uint32 columnSize;
   ECOA::uint16 decimalDigits;
   ECOA::boolean8 nullable;
} ColumnInfo;

/* Enum type */
struct ScrollOrntn
{
   ECOA::uint8 value;
   enum EnumValues {
      NEXT = 0,
      PRIOR = 1,
      FIRST = 2,
      LAST = 3,
      ABSOLUTE = 4,
      RELATIVE = 5,
      BOOKMARK = 6
   };
inline void operator = (ECOA::uint8 i) { value = i; }
inline operator ECOA::uint8() const { return value; }
};

/* Record type */
typedef struct
{
   ScrollOrntn orientation;
   ECOA::int64 offset;
} ScrollSet;

/* Array type */
const ECOA::uint32 IntegerList_MAXSIZE  = 128;
typedef struct
{
   ECOA::uint32 current_size;
   ECOA::int32 data[IntegerList_MAXSIZE];
} IntegerList;

/* Array type */
const ECOA::uint32 UpdateTableList_MAXSIZE  = 32;
typedef struct
{
   ECOA::uint32 current_size;
   NameString data[UpdateTableList_MAXSIZE];
} UpdateTableList;

/* Array type */
const ECOA::uint32 TableNameList_MAXSIZE  = 32;
typedef struct
{
   ECOA::uint32 current_size;
   NameString data[TableNameList_MAXSIZE];
} TableNameList;

/* Fixed array type */
const ECOA::uint32 SQLStatusCode_MAXSIZE  = 5;
typedef ECOA::char8 SQLStatusCode[SQLStatusCode_MAXSIZE];

/* Record type */
typedef struct
{
   SQLStatusCode SQLState;
   ECOA::int64 NativeError;
   Buffer MessageText;
   ECOA::int16 MessageLength;
} ResultStatus;


} /* SQL */

} /* ECOA */

#endif  /* ECOA__SQL_HPP */
