-- File ECOA-DBM.ads

-- Standard ECOA Types
with ECOA;


package ECOA.DBM is

-- Simple type, min is 0, max is 4294967295
subtype ConnectionId is ECOA.Unsigned_32_Type range 0 .. 4294967295;

-- Enum type
type Option is 
   (Option_SETCACHESIZE,
Option_SETFASTMODE,
Option_SETSYNCMODE,
Option_SETCENTFREE,
Option_SETCOALESCEBLKS,
Option_SETMAXMAPSIZE,
Option_SETMMAP,
Option_GETFLAGS,
Option_GETMMAP,
Option_GETCACHESIZE,
Option_GETSYNMODE,
Option_GETCENTFREE,
Option_GETCOALESCEBLKS,
Option_GETMAXMAPSIZE,
Option_GETDBNAME);
for Option use
   (Option_SETCACHESIZE => 1,
Option_SETFASTMODE => 2,
Option_SETSYNCMODE => 3,
Option_SETCENTFREE => 4,
Option_SETCOALESCEBLKS => 5,
Option_SETMAXMAPSIZE => 6,
Option_SETMMAP => 7,
Option_GETFLAGS => 8,
Option_GETMMAP => 9,
Option_GETCACHESIZE => 10,
Option_GETSYNMODE => 11,
Option_GETCENTFREE => 12,
Option_GETCOALESCEBLKS => 13,
Option_GETMAXMAPSIZE => 14,
Option_GETDBNAME => 15);
for Option'Size use ECOA.Signed_16_Type'Size;
for Option'Alignment use ECOA.Signed_16_Type'Alignment;

-- Enum type
type OpenFlag is 
   (OpenFlag_READER,
OpenFlag_WRITER,
OpenFlag_WRCREAT,
OpenFlag_NEWDB,
OpenFlag_FAST,
OpenFlag_SYNC,
OpenFlag_NOLOCK,
OpenFlag_NOMMAP,
OpenFlag_CLOEXEC);
for OpenFlag use
   (OpenFlag_READER => 0,
OpenFlag_WRITER => 1,
OpenFlag_WRCREAT => 2,
OpenFlag_NEWDB => 3,
OpenFlag_FAST => 16,
OpenFlag_SYNC => 32,
OpenFlag_NOLOCK => 64,
OpenFlag_NOMMAP => 128,
OpenFlag_CLOEXEC => 256);
for OpenFlag'Size use ECOA.Signed_16_Type'Size;
for OpenFlag'Alignment use ECOA.Signed_16_Type'Alignment;

-- Simple type, min is -2147483647, max is 2147483647
subtype OpenFlags is ECOA.Signed_32_Type range -2147483647 .. 2147483647;

-- Enum type
type StoreFlag is 
   (StoreFlag_INSERT,
StoreFlag_REPLACE);
for StoreFlag use
   (StoreFlag_INSERT => 0,
StoreFlag_REPLACE => 1);
for StoreFlag'Size use ECOA.Signed_16_Type'Size;
for StoreFlag'Alignment use ECOA.Signed_16_Type'Alignment;

-- Enum type
type Permission is 
   (Permission_READ,
Permission_WRITE,
Permission_EXECUTE,
Permission_READEXEC,
Permission_ALL);
for Permission use
   (Permission_READ => 4,
Permission_WRITE => 2,
Permission_EXECUTE => 1,
Permission_READEXEC => 5,
Permission_ALL => 7);
for Permission'Size use ECOA.Unsigned_8_Type'Size;
for Permission'Alignment use ECOA.Unsigned_8_Type'Alignment;

-- Record type
type Permissions is
   record
      Owner : Permission;
      Group : Permission;
      Others : Permission;
   end record;

-- Array type
type String_Size is range 0..65536;
for String_Size'Size use ECOA.Unsigned_32_Type'Size;
subtype String_Index is String_Size range 0..65536-1;
type String_Data is array
    (String_Index) of ECOA.Character_8_Type;
type String is
   record
      Current_Size : String_Size;
      Data         : String_Data;
   end record;

-- Array type
type NameString_Size is range 0..128;
for NameString_Size'Size use ECOA.Unsigned_32_Type'Size;
subtype NameString_Index is NameString_Size range 0..128-1;
type NameString_Data is array
    (NameString_Index) of ECOA.Character_8_Type;
type NameString is
   record
      Current_Size : NameString_Size;
      Data         : NameString_Data;
   end record;

-- Array type
type Datum_Size is range 0..65536;
for Datum_Size'Size use ECOA.Unsigned_32_Type'Size;
subtype Datum_Index is Datum_Size range 0..65536-1;
type Datum_Data is array
    (Datum_Index) of ECOA.Character_8_Type;
type Datum is
   record
      Current_Size : Datum_Size;
      Data         : Datum_Data;
   end record;


end ECOA.DBM;
