-- File ECOA-FileIO.ads

-- Standard ECOA Types
with ECOA;


package ECOA.FileIO is

-- Array type
type string_Size is range 0..65536;
for string_Size'Size use ECOA.Unsigned_32_Type'Size;
subtype string_Index is string_Size range 0..65536-1;
type string_Data is array
    (string_Index) of ECOA.Character_8_Type;
type string is
   record
      Current_Size : string_Size;
      Data         : string_Data;
   end record;

-- Fixed Array type
type FileName_Index is range 0..256-1;
type FileName is array
    (FileName_Index) of ECOA.Character_8_Type;

-- Enum type
type ModeFlag is 
   (ModeFlag_READ,
ModeFlag_WRITE,
ModeFlag_READWRITE,
ModeFlag_APPEND,
ModeFlag_NONBLOCK,
ModeFlag_CREATE,
ModeFlag_TRUNCATE,
ModeFlag_EXCLUSIVE,
ModeFlag_NOCTTY,
ModeFlag_PRIVATE,
ModeFlag_LARGEFILE,
ModeFlag_BINARY);
for ModeFlag use
   (ModeFlag_READ => 0,
ModeFlag_WRITE => 1,
ModeFlag_READWRITE => 2,
ModeFlag_APPEND => 8,
ModeFlag_NONBLOCK => 128,
ModeFlag_CREATE => 256,
ModeFlag_TRUNCATE => 512,
ModeFlag_EXCLUSIVE => 1024,
ModeFlag_NOCTTY => 2048,
ModeFlag_PRIVATE => 4096,
ModeFlag_LARGEFILE => 8192,
ModeFlag_BINARY => 16384);
for ModeFlag'Size use ECOA.Signed_16_Type'Size;
for ModeFlag'Alignment use ECOA.Signed_16_Type'Alignment;

-- Enum type
type SeekBase is 
   (SeekBase_SEEK_SET,
SeekBase_SEEK_CUR,
SeekBase_SEEK_END);
for SeekBase use
   (SeekBase_SEEK_SET => 0,
SeekBase_SEEK_CUR => 1,
SeekBase_SEEK_END => 2);
for SeekBase'Size use ECOA.Unsigned_8_Type'Size;
for SeekBase'Alignment use ECOA.Unsigned_8_Type'Alignment;

-- Enum type
type Permission is 
   (Permission_READ,
Permission_WRITE,
Permission_EXECUTE,
Permission_READEXEC,
Permission_ALL);
for Permission use
   (Permission_READ => 4,
Permission_WRITE => 2,
Permission_EXECUTE => 1,
Permission_READEXEC => 5,
Permission_ALL => 7);
for Permission'Size use ECOA.Unsigned_8_Type'Size;
for Permission'Alignment use ECOA.Unsigned_8_Type'Alignment;

-- Enum type
type EntryType is 
   (EntryType_NONE,
EntryType_CHARACTER,
EntryType_DIRECTORY,
EntryType_BLOCK,
EntryType_REGULAR,
EntryType_LINK,
EntryType_OTHER);
for EntryType use
   (EntryType_NONE => 0,
EntryType_CHARACTER => 131072,
EntryType_DIRECTORY => 262144,
EntryType_BLOCK => 393216,
EntryType_REGULAR => 1048576,
EntryType_LINK => 1179648,
EntryType_OTHER => 8912896);
for EntryType'Size use ECOA.Signed_32_Type'Size;
for EntryType'Alignment use ECOA.Signed_32_Type'Alignment;

-- Record type
type Permissions is
   record
      Owner : Permission;
      Group : Permission;
      Others : Permission;
   end record;

-- Record type
type FileSpec is
   record
      FileName : FileName;
      Access : ModeFlag;
      CreatePermissions : Permissions;
   end record;

-- Record type
type DirEntry is
   record
      Name : FileName;
      EntryType : EntryType;
   end record;


end ECOA.FileIO;
