-- File ECOA-SQL.ads

-- Standard ECOA Types
with ECOA;


package ECOA.SQL is

-- Enum type
type DataType is 
   (DataType_CHAR,
DataType_VARCHAR,
DataType_NUMERIC,
DataType_DECIMAL,
DataType_INTEGER,
DataType_SMALLINT,
DataType_FLOAT,
DataType_REAL,
DataType_DOUBLE,
DataType_DATE,
DataType_TIME,
DataType_TIMESTAMP,
DataType_CLOB,
DataType_BLOB);
for DataType use
   (DataType_CHAR => 0,
DataType_VARCHAR => 1,
DataType_NUMERIC => 2,
DataType_DECIMAL => 3,
DataType_INTEGER => 4,
DataType_SMALLINT => 5,
DataType_FLOAT => 6,
DataType_REAL => 7,
DataType_DOUBLE => 8,
DataType_DATE => 9,
DataType_TIME => 10,
DataType_TIMESTAMP => 11,
DataType_CLOB => 12,
DataType_BLOB => 13);
for DataType'Size use ECOA.Unsigned_8_Type'Size;
for DataType'Alignment use ECOA.Unsigned_8_Type'Alignment;

-- Enum type
type FileType is 
   (FileType_SQL,
FileType_XML);
for FileType use
   (FileType_SQL => 0,
FileType_XML => 1);
for FileType'Size use ECOA.Unsigned_8_Type'Size;
for FileType'Alignment use ECOA.Unsigned_8_Type'Alignment;

-- Enum type
type Error is 
   (Error_NO_ERROR,
Error_NOT_CONNECTED,
Error_END_OF_RESULTS,
Error_UNEXPECTED_MESSAGE,
Error_CLIENT_NOT_VALID,
Error_SQL_ERROR,
Error_SQL_NO_DATA,
Error_SQL_NEED_DATA,
Error_SQL_NOT_READY);
for Error use
   (Error_NO_ERROR => 0,
Error_NOT_CONNECTED => 1,
Error_END_OF_RESULTS => 2,
Error_UNEXPECTED_MESSAGE => 3,
Error_CLIENT_NOT_VALID => 4,
Error_SQL_ERROR => 5,
Error_SQL_NO_DATA => 6,
Error_SQL_NEED_DATA => 7,
Error_SQL_NOT_READY => 8);
for Error'Size use ECOA.Unsigned_8_Type'Size;
for Error'Alignment use ECOA.Unsigned_8_Type'Alignment;

-- Fixed Array type
type StringZ_Index is range 0..4096-1;
type StringZ is array
    (StringZ_Index) of ECOA.Character_8_Type;

-- Array type
type String_Size is range 0..4096;
for String_Size'Size use ECOA.Unsigned_32_Type'Size;
subtype String_Index is String_Size range 0..4096-1;
type String_Data is array
    (String_Index) of ECOA.Character_8_Type;
type String is
   record
      Current_Size : String_Size;
      Data         : String_Data;
   end record;

-- Array type
type NameString_Size is range 0..128;
for NameString_Size'Size use ECOA.Unsigned_32_Type'Size;
subtype NameString_Index is NameString_Size range 0..128-1;
type NameString_Data is array
    (NameString_Index) of ECOA.Character_8_Type;
type NameString is
   record
      Current_Size : NameString_Size;
      Data         : NameString_Data;
   end record;

-- Fixed Array type
type CLOB_Index is range 0..1024-1;
type CLOB is array
    (CLOB_Index) of ECOA.Character_8_Type;

-- Fixed Array type
type BLOB_Index is range 0..65536-1;
type BLOB is array
    (BLOB_Index) of ECOA.Byte_Type;

-- Simple type, min is 0, max is 18446744073709551615
subtype ConnectionId is ECOA.uint64 range 0 .. 18446744073709551615;

-- Simple type, min is 0, max is 18446744073709551615
subtype ResultSetId is ECOA.uint64 range 0 .. 18446744073709551615;

-- Fixed Array type
type Buffer_Index is range 0..65536-1;
type Buffer is array
    (Buffer_Index) of ECOA.Byte_Type;

-- Record type
type ColumnValue is
   record
      dataType : DataType;
      dataSize : ECOA.Unsigned_32_Type;
      value : Buffer;
   end record;

-- Record type
type ColumnInfo is
   record
      name : NameString;
      dataType : DataType;
      columnSize : ECOA.Unsigned_32_Type;
      decimalDigits : ECOA.Unsigned_16_Type;
      nullable : ECOA.Boolean_8_Type;
   end record;

-- Enum type
type ScrollOrntn is 
   (ScrollOrntn_NEXT,
ScrollOrntn_PRIOR,
ScrollOrntn_FIRST,
ScrollOrntn_LAST,
ScrollOrntn_ABSOLUTE,
ScrollOrntn_RELATIVE,
ScrollOrntn_BOOKMARK);
for ScrollOrntn use
   (ScrollOrntn_NEXT => 0,
ScrollOrntn_PRIOR => 1,
ScrollOrntn_FIRST => 2,
ScrollOrntn_LAST => 3,
ScrollOrntn_ABSOLUTE => 4,
ScrollOrntn_RELATIVE => 5,
ScrollOrntn_BOOKMARK => 6);
for ScrollOrntn'Size use ECOA.Unsigned_8_Type'Size;
for ScrollOrntn'Alignment use ECOA.Unsigned_8_Type'Alignment;

-- Record type
type ScrollSet is
   record
      orientation : ScrollOrntn;
      offset : ECOA.Signed_64_Type;
   end record;

-- Array type
type IntegerList_Size is range 0..128;
for IntegerList_Size'Size use ECOA.Unsigned_32_Type'Size;
subtype IntegerList_Index is IntegerList_Size range 0..128-1;
type IntegerList_Data is array
    (IntegerList_Index) of ECOA.Signed_32_Type;
type IntegerList is
   record
      Current_Size : IntegerList_Size;
      Data         : IntegerList_Data;
   end record;

-- Array type
type UpdateTableList_Size is range 0..32;
for UpdateTableList_Size'Size use ECOA.Unsigned_32_Type'Size;
subtype UpdateTableList_Index is UpdateTableList_Size range 0..32-1;
type UpdateTableList_Data is array
    (UpdateTableList_Index) of NameString;
type UpdateTableList is
   record
      Current_Size : UpdateTableList_Size;
      Data         : UpdateTableList_Data;
   end record;

-- Array type
type TableNameList_Size is range 0..32;
for TableNameList_Size'Size use ECOA.Unsigned_32_Type'Size;
subtype TableNameList_Index is TableNameList_Size range 0..32-1;
type TableNameList_Data is array
    (TableNameList_Index) of NameString;
type TableNameList is
   record
      Current_Size : TableNameList_Size;
      Data         : TableNameList_Data;
   end record;

-- Fixed Array type
type SQLStatusCode_Index is range 0..5-1;
type SQLStatusCode is array
    (SQLStatusCode_Index) of ECOA.Character_8_Type;

-- Record type
type ResultStatus is
   record
      SQLState : SQLStatusCode;
      NativeError : ECOA.Signed_64_Type;
      MessageText : Buffer;
      MessageLength : ECOA.Signed_16_Type;
   end record;


end ECOA.SQL;
