-- File demo1B.ads

-- Standard ECOA Types
with ECOA;


package demo1B is

-- Enum type
type T_Align_State is 
   (T_Align_State_NOT_ALIGNED,
T_Align_State_ALIGNED);
for T_Align_State use
   (T_Align_State_NOT_ALIGNED => 0,
T_Align_State_ALIGNED => 1);
for T_Align_State'Size use ECOA.Unsigned_8_Type'Size;
for T_Align_State'Alignment use ECOA.Unsigned_8_Type'Alignment;

-- Enum type
type T_Readiness_State is 
   (T_Readiness_State_NOT_READY,
T_Readiness_State_READY);
for T_Readiness_State use
   (T_Readiness_State_NOT_READY => 0,
T_Readiness_State_READY => 1);
for T_Readiness_State'Size use ECOA.Unsigned_8_Type'Size;
for T_Readiness_State'Alignment use ECOA.Unsigned_8_Type'Alignment;

-- Enum type
type T_Commanded_Power_State is 
   (T_Commanded_Power_State_OFF,
T_Commanded_Power_State_ON);
for T_Commanded_Power_State use
   (T_Commanded_Power_State_OFF => 0,
T_Commanded_Power_State_ON => 1);
for T_Commanded_Power_State'Size use ECOA.Unsigned_8_Type'Size;
for T_Commanded_Power_State'Alignment use ECOA.Unsigned_8_Type'Alignment;

-- Enum type
type T_Availability is 
   (T_Availability_UNAVAILABLE,
T_Availability_AVAILABLE);
for T_Availability use
   (T_Availability_UNAVAILABLE => 0,
T_Availability_AVAILABLE => 1);
for T_Availability'Size use ECOA.Unsigned_8_Type'Size;
for T_Availability'Alignment use ECOA.Unsigned_8_Type'Alignment;

-- Enum type
type T_Sensor_Mode is 
   (T_Sensor_Mode_PURSUIT,
T_Sensor_Mode_MAP);
for T_Sensor_Mode use
   (T_Sensor_Mode_PURSUIT => 0,
T_Sensor_Mode_MAP => 1);
for T_Sensor_Mode'Size use ECOA.Unsigned_8_Type'Size;
for T_Sensor_Mode'Alignment use ECOA.Unsigned_8_Type'Alignment;

-- Enum type
type T_EO_IR_Mode is 
   (T_EO_IR_Mode_NONE,
T_EO_IR_Mode_TRACKING);
for T_EO_IR_Mode use
   (T_EO_IR_Mode_NONE => 0,
T_EO_IR_Mode_TRACKING => 1);
for T_EO_IR_Mode'Size use ECOA.Unsigned_8_Type'Size;
for T_EO_IR_Mode'Alignment use ECOA.Unsigned_8_Type'Alignment;

-- Enum type
type T_Radar_Mode is 
   (T_Radar_Mode_NONE,
T_Radar_Mode_GMTI,
T_Radar_Mode_SAR);
for T_Radar_Mode use
   (T_Radar_Mode_NONE => 0,
T_Radar_Mode_GMTI => 1,
T_Radar_Mode_SAR => 2);
for T_Radar_Mode'Size use ECOA.Unsigned_8_Type'Size;
for T_Radar_Mode'Alignment use ECOA.Unsigned_8_Type'Alignment;

-- Enum type
type T_SVI is 
   (T_SVI_LOW_RATE,
T_SVI_HIGHT_RATE);
for T_SVI use
   (T_SVI_LOW_RATE => 0,
T_SVI_HIGHT_RATE => 1);
for T_SVI'Size use ECOA.Unsigned_8_Type'Size;
for T_SVI'Alignment use ECOA.Unsigned_8_Type'Alignment;

-- Simple type, min is 0, max is 4294967295
subtype T_Tactical_Item_ID is ECOA.Unsigned_32_Type range 0 .. 4294967295;

-- Simple type, min is -3.402823466e+38, max is 3.402823466e+38, units are radian
subtype T_Angle is ECOA.Float_32_Type range -3.402823466e+38 .. 3.402823466e+38;

-- Simple type, min is -3.402823466e+38, max is 3.402823466e+38, units are metre
subtype T_Length is ECOA.Float_32_Type range -3.402823466e+38 .. 3.402823466e+38;

-- Simple type, min is -3.402823466e+38, max is 3.402823466e+38, units are metre/second
subtype T_Speed is ECOA.Float_32_Type range -3.402823466e+38 .. 3.402823466e+38;

-- Simple type, min is -3.402823466e+38, max is 3.402823466e+38, units are radian/second
subtype T_AngularSpeed is ECOA.Float_32_Type range -3.402823466e+38 .. 3.402823466e+38;

-- Simple type, min is -3.402823466e+38, max is 3.402823466e+38, units are Throttle_Setting
subtype T_SVIPower is ECOA.Float_32_Type range -3.402823466e+38 .. 3.402823466e+38;

-- Simple type, min is -3.402823466e+38, max is 3.402823466e+38, units are squaremetre
subtype T_Surface is ECOA.Float_32_Type range -3.402823466e+38 .. 3.402823466e+38;

-- Record type
type T_2D_Position is
   record
      Latitude : T_Angle;
      Longitude : T_Angle;
   end record;

-- Record type
type T_3D_Position is
   record
      Latitude : T_Angle;
      Longitude : T_Angle;
      Height : T_Length;
   end record;

-- Simple type, min is -9223372036854775807, max is 9223372036854775807, units are nanoseconds
subtype T_Time is ECOA.Signed_64_Type range -9223372036854775807 .. 9223372036854775807;

-- Record type
type T_Waypoint is
   record
      Location : T_3D_Position;
      Time : T_Time;
   end record;

-- Record type
type T_AV_Control is
   record
      Turn_Rate : T_AngularSpeed;
      Climb_Rate : T_Speed;
      Power : T_SVIPower;
   end record;

-- Record type
type T_AV_State is
   record
      Location : T_3D_Position;
      Airspeed : T_Speed;
      Heading : T_Angle;
      Power : T_SVIPower;
   end record;

-- Record type
type T_Target_Position is
   record
      Tactical_Item_ID : T_Tactical_Item_ID;
      Location : T_3D_Position;
      Is_Valid : ECOA.Boolean_8_Type;
   end record;

-- Record type
type T_State_Vector is
   record
      Location : T_3D_Position;
      Air_Speed : T_Speed;
      Heading : T_Angle;
      Is_Valid : ECOA.Boolean_8_Type;
   end record;

-- Record type
type T_Weapon_Status is
   record
      Power_State : T_Commanded_Power_State;
      Align_State : T_Align_State;
      Readiness_State : T_Readiness_State;
   end record;

-- Record type
type T_Weapon_Basket is
   record
      Centre : T_3D_Position;
      Radius : T_Length;
   end record;

-- Record type
type T_Tactical_Item is
   record
      Tactical_Item_ID : T_Tactical_Item_ID;
      Is_Valid : ECOA.Boolean_8_Type;
      Location : T_3D_Position;
      Radial_Speed : T_Speed;
      RCS : T_Surface;
   end record;

-- Array type
type T_Tactical_List_Size is range 0..6;
for T_Tactical_List_Size'Size use ECOA.Unsigned_32_Type'Size;
subtype T_Tactical_List_Index is T_Tactical_List_Size range 0..6-1;
type T_Tactical_List_Data is array
    (T_Tactical_List_Index) of T_Tactical_Item;
type T_Tactical_List is
   record
      Current_Size : T_Tactical_List_Size;
      Data         : T_Tactical_List_Data;
   end record;

-- Array type
type T_ImageBuffer_Size is range 0..262144;
for T_ImageBuffer_Size'Size use ECOA.Unsigned_32_Type'Size;
subtype T_ImageBuffer_Index is T_ImageBuffer_Size range 0..262144-1;
type T_ImageBuffer_Data is array
    (T_ImageBuffer_Index) of ECOA.Unsigned_8_Type;
type T_ImageBuffer is
   record
      Current_Size : T_ImageBuffer_Size;
      Data         : T_ImageBuffer_Data;
   end record;

-- Record type
type T_SAR_Map_Parameters is
   record
      Center_Position : T_3D_Position;
      TopLeftCorner_Position : T_3D_Position;
   end record;

-- Record type
type T_SAR_Map is
   record
      Parameters : T_SAR_Map_Parameters;
      Image : T_ImageBuffer;
   end record;


end demo1B;
