-- File demo1B.ads

-- Standard ECOA Types
with ECOA;


package demo1B is

-- Enum type
type T_Align_State is new ECOA.Unsigned_8_Type;
T_Align_State_NOT_ALIGNED : constant T_Align_State := 0;
T_Align_State_ALIGNED : constant T_Align_State := 1;

-- Enum type
type T_Readiness_State is new ECOA.Unsigned_8_Type;
T_Readiness_State_NOT_READY : constant T_Readiness_State := 0;
T_Readiness_State_READY : constant T_Readiness_State := 1;

-- Enum type
type T_Commanded_Power_State is new ECOA.Unsigned_8_Type;
T_Commanded_Power_State_OFF : constant T_Commanded_Power_State := 0;
T_Commanded_Power_State_ON : constant T_Commanded_Power_State := 1;

-- Enum type
type T_Availability is new ECOA.Unsigned_8_Type;
T_Availability_UNAVAILABLE : constant T_Availability := 0;
T_Availability_AVAILABLE : constant T_Availability := 1;

-- Enum type
type T_Sensor_Mode is new ECOA.Unsigned_8_Type;
T_Sensor_Mode_PURSUIT : constant T_Sensor_Mode := 0;
T_Sensor_Mode_MAP : constant T_Sensor_Mode := 1;

-- Enum type
type T_EO_IR_Mode is new ECOA.Unsigned_8_Type;
T_EO_IR_Mode_NONE : constant T_EO_IR_Mode := 0;
T_EO_IR_Mode_TRACKING : constant T_EO_IR_Mode := 1;

-- Enum type
type T_Radar_Mode is new ECOA.Unsigned_8_Type;
T_Radar_Mode_NONE : constant T_Radar_Mode := 0;
T_Radar_Mode_GMTI : constant T_Radar_Mode := 1;
T_Radar_Mode_SAR : constant T_Radar_Mode := 2;

-- Enum type
type T_SVI is new ECOA.Unsigned_8_Type;
T_SVI_LOW_RATE : constant T_SVI := 0;
T_SVI_HIGHT_RATE : constant T_SVI := 1;

-- Simple type, min is 0, max is 4294967295
type T_Tactical_Item_ID is new ECOA.Unsigned_32_Type range 0 .. 4294967295;

-- Simple type, min is -3.402823466e+38, max is 3.402823466e+38, units are radian
type T_Angle is new ECOA.Float_32_Type range -3.402823466e+38 .. 3.402823466e+38;

-- Simple type, min is -3.402823466e+38, max is 3.402823466e+38, units are metre
type T_Length is new ECOA.Float_32_Type range -3.402823466e+38 .. 3.402823466e+38;

-- Simple type, min is -3.402823466e+38, max is 3.402823466e+38, units are metre/second
type T_Speed is new ECOA.Float_32_Type range -3.402823466e+38 .. 3.402823466e+38;

-- Simple type, min is -3.402823466e+38, max is 3.402823466e+38, units are radian/second
type T_AngularSpeed is new ECOA.Float_32_Type range -3.402823466e+38 .. 3.402823466e+38;

-- Simple type, min is -3.402823466e+38, max is 3.402823466e+38, units are Throttle_Setting
type T_SVIPower is new ECOA.Float_32_Type range -3.402823466e+38 .. 3.402823466e+38;

-- Simple type, min is -3.402823466e+38, max is 3.402823466e+38, units are squaremetre
type T_Surface is new ECOA.Float_32_Type range -3.402823466e+38 .. 3.402823466e+38;

-- Record type
type T_2D_Position is
   record
      Latitude : T_Angle;
      Longitude : T_Angle;
   end record;

-- Record type
type T_3D_Position is
   record
      Latitude : T_Angle;
      Longitude : T_Angle;
      Height : T_Length;
   end record;

-- Simple type, min is -9223372036854775807, max is 9223372036854775807, units are nanoseconds
type T_Time is new ECOA.Signed_64_Type range -9223372036854775807 .. 9223372036854775807;

-- Record type
type T_Waypoint is
   record
      Location : T_3D_Position;
      Time : T_Time;
   end record;

-- Record type
type T_AV_Control is
   record
      Turn_Rate : T_AngularSpeed;
      Climb_Rate : T_Speed;
      Power : T_SVIPower;
   end record;

-- Record type
type T_AV_State is
   record
      Location : T_3D_Position;
      Airspeed : T_Speed;
      Heading : T_Angle;
      Power : T_SVIPower;
   end record;

-- Record type
type T_Target_Position is
   record
      Tactical_Item_ID : T_Tactical_Item_ID;
      Location : T_3D_Position;
      Is_Valid : ECOA.Boolean_8_Type;
   end record;

-- Record type
type T_State_Vector is
   record
      Location : T_3D_Position;
      Air_Speed : T_Speed;
      Heading : T_Angle;
      Is_Valid : ECOA.Boolean_8_Type;
   end record;

-- Record type
type T_Weapon_Status is
   record
      Power_State : T_Commanded_Power_State;
      Align_State : T_Align_State;
      Readiness_State : T_Readiness_State;
   end record;

-- Record type
type T_Weapon_Basket is
   record
      Centre : T_3D_Position;
      Radius : T_Length;
   end record;

-- Record type
type T_Tactical_Item is
   record
      Tactical_Item_ID : T_Tactical_Item_ID;
      Is_Valid : ECOA.Boolean_8_Type;
      Location : T_3D_Position;
      Radial_Speed : T_Speed;
      RCS : T_Surface;
   end record;

-- Array type
type T_Tactical_List_Size is new ECOA.Unsigned_32_Type range 0..6;
subtype T_Tactical_List_Index is T_Tactical_List_Size range 0..6-1;
type T_Tactical_List_Data is array
    (T_Tactical_List_Index) of T_Tactical_Item;
type T_Tactical_List is
   record
      Current_Size : T_Tactical_List_Size;
      Data         : T_Tactical_List_Data;
   end record;

-- Array type
type T_ImageBuffer_Size is new ECOA.Unsigned_32_Type range 0..262144;
subtype T_ImageBuffer_Index is T_ImageBuffer_Size range 0..262144-1;
type T_ImageBuffer_Data is array
    (T_ImageBuffer_Index) of ECOA.Unsigned_8_Type;
type T_ImageBuffer is
   record
      Current_Size : T_ImageBuffer_Size;
      Data         : T_ImageBuffer_Data;
   end record;

-- Record type
type T_SAR_Map_Parameters is
   record
      Center_Position : T_3D_Position;
      TopLeftCorner_Position : T_3D_Position;
   end record;

-- Record type
type T_SAR_Map is
   record
      Parameters : T_SAR_Map_Parameters;
      Image : T_ImageBuffer;
   end record;


end demo1B;
