/* File demo1B.hpp */

#if !defined(DEMO1B_HPP)
#define DEMO1B_HPP

#include "ECOA.hpp" /* Fundamental types declared for the platform */

namespace demo1B {


/* Enum type */
struct T_Align_State
{
   ECOA::uint8 value;
   enum EnumValues {
      NOT_ALIGNED = 0,
      ALIGNED = 1
   };
inline void operator = (ECOA::uint8 i) { value = i; }
inline operator ECOA::uint8() const { return value; }
};

/* Enum type */
struct T_Readiness_State
{
   ECOA::uint8 value;
   enum EnumValues {
      NOT_READY = 0,
      READY = 1
   };
inline void operator = (ECOA::uint8 i) { value = i; }
inline operator ECOA::uint8() const { return value; }
};

/* Enum type */
struct T_Commanded_Power_State
{
   ECOA::uint8 value;
   enum EnumValues {
      OFF = 0,
      ON = 1
   };
inline void operator = (ECOA::uint8 i) { value = i; }
inline operator ECOA::uint8() const { return value; }
};

/* Enum type */
struct T_Availability
{
   ECOA::uint8 value;
   enum EnumValues {
      UNAVAILABLE = 0,
      AVAILABLE = 1
   };
inline void operator = (ECOA::uint8 i) { value = i; }
inline operator ECOA::uint8() const { return value; }
};

/* Enum type */
struct T_Sensor_Mode
{
   ECOA::uint8 value;
   enum EnumValues {
      PURSUIT = 0,
      MAP = 1
   };
inline void operator = (ECOA::uint8 i) { value = i; }
inline operator ECOA::uint8() const { return value; }
};

/* Enum type */
struct T_EO_IR_Mode
{
   ECOA::uint8 value;
   enum EnumValues {
      NONE = 0,
      TRACKING = 1
   };
inline void operator = (ECOA::uint8 i) { value = i; }
inline operator ECOA::uint8() const { return value; }
};

/* Enum type */
struct T_Radar_Mode
{
   ECOA::uint8 value;
   enum EnumValues {
      NONE = 0,
      GMTI = 1,
      SAR = 2
   };
inline void operator = (ECOA::uint8 i) { value = i; }
inline operator ECOA::uint8() const { return value; }
};

/* Enum type */
struct T_SVI
{
   ECOA::uint8 value;
   enum EnumValues {
      LOW_RATE = 0,
      HIGHT_RATE = 1
   };
inline void operator = (ECOA::uint8 i) { value = i; }
inline operator ECOA::uint8() const { return value; }
};

/* Simple type, min is 0, max is 4294967295 */
typedef ECOA::uint32 T_Tactical_Item_ID;
static const ECOA::uint32 T_Tactical_Item_ID_minRange = 0;
static const ECOA::uint32 T_Tactical_Item_ID_maxRange = 4294967295;

/* Simple type, min is -3.402823466e+38, max is 3.402823466e+38, units are radian */
typedef ECOA::float32 T_Angle;
static const ECOA::float32 T_Angle_minRange = -3.402823466e+38;
static const ECOA::float32 T_Angle_maxRange = 3.402823466e+38;

/* Simple type, min is -3.402823466e+38, max is 3.402823466e+38, units are metre */
typedef ECOA::float32 T_Length;
static const ECOA::float32 T_Length_minRange = -3.402823466e+38;
static const ECOA::float32 T_Length_maxRange = 3.402823466e+38;

/* Simple type, min is -3.402823466e+38, max is 3.402823466e+38, units are metre/second */
typedef ECOA::float32 T_Speed;
static const ECOA::float32 T_Speed_minRange = -3.402823466e+38;
static const ECOA::float32 T_Speed_maxRange = 3.402823466e+38;

/* Simple type, min is -3.402823466e+38, max is 3.402823466e+38, units are radian/second */
typedef ECOA::float32 T_AngularSpeed;
static const ECOA::float32 T_AngularSpeed_minRange = -3.402823466e+38;
static const ECOA::float32 T_AngularSpeed_maxRange = 3.402823466e+38;

/* Simple type, min is -3.402823466e+38, max is 3.402823466e+38, units are Throttle_Setting */
typedef ECOA::float32 T_SVIPower;
static const ECOA::float32 T_SVIPower_minRange = -3.402823466e+38;
static const ECOA::float32 T_SVIPower_maxRange = 3.402823466e+38;

/* Simple type, min is -3.402823466e+38, max is 3.402823466e+38, units are squaremetre */
typedef ECOA::float32 T_Surface;
static const ECOA::float32 T_Surface_minRange = -3.402823466e+38;
static const ECOA::float32 T_Surface_maxRange = 3.402823466e+38;

/* Record type */
typedef struct
{
   T_Angle Latitude;
   T_Angle Longitude;
} T_2D_Position;

/* Record type */
typedef struct
{
   T_Angle Latitude;
   T_Angle Longitude;
   T_Length Height;
} T_3D_Position;

/* Simple type, min is -9223372036854775807, max is 9223372036854775807, units are nanoseconds */
typedef ECOA::int64 T_Time;
static const ECOA::int64 T_Time_minRange = -9223372036854775807;
static const ECOA::int64 T_Time_maxRange = 9223372036854775807;

/* Record type */
typedef struct
{
   T_3D_Position Location;
   T_Time Time;
} T_Waypoint;

/* Record type */
typedef struct
{
   T_AngularSpeed Turn_Rate;
   T_Speed Climb_Rate;
   T_SVIPower Power;
} T_AV_Control;

/* Record type */
typedef struct
{
   T_3D_Position Location;
   T_Speed Airspeed;
   T_Angle Heading;
   T_SVIPower Power;
} T_AV_State;

/* Record type */
typedef struct
{
   T_Tactical_Item_ID Tactical_Item_ID;
   T_3D_Position Location;
   ECOA::boolean8 Is_Valid;
} T_Target_Position;

/* Record type */
typedef struct
{
   T_3D_Position Location;
   T_Speed Air_Speed;
   T_Angle Heading;
   ECOA::boolean8 Is_Valid;
} T_State_Vector;

/* Record type */
typedef struct
{
   T_Commanded_Power_State Power_State;
   T_Align_State Align_State;
   T_Readiness_State Readiness_State;
} T_Weapon_Status;

/* Record type */
typedef struct
{
   T_3D_Position Centre;
   T_Length Radius;
} T_Weapon_Basket;

/* Record type */
typedef struct
{
   T_Tactical_Item_ID Tactical_Item_ID;
   ECOA::boolean8 Is_Valid;
   T_3D_Position Location;
   T_Speed Radial_Speed;
   T_Surface RCS;
} T_Tactical_Item;

/* Array type */
const ECOA::uint32 T_Tactical_List_MAXSIZE  = 6;
typedef struct
{
   ECOA::uint32 current_size;
   T_Tactical_Item data[T_Tactical_List_MAXSIZE];
} T_Tactical_List;

/* Array type */
const ECOA::uint32 T_ImageBuffer_MAXSIZE  = 262144;
typedef struct
{
   ECOA::uint32 current_size;
   ECOA::uint8 data[T_ImageBuffer_MAXSIZE];
} T_ImageBuffer;

/* Record type */
typedef struct
{
   T_3D_Position Center_Position;
   T_3D_Position TopLeftCorner_Position;
} T_SAR_Map_Parameters;

/* Record type */
typedef struct
{
   T_SAR_Map_Parameters Parameters;
   T_ImageBuffer Image;
} T_SAR_Map;


} /* demo1B */

#endif  /* DEMO1B_HPP */
