/*
 * Common__KLV.c
 *
 *  Created on: Feb 22, 2013
 *      Author: FCAS_User
 *      
 *  $Id: CommonKLV.c,v 2.0 2017/08/02 15:38:05 ksappleb Exp $
 */

/*
#include <osl003_apos.h>
#include <osl003_apos_ext.h>
*/

#include "CommonKLV.h"

/* Procedure to get a KLV item at a given index */

BAE__MAI__Mission_System__General__error Common__KLV_Get_Item_At_Index(ECOA__uint8 klv_index,
                                                                       const Common__KLV_Message_Type *KLV_Data,
                                                                       Common__KLV_Type *KLV_Element)
{
  ECOA__uint8 *ptr   = NULL;

  /* Check there are KLV items present */

  if(KLV_Data->KLV_Header.Number_Of_Items == 0) return(BAE__MAI__Mission_System__General__error_ERROR);

  /* Check that the data has enough items to contain this index */

  if(klv_index > (KLV_Data->KLV_Header.Number_Of_Items - 1)) return(BAE__MAI__Mission_System__General__error_ERROR);

  /* Get a pointer to the start of the KLV data at the index offset */

  ptr = (ECOA__uint8 *)KLV_Data->KLV_Elements + *(KLV_Data->KLV_Header.Offsets + klv_index);

  /* Copy the Key at this offset */

  memcpy((void *)&(KLV_Element->KLV_Key), (void *)ptr, sizeof(BAE__MAI__Mission_System__General__KLV_Key));

  /* Increment the pointer */

  ptr += sizeof(BAE__MAI__Mission_System__General__KLV_Key);

  /* Copy the length */

  memcpy((void *)&(KLV_Element->KLV_Length), (void *)ptr, sizeof(ECOA__uint8));

  /* Increment the pointer */

  ptr += sizeof(ECOA__uint8);

  /* Allocate memory for the KLV Value */

  KLV_Element->KLV_Value = (ECOA__uint8 *)malloc(KLV_Element->KLV_Length);

  /* Check the memory was allocated */

  if(KLV_Element->KLV_Value == NULL)  return(BAE__MAI__Mission_System__General__error_ERROR);

  /* Copy the Value */

  memcpy(KLV_Element->KLV_Value, ptr, KLV_Element->KLV_Length);

  /* Return success */

  return(BAE__MAI__Mission_System__General__error_OK);
}

/* Procedure to get the length of a KLV element */

ECOA__uint32 Common__KLV_Get_Element_Size(const Common__KLV_Type *KLV)
{
  /* Check a NULL KLV has not been passed */

  if(KLV == NULL) return(0);

  /* Return the total size of the KLV element */

  return(sizeof(BAE__MAI__Mission_System__General__KLV_Key) + sizeof(ECOA__uint8) + KLV->KLV_Length);
}

/* Procedure to free a KLV message */

void Common__KLV_Free_KLV_Message(Common__KLV_Message_Type **KLV_Message)
{
  /* Ensure a NULL message has not been passed */

  if((KLV_Message == NULL) || (*KLV_Message == NULL)) return;

  /* Free the KLV elements */

  free((*KLV_Message)->KLV_Elements);

  (*KLV_Message)->KLV_Elements = NULL;

  /* Free the message offsets */

  free((*KLV_Message)->KLV_Header.Offsets);

  (*KLV_Message)->KLV_Header.Offsets = NULL;

  /* Free the KLV Message */

  free(*KLV_Message);

  *KLV_Message = NULL;
}

/* Procedure to create a KLV element */

BAE__MAI__Mission_System__General__error Common__KLV_Create_Element(Common__KLV_Message_Type **KLV_Data, ECOA__uint8 Key, ECOA__uint8 Length, const void *Value)
{
  /* Allocate the element */

  Common__KLV_Type Element;

  /* Allocate the element's value */

  Element.KLV_Value = (ECOA__uint8  *)calloc(1, Length);

  /* Check the element's value was allocated */

  if(Element.KLV_Value == NULL) return(BAE__MAI__Mission_System__General__error_ERROR);

  /* Assign the key and length */

  Element.KLV_Key     = Key;
  Element.KLV_Length  = Length;

  /* Copy the value */

  memcpy(Element.KLV_Value, Value, Length);

  /* Add the KLV element to the KLV data */

  BAE__MAI__Mission_System__General__error error = Common__KLV_Add_Item(KLV_Data, &Element);

  /* Free the KLV Value */

  free(Element.KLV_Value);

  /* Finished */

  return(error);
}

/* Procedure to add a KLV item */

BAE__MAI__Mission_System__General__error Common__KLV_Add_Item(Common__KLV_Message_Type **KLV_Data,
                                                              const Common__KLV_Type *KLV)
{
  /* Check a NULL KLV has not been passed */

  if(KLV == NULL) return(BAE__MAI__Mission_System__General__error_ERROR);

  /* Check if the KLV data is unallocated */

  if(*KLV_Data == NULL)
  {
    /* Allocate the KLV Data */

    *KLV_Data = (Common__KLV_Message_Type *)calloc(1, sizeof(Common__KLV_Message_Type));

    /* Check the KLV data has been allocated */

    if(*KLV_Data == NULL) return(BAE__MAI__Mission_System__General__error_ERROR);
  }

  /* Check the new size required of the KLV elements */

  ECOA__uint32 newSize = (*KLV_Data)->KLV_Header.Data_Size + Common__KLV_Get_Element_Size(KLV);

  /* Reallocate the KLV elements */

  (*KLV_Data)->KLV_Elements = (ECOA__uint8 *)realloc((*KLV_Data)->KLV_Elements, newSize);

  /* Check the KLV elements were allocated */

  if((*KLV_Data)->KLV_Elements == NULL)  return(BAE__MAI__Mission_System__General__error_ERROR);

  /* Reallocate the KLV offsets */

  (*KLV_Data)->KLV_Header.Offsets = (ECOA__uint32 *)realloc((*KLV_Data)->KLV_Header.Offsets, ((*KLV_Data)->KLV_Header.Number_Of_Items + 1) * sizeof(ECOA__uint32));

  /* Check the KLV offsets were allocated */

  if((*KLV_Data)->KLV_Header.Offsets == NULL)  return(BAE__MAI__Mission_System__General__error_ERROR);

  /* Assign a pointer to the offset for the new data */

  ECOA__uint8 *ptr = (*KLV_Data)->KLV_Elements + (*KLV_Data)->KLV_Header.Data_Size;

  /* Copy the offset to the header */

  uint32 *ptrOffset = (*KLV_Data)->KLV_Header.Offsets + (*KLV_Data)->KLV_Header.Number_Of_Items;

  *ptrOffset = (uint32)((unsigned long int)ptr - (unsigned long int)(*KLV_Data)->KLV_Elements);

  /* Copy the KLV element */

  memcpy(ptr, &(KLV->KLV_Key), sizeof(KLV->KLV_Key));       ptr += sizeof(KLV->KLV_Key);
  memcpy(ptr, &(KLV->KLV_Length), sizeof(KLV->KLV_Length)); ptr += sizeof(KLV->KLV_Length);
  memcpy(ptr, KLV->KLV_Value, KLV->KLV_Length);

  /* Update the header */

  (*KLV_Data)->KLV_Header.Number_Of_Items++;
  (*KLV_Data)->KLV_Header.Data_Size = newSize;

  return(BAE__MAI__Mission_System__General__error_OK);
}

/* Procedure to perform a memcpy while ensuring type constraints are not broken */

void Common__KLV_Copy_Data(void *dest, void *src, ECOA__uint32 dest_length, ECOA__uint32 src_length)
{
  memcpy(dest, src, (src_length > dest_length ? dest_length : src_length));
}

/* Procedure to convert the common format KLV data expected by the applications to ECOA format */

void Common__Convert_KLV_Common_To_ECOA(const Common__KLV_Message_Type *KLV,
                                        BAE__MAI__Mission_System__General__KLV_Header_Type *header,
                                        BAE__MAI__Mission_System__General__KLV_Data_Array_Type *data)
{
  ECOA__uint32 i = 0;

  /* Copy the number of items and the data size */

  header->Number_Of_Items = KLV->KLV_Header.Number_Of_Items;
  header->Data_Size       = KLV->KLV_Header.Data_Size;

  /* Assign the current_size field */

  data->current_size = header->Number_Of_Items;

  /* Loop through each item */

  for(i = 0 ; i < header->Number_Of_Items ; i++)
  {
    /* Assign the offset for this KLV element */

    header->Offsets[i] = *(KLV->KLV_Header.Offsets + i);

    /* Assign a pointer to the KLV element at this index */

    ECOA__uint8 *ptr = KLV->KLV_Elements + *(KLV->KLV_Header.Offsets + i);

    /* Copy the Key and Length */

    memcpy(&(data->data[i].Key),    ptr, sizeof(BAE__MAI__Mission_System__General__KLV_Key)); ptr += sizeof(ECOA__uint8);
    memcpy(&(data->data[i].Length), ptr, sizeof(ECOA__uint8));                                ptr += sizeof(ECOA__uint8);

    /* Assign the current_size field */

    data->data[i].Value.current_size = data->data[i].Length;

    /* Copy the Value */

    memcpy(data->data[i].Value.data, ptr, data->data[i].Length);
  }

  return;
}

/* Procedure to convert ECOA format KLV data to the common format expected by the applications */

ECOA__boolean8 Common__Convert_KLV_ECOA_To_Common(const BAE__MAI__Mission_System__General__KLV_Header_Type *header,
                                                  const BAE__MAI__Mission_System__General__KLV_Data_Array_Type *data,
                                                  Common__KLV_Message_Type **KLV)
{
  ECOA__uint8 i = 0;

  for(i = 0 ; i < header->Number_Of_Items ; i++)
  {
    Common__KLV_Create_Element(KLV, data->data[i].Key, data->data[i].Length, (void *)data->data[i].Value.data);
  }

  if(*KLV == NULL)  return(ECOA__FALSE);

  return(ECOA__TRUE);
}

/* Procedure to get a key for a parameter string */

ECOA__boolean8 Common__KLV_Get_Key_For_String(char *Name, ECOA__uint8 Length, ECOA__uint8 *key)
{
  if     (strncmp(Name, "FOCAL", Length) == 0) { *key = KLV_KEY_ECOA_SENSOR_C2_FOCAL_LENGTH;            return(ECOA__TRUE); }
  else if(strncmp(Name, "DZIND", Length) == 0) { *key = KLV_KEY_ECOA_SENSOR_C2_DIGITAL_ZOOM_INDEX;      return(ECOA__TRUE); }
  else if(strncmp(Name, "DZRAT", Length) == 0) { *key = KLV_KEY_ECOA_SENSOR_C2_DIGITAL_ZOOM_RATIO;      return(ECOA__TRUE); }
  else if(strncmp(Name, "DZSTP", Length) == 0) { *key = KLV_KEY_ECOA_SENSOR_C2_DIGITAL_ZOOM_STEP;       return(ECOA__TRUE); }
  else if(strncmp(Name, "OZIND", Length) == 0) { *key = KLV_KEY_ECOA_SENSOR_C2_OPTICAL_ZOOM_INDEX;      return(ECOA__TRUE); }
  else if(strncmp(Name, "OZRAT", Length) == 0) { *key = KLV_KEY_ECOA_SENSOR_C2_OPTICAL_ZOOM_RATIO;      return(ECOA__TRUE); }
  else if(strncmp(Name, "OZSTP", Length) == 0) { *key = KLV_KEY_ECOA_SENSOR_C2_OPTICAL_ZOOM_STEP;       return(ECOA__TRUE); }
  else if(strncmp(Name, "FCPOS", Length) == 0) { *key = KLV_KEY_ECOA_SENSOR_C2_FOCUS_POSITION;          return(ECOA__TRUE); }
  else if(strncmp(Name, "FCSTP", Length) == 0) { *key = KLV_KEY_ECOA_SENSOR_C2_FOCUS_STEP;              return(ECOA__TRUE); }
  else if(strncmp(Name, "GOFOC", Length) == 0) { *key = KLV_KEY_ECOA_SENSOR_C2_GEO_FOCUS;               return(ECOA__TRUE); }
  else if(strncmp(Name, "PLRTY", Length) == 0) { *key = KLV_KEY_ECOA_SENSOR_C2_IR_POLARITY;             return(ECOA__TRUE); }
  else if(strncmp(Name, "IRCAL", Length) == 0) { *key = KLV_KEY_ECOA_SENSOR_C2_IR_CALIBRATION;          return(ECOA__TRUE); }
  else if(strncmp(Name, "YPTCH", Length) == 0) { *key = KLV_KEY_ECOA_SENSOR_C2_Y_PITCH;                 return(ECOA__TRUE); }
  else if(strncmp(Name, "ZPTCH", Length) == 0) { *key = KLV_KEY_ECOA_SENSOR_C2_Z_PITCH;                 return(ECOA__TRUE); }
  else if(strncmp(Name, "APTRE", Length) == 0) { *key = KLV_KEY_ECOA_SENSOR_C2_APERTURE;                return(ECOA__TRUE); }
  else if(strncmp(Name, "EXPSR", Length) == 0) { *key = KLV_KEY_ECOA_SENSOR_C2_EXPOSURE_TIME;           return(ECOA__TRUE); }
  else if(strncmp(Name, "QULTY", Length) == 0) { *key = KLV_KEY_ECOA_SENSOR_C2_QUALTIY;                 return(ECOA__TRUE); }
  else if(strncmp(Name, "FCMOD", Length) == 0) { *key = KLV_KEY_ECOA_SENSOR_C2_FOCUS_MODE;              return(ECOA__TRUE); }
  else if(strncmp(Name, "RDMOD", Length) == 0) { *key = KLV_KEY_ECOA_SENSOR_C2_RADAR_MODE;              return(ECOA__TRUE); }
  else if(strncmp(Name, "MODE",  Length) == 0) { *key = KLV_KEY_ECOA_SENSOR_C2_RADAR_MODE;              return(ECOA__TRUE); }
  else if(strncmp(Name, "RDRNG", Length) == 0) { *key = KLV_KEY_ECOA_SENSOR_C2_RADAR_RANGE;             return(ECOA__TRUE); }
  else if(strncmp(Name, "RANGE", Length) == 0) { *key = KLV_KEY_ECOA_SENSOR_C2_RADAR_RANGE;             return(ECOA__TRUE); }
  else if(strncmp(Name, "RDSCW", Length) == 0) { *key = KLV_KEY_ECOA_SENSOR_C2_RADAR_SCAN_WIDTH;        return(ECOA__TRUE); }
  else if(strncmp(Name, "SCANW", Length) == 0) { *key = KLV_KEY_ECOA_SENSOR_C2_RADAR_SCAN_WIDTH;        return(ECOA__TRUE); }
  else if(strncmp(Name, "RDAAZ", Length) == 0) { *key = KLV_KEY_ECOA_SENSOR_C2_RADAR_ANTENNA_AZIMUTH;   return(ECOA__TRUE); }
  else if(strncmp(Name, "RDAEL", Length) == 0) { *key = KLV_KEY_ECOA_SENSOR_C2_RADAR_ANTENNA_ELEVATION; return(ECOA__TRUE); }
  else if(strncmp(Name, "RDSCM", Length) == 0) { *key = KLV_KEY_ECOA_SENSOR_C2_RADAR_SCAN_MODE;         return(ECOA__TRUE); }
  else if(strncmp(Name, "SCANM", Length) == 0) { *key = KLV_KEY_ECOA_SENSOR_C2_RADAR_SCAN_MODE;         return(ECOA__TRUE); }
  else if(strncmp(Name, "RDBRS", Length) == 0) { *key = KLV_KEY_ECOA_SENSOR_C2_RADAR_SCAN_BARS;         return(ECOA__TRUE); }
  else if(strncmp(Name, "IRCUT", Length) == 0) { *key = KLV_KEY_ECOA_SENSOR_C2_IR_CUT_FILTER;           return(ECOA__TRUE); }
  else if(strncmp(Name, "SHSPD", Length) == 0) { *key = KLV_KEY_ECOA_SENSOR_C2_SHUTTER_SPEED;           return(ECOA__TRUE); }
  else if(strncmp(Name, "NTRKS", Length) == 0) { *key = KLV_KEY_ECOA_TRACK_DATA_NUMBER_OF_TRACKS;       return(ECOA__TRUE); }

  return(ECOA__FALSE);
}

/* Procedure to scale a Sensor C2 - Point */

ECOA__boolean8 Common__KLV_Scale_Sensor_C2_Point(const ECOA__double64 *Latitude, const ECOA__double64 *Longitude, const ECOA__double64 *Altitude, ECOA__uint8 *Length, ECOA__uint8 **Value)
{
  /* Check that a NULL value has not been supplied */

  if((Latitude  == NULL) ||
     (Longitude == NULL) ||
     (Altitude  == NULL) ||
     (Length    == NULL) ||
     (Value     == NULL))    return(ECOA__FALSE);

  /* Set the length and allocate the value */

  if((*Value = (ECOA__uint8 *)calloc(1, *Length = 10)) == NULL)  return(ECOA__FALSE);

  /* Scale the supplied parameters */

  *(ECOA__int32 *) (*Value + 0) |= (ECOA__int32) (*Latitude  * KLV_LSB_ECOA_32_BIT_ANGLE);
  *(ECOA__int32 *) (*Value + 4) |= (ECOA__int32) (*Longitude * KLV_LSB_ECOA_32_BIT_ANGLE);
  *(ECOA__uint16 *)(*Value + 8) |= (ECOA__uint16)(*Altitude  * KLV_LSB_ECOA_ALTITUDE);

  /* Finished */

  return(ECOA__TRUE);
}

/* Procedure to de-scale a Sensor C2 - Point */

ECOA__boolean8 Common__KLV_Descale_Sensor_C2_Point(const ECOA__uint8 *Value, ECOA__double64 *Latitude, ECOA__double64 *Longitude, ECOA__double64 *Altitude)
{
  /* Check that a NULL value has not been supplied */

  if((Latitude  == NULL) ||
     (Longitude == NULL) ||
     (Altitude  == NULL) ||
     (Value     == NULL))    return(ECOA__FALSE);

  /* De-scale the supplied value */

  *Latitude  = (ECOA__double64)(*(ECOA__int32 *)(Value + 0)) / KLV_LSB_ECOA_32_BIT_ANGLE;
  *Longitude = (ECOA__double64)(*(ECOA__int32 *)(Value + 4)) / KLV_LSB_ECOA_32_BIT_ANGLE;
  *Altitude  = (ECOA__double64)(*(ECOA__int16 *)(Value + 8)) / KLV_LSB_ECOA_ALTITUDE;

  /* Finished */

  return(ECOA__TRUE);
}

/* Procedure to scale a Sensor C2 - Sensor Angle */

ECOA__boolean8 Common__KLV_Scale_Sensor_C2_Sensor_Angle(const ECOA__double64 *Yaw, const ECOA__double64 *Pitch, const ECOA__double64 *Roll, ECOA__uint8 *Length, ECOA__uint8 **Value)
{
  /* Check that a NULL value has not been supplied */

  if((Yaw     == NULL) ||
     (Pitch   == NULL) ||
     (Roll    == NULL) ||
     (Length  == NULL) ||
     (Value   == NULL))    return(ECOA__FALSE);

  /* Set the length and allocate the value */

  if((*Value = (ECOA__uint8 *)calloc(1, *Length = 6)) == NULL)  return(ECOA__FALSE);

  /* Scale the supplied parameters */

  *(ECOA__int16 *)(*Value + 0) |= (ECOA__int16)(*Yaw   * KLV_LSB_ECOA_16_BIT_ANGLE);
  *(ECOA__int16 *)(*Value + 2) |= (ECOA__int16)(*Pitch * KLV_LSB_ECOA_16_BIT_ANGLE);
  *(ECOA__int16 *)(*Value + 4) |= (ECOA__int16)(*Roll  * KLV_LSB_ECOA_16_BIT_ANGLE);

  /* Finished */

  return(ECOA__TRUE);
}

/* Procedure to de-scale a Sensor C2 - Sensor Angle */

ECOA__boolean8 Common__KLV_Descale_Sensor_C2_Sensor_Angle(const ECOA__uint8 *Value, ECOA__double64 *Yaw, ECOA__double64 *Pitch, ECOA__double64 *Roll)
{
  /* Check that a NULL value has not been supplied */

  if((Yaw   == NULL) ||
     (Pitch == NULL) ||
     (Roll  == NULL) ||
     (Value == NULL))    return(ECOA__FALSE);

  /* De-scale the supplied value */

  *Yaw   = (ECOA__double64)(*(ECOA__int16 *)(Value + 0)) / KLV_LSB_ECOA_16_BIT_ANGLE;
  *Pitch = (ECOA__double64)(*(ECOA__int16 *)(Value + 2)) / KLV_LSB_ECOA_16_BIT_ANGLE;
  *Roll  = (ECOA__double64)(*(ECOA__int16 *)(Value + 4)) / KLV_LSB_ECOA_16_BIT_ANGLE;

  /* Finished */

  return(ECOA__TRUE);
}

/* Procedure to scale a Sensor C2 - Navigation Angle */

ECOA__boolean8 Common__KLV_Scale_Sensor_C2_Navigation_Angle(const ECOA__double64 *Bearing, const ECOA__double64 *Inclination, const ECOA__double64 *Bank, ECOA__uint8 *Length, ECOA__uint8 **Value)
{
  /* Check that a NULL value has not been supplied */

  if((Bearing     == NULL) ||
     (Inclination == NULL) ||
     (Bank        == NULL) ||
     (Length      == NULL) ||
     (Value       == NULL))    return(ECOA__FALSE);

  /* Set the length and allocate the value */

  if((*Value = (ECOA__uint8 *)calloc(1, *Length = 6)) == NULL)  return(ECOA__FALSE);

  /* Scale the supplied parameters */

  *(ECOA__int16 *)(*Value + 0) |= (ECOA__int16)(*Bearing     * KLV_LSB_ECOA_16_BIT_ANGLE);
  *(ECOA__int16 *)(*Value + 2) |= (ECOA__int16)(*Inclination * KLV_LSB_ECOA_16_BIT_ANGLE);
  *(ECOA__int16 *)(*Value + 4) |= (ECOA__int16)(*Bank        * KLV_LSB_ECOA_16_BIT_ANGLE);

  /* Finished */

  return(ECOA__TRUE);
}

/* Procedure to de-scale a Sensor C2 - Navigation Angle */

ECOA__boolean8 Common__KLV_Descale_Sensor_Navigation_Angle(const ECOA__uint8 *Value, ECOA__double64 *Bearing, ECOA__double64 *Inclination, ECOA__double64 *Bank)
{
  /* Check that a NULL value has not been supplied */

  if((Bearing     == NULL) ||
     (Inclination == NULL) ||
     (Bank        == NULL) ||
     (Value       == NULL))    return(ECOA__FALSE);

  /* De-scale the supplied value */

  *Bearing      = (ECOA__double64)(*(ECOA__int16 *)(Value + 0)) / KLV_LSB_ECOA_16_BIT_ANGLE;
  *Inclination  = (ECOA__double64)(*(ECOA__int16 *)(Value + 2)) / KLV_LSB_ECOA_16_BIT_ANGLE;
  *Bank         = (ECOA__double64)(*(ECOA__int16 *)(Value + 4)) / KLV_LSB_ECOA_16_BIT_ANGLE;

  /* Finished */

  return(ECOA__TRUE);
}

/* Procedure to scale a Sensor C2 - Rate */

ECOA__boolean8 Common__KLV_Scale_Sensor_C2_Rate(const ECOA__double64 *Yaw, const ECOA__double64 *Pitch, const ECOA__double64 *Roll, ECOA__uint8 *Length, ECOA__uint8 **Value)
{
  /* Check that a NULL value has not been supplied */

  if((Yaw     == NULL) ||
     (Pitch   == NULL) ||
     (Roll    == NULL) ||
     (Length  == NULL) ||
     (Value   == NULL))    return(ECOA__FALSE);

  /* Set the length and allocate the value */

  if((*Value = (ECOA__uint8 *)calloc(1, *Length = 6)) == NULL)  return(ECOA__FALSE);

  /* Scale the supplied parameters */

  *(ECOA__int16 *)(*Value + 0) |= (ECOA__int16)(*Yaw   * KLV_LSB_ECOA_16_BIT_ANGLE);
  *(ECOA__int16 *)(*Value + 2) |= (ECOA__int16)(*Pitch * KLV_LSB_ECOA_16_BIT_ANGLE);
  *(ECOA__int16 *)(*Value + 4) |= (ECOA__int16)(*Roll  * KLV_LSB_ECOA_16_BIT_ANGLE);

  /* Finished */

  return(ECOA__TRUE);
}

/* Procedure to de-scale a Sensor C2 - Rate */

ECOA__boolean8 Common__KLV_Descale_Sensor_C2_Rate(const ECOA__uint8 *Value, ECOA__double64 *Yaw, ECOA__double64 *Pitch, ECOA__double64 *Roll)
{
  /* Check that a NULL value has not been supplied */

  if((Yaw   == NULL) ||
     (Pitch == NULL) ||
     (Roll  == NULL) ||
     (Value == NULL))    return(ECOA__FALSE);

  /* De-scale the supplied value */

  *Yaw   = (ECOA__double64)(*(ECOA__int16 *)(Value + 0)) / KLV_LSB_ECOA_16_BIT_ANGLE;
  *Pitch = (ECOA__double64)(*(ECOA__int16 *)(Value + 2)) / KLV_LSB_ECOA_16_BIT_ANGLE;
  *Roll  = (ECOA__double64)(*(ECOA__int16 *)(Value + 4)) / KLV_LSB_ECOA_16_BIT_ANGLE;

  /* Finished */

  return(ECOA__TRUE);
}

/* Procedure to scale a Sensor C2 - Focal Length */

ECOA__boolean8 Common__KLV_Scale_Sensor_C2_Focal_Length(const ECOA__float32 *Focal_Length, ECOA__uint8 *Length, ECOA__uint8 **Value)
{
  /* Check that a NULL value has not been supplied */

  if((Focal_Length  == NULL) ||
     (Value         == NULL))    return(ECOA__FALSE);

  /* Set the length and allocate the value */

  if((*Value = (ECOA__uint8 *)calloc(1, *Length = 4)) == NULL)  return(ECOA__FALSE);

  /* Scale the supplied parameters */

  *(ECOA__uint32 *)(*Value) |= (ECOA__uint32)round(*Focal_Length * KLV_LSB_ECOA_FOCAL_LENGTH);

  /* Finished */

  return(ECOA__TRUE);
}

/* Procedure to de-scale a Sensor C2 - Focal Length */

ECOA__boolean8 Common__KLV_Descale_Sensor_C2_Focal_Length(const ECOA__uint8 *Value, ECOA__float32 *Focal_Length)
{
  /* Check that a NULL value has not been supplied */

  if((Focal_Length  == NULL) ||
     (Value         == NULL))    return(ECOA__FALSE);

  /* De-scale the supplied value */

  *Focal_Length = (ECOA__float32)round(*(ECOA__uint32 *)(Value)) / KLV_LSB_ECOA_FOCAL_LENGTH;

  /* Finished */

  return(ECOA__TRUE);
}

/* Procedure to scale a Sensor C2 - Pitch */

ECOA__boolean8 Common__KLV_Scale_Sensor_C2_Pitch(const ECOA__float32 *Pitch, ECOA__uint8 *Length, ECOA__uint8 **Value)
{
  /* Check that a NULL value has not been supplied */

  if((Pitch == NULL) ||
     (Value == NULL))    return(ECOA__FALSE);

  /* Set the length and allocate the value */

  if((*Value = (ECOA__uint8 *)calloc(1, *Length = 2)) == NULL)  return(ECOA__FALSE);

  /* Scale the supplied parameters */

  *(ECOA__uint16 *)(*Value) |= (ECOA__uint16)round(*Pitch * KLV_LSB_ECOA_PITCH);

  /* Finished */

  return(ECOA__TRUE);
}

/* Procedure to de-scale a Sensor C2 - Pitch */

ECOA__boolean8 Common__KLV_Descale_Sensor_C2_Pitch(const ECOA__uint8 *Value, ECOA__float32 *Pitch)
{
  /* Check that a NULL value has not been supplied */

  if((Pitch == NULL) ||
     (Value == NULL))    return(ECOA__FALSE);

  /* De-scale the supplied value */

  *Pitch = (ECOA__float32)round(*(ECOA__uint16 *)(Value)) / KLV_LSB_ECOA_PITCH;

  /* Finished */

  return(ECOA__TRUE);
}

/* Procedure to scale a Sensor C2 - Aperture */

ECOA__boolean8 Common__KLV_Scale_Sensor_C2_Aperture(const ECOA__float32 *Aperture, ECOA__uint8 *Length, ECOA__uint8 **Value)
{
  /* Check that a NULL value has not been supplied */

  if((Aperture == NULL) ||
     (Value    == NULL))    return(ECOA__FALSE);

  /* Set the length and allocate the value */

  if((*Value = (ECOA__uint8 *)calloc(1, *Length = 2)) == NULL)  return(ECOA__FALSE);

  /* Scale the supplied parameters */

  *(ECOA__uint16 *)(*Value) |= (ECOA__uint16)round(*Aperture * KLV_LSB_ECOA_APERTURE);

  /* Finished */

  return(ECOA__TRUE);
}

/* Procedure to de-scale a Sensor C2 - Aperture */

ECOA__boolean8 Common__KLV_Descale_Sensor_C2_Aperture(const ECOA__uint8 *Value, ECOA__float32 *Aperture)
{
  /* Check that a NULL value has not been supplied */

  if((Aperture == NULL) ||
     (Value    == NULL))    return(ECOA__FALSE);

  /* De-scale the supplied value */

  *Aperture = (ECOA__float32)round(*(ECOA__uint16 *)(Value)) / KLV_LSB_ECOA_APERTURE;

  /* Finished */

  return(ECOA__TRUE);
}

/* Procedure to scale a Sensor C2 - Exposure */

ECOA__boolean8 Common__KLV_Scale_Sensor_C2_Exposure(const ECOA__float32 *Exposure, ECOA__uint8 *Length, ECOA__uint8 **Value)
{
  /* Check that a NULL value has not been supplied */

  if((Exposure == NULL) ||
     (Value    == NULL))    return(ECOA__FALSE);

  /* Set the length and allocate the value */

  if((*Value = (ECOA__uint8 *)calloc(1, *Length = 2)) == NULL)  return(ECOA__FALSE);

  /* Scale the supplied parameters */

  *(ECOA__uint16 *)(*Value) |= (ECOA__uint16)round((*Exposure / 1000) * KLV_LSB_ECOA_EXPOSURE_TIME);

  /* Finished */

  return(ECOA__TRUE);
}

/* Procedure to de-scale a Sensor C2 - Exposure */

ECOA__boolean8 Common__KLV_Descale_Sensor_C2_Exposure(const ECOA__uint8 *Value, ECOA__float32 *Exposure)
{
  /* Check that a NULL value has not been supplied */

  if((Exposure == NULL) ||
     (Value    == NULL))    return(ECOA__FALSE);

  /* De-scale the supplied value */

  *Exposure = (ECOA__float32)(round(*(ECOA__uint16 *)(Value)) / KLV_LSB_ECOA_EXPOSURE_TIME * 1000);

  /* Finished */

  return(ECOA__TRUE);
}

/* Procedure to scale a Sensor C2 - Quality */

ECOA__boolean8 Common__KLV_Scale_Sensor_C2_Quality(const ECOA__float32 *Quality, ECOA__uint8 *Length, ECOA__uint8 **Value)
{
  /* Check that a NULL value has not been supplied */

  if((Quality == NULL) ||
     (Value   == NULL))    return(ECOA__FALSE);

  /* Set the length and allocate the value */

  if((*Value = (ECOA__uint8 *)calloc(1, *Length = 2)) == NULL)  return(ECOA__FALSE);

  /* Scale the supplied parameters */

  *(ECOA__uint16 *)(*Value) |= (ECOA__uint16)round(*Quality * KLV_LSB_ECOA_QUALITY);

  /* Finished */

  return(ECOA__TRUE);
}

/* Procedure to de-scale a Sensor C2 - Quality */

ECOA__boolean8 Common__KLV_Descale_Sensor_C2_Quality(const ECOA__uint8 *Value, ECOA__float32 *Quality)
{
  /* Check that a NULL value has not been supplied */

  if((Quality == NULL) ||
     (Value   == NULL))    return(ECOA__FALSE);

  /* De-scale the supplied value */

  *Quality = (ECOA__float32)round(*(ECOA__uint16 *)(Value)) / KLV_LSB_ECOA_QUALITY;

  /* Finished */

  return(ECOA__TRUE);
}

/* Procedure to scale a Sensor C2 - Radar Scan Width */

ECOA__boolean8 Common__KLV_Scale_Sensor_C2_Radar_Scan_Width(const ECOA__float32 *Width, ECOA__uint8 *Length, ECOA__uint8 **Value)
{
  /* Check that a NULL value has not been supplied */

  if((Width == NULL) ||
     (Value == NULL))    return(ECOA__FALSE);

  /* Set the length and allocate the value */

  if((*Value = (ECOA__uint8 *)calloc(1, *Length = 4)) == NULL)  return(ECOA__FALSE);

  /* Scale the supplied parameters */

  *(ECOA__uint32 *)(*Value) |= (ECOA__uint32)round(*Width * KLV_LSB_ECOA_32_BIT_ABSOLUTE_ANGLE);

  /* Finished */

  return(ECOA__TRUE);
}

/* Procedure to de-scale a Sensor C2 - Radar Scan Width */

ECOA__boolean8 Common__KLV_Descale_Sensor_C2_Radar_Scan_Width(const ECOA__uint8 *Value, ECOA__float32 *Width)
{
  /* Check that a NULL value has not been supplied */

  if((Width == NULL) ||
     (Value == NULL))    return(ECOA__FALSE);

  /* De-scale the supplied value */

  *Width = (ECOA__float32)round(*(ECOA__uint32 *)(Value)) / KLV_LSB_ECOA_32_BIT_ABSOLUTE_ANGLE;

  /* Finished */

  return(ECOA__TRUE);
}

/* Procedure to scale a Sensor C2 - Radar Antenna Azimuth */

ECOA__boolean8 Common__KLV_Scale_Sensor_C2_Radar_Antenna_Azimuth(const ECOA__float32 *Azimuth, ECOA__uint8 *Length, ECOA__uint8 **Value)
{
  /* Check that a NULL value has not been supplied */

  if((Azimuth == NULL) ||
     (Value   == NULL))    return(ECOA__FALSE);

  /* Set the length and allocate the value */

  if((*Value = (ECOA__uint8 *)calloc(1, *Length = 2)) == NULL)  return(ECOA__FALSE);

  /* Scale the supplied parameters */

  *(ECOA__int16 *)(*Value) |= (ECOA__int16)round(*Azimuth * KLV_LSB_ECOA_16_BIT_ANGLE);

  /* Finished */

  return(ECOA__TRUE);
}

/* Procedure to de-scale a Sensor C2 - Radar Antenna Azimuth */

ECOA__boolean8 Common__KLV_Descale_Sensor_C2_Radar_Antenna_Azimuth(const ECOA__uint8 *Value, ECOA__float32 *Azimuth)
{
  /* Check that a NULL value has not been supplied */

  if((Azimuth == NULL) ||
     (Value   == NULL))    return(ECOA__FALSE);

  /* De-scale the supplied value */

  *Azimuth = (ECOA__float32)round(*(ECOA__int16 *)(Value)) / KLV_LSB_ECOA_16_BIT_ANGLE;

  /* Finished */

  return(ECOA__TRUE);
}

/* Procedure to scale a Sensor C2 - Radar Antenna Elevation */

ECOA__boolean8 Common__KLV_Scale_Sensor_C2_Radar_Antenna_Elevation(const ECOA__float32 *Elevation, ECOA__uint8 *Length, ECOA__uint8 **Value)
{
  /* Check that a NULL value has not been supplied */

  if((Elevation == NULL) ||
     (Value     == NULL))    return(ECOA__FALSE);

  /* Set the length and allocate the value */

  if((*Value = (ECOA__uint8 *)calloc(1, *Length = 2)) == NULL)  return(ECOA__FALSE);

  /* Scale the supplied parameters */

  *(ECOA__int16 *)(*Value) |= (ECOA__int16)round(*Elevation * KLV_LSB_ECOA_16_BIT_ANGLE);

  /* Finished */

  return(ECOA__TRUE);
}

/* Procedure to de-scale a Sensor C2 - Radar Antenna Elevation */

ECOA__boolean8 Common__KLV_Descale_Sensor_C2_Radar_Antenna_Elevation(const ECOA__uint8 *Value, ECOA__float32 *Elevation)
{
  /* Check that a NULL value has not been supplied */

  if((Elevation == NULL) ||
     (Value     == NULL))    return(ECOA__FALSE);

  /* De-scale the supplied value */

  *Elevation = (ECOA__float32)round(*(ECOA__int16 *)(Value)) / KLV_LSB_ECOA_16_BIT_ANGLE;

  /* Finished */

  return(ECOA__TRUE);
}

/* Procedure to scale a Sensor Data - Sightline Position */

ECOA__boolean8 Common__KLV_Scale_Sensor_Data_Sightline_Position(const ECOA__double64 *Latitude, const ECOA__double64 *Longitude, const ECOA__double64 *Altitude, ECOA__uint8 *Length, ECOA__uint8 **Value)
{
  /* Check that a NULL value has not been supplied */

  if((Latitude  == NULL) ||
     (Longitude == NULL) ||
     (Altitude  == NULL) ||
     (Length    == NULL) ||
     (Value     == NULL))    return(ECOA__FALSE);

  /* Set the length and allocate the value */

  if((*Value = (ECOA__uint8 *)calloc(1, *Length = 10)) == NULL)  return(ECOA__FALSE);

  /* Scale the supplied parameters */

  *(ECOA__int32 *) (*Value + 0) |= (ECOA__int32) (*Latitude  * KLV_LSB_ECOA_32_BIT_ANGLE);
  *(ECOA__int32 *) (*Value + 4) |= (ECOA__int32) (*Longitude * KLV_LSB_ECOA_32_BIT_ANGLE);
  *(ECOA__uint16 *)(*Value + 8) |= (ECOA__uint16)(*Altitude  * KLV_LSB_ECOA_ALTITUDE);

  /* Finished */

  return(ECOA__TRUE);
}

/* Procedure to de-scale a Sensor Data - Sightline Position */

ECOA__boolean8 Common__KLV_Descale_Sensor_Data_Sightline_Position(const ECOA__uint8 *Value, ECOA__double64 *Latitude, ECOA__double64 *Longitude, ECOA__double64 *Altitude)
{
  /* Check that a NULL value has not been supplied */

  if((Latitude  == NULL) ||
     (Longitude == NULL) ||
     (Altitude  == NULL) ||
     (Value     == NULL))    return(ECOA__FALSE);

  /* De-scale the supplied value */

  *Latitude  = (ECOA__double64)(*(ECOA__int32 *) (Value + 0)) / KLV_LSB_ECOA_32_BIT_ANGLE;
  *Longitude = (ECOA__double64)(*(ECOA__int32 *) (Value + 4)) / KLV_LSB_ECOA_32_BIT_ANGLE;
  *Altitude  = (ECOA__double64)(*(ECOA__uint16 *)(Value + 8)) / KLV_LSB_ECOA_ALTITUDE;

  /* Finished */

  return(ECOA__TRUE);
}

/* Procedure to scale a Sensor Data - Sensor Position */

ECOA__boolean8 Common__KLV_Scale_Sensor_Data_Sensor_Position(const ECOA__double64 *Latitude, const ECOA__double64 *Longitude, const ECOA__float32 *Altitude, ECOA__uint8 *Length, ECOA__uint8 **Value)
{
  /* Check that a NULL value has not been supplied */

  if((Latitude  == NULL) ||
     (Longitude == NULL) ||
     (Altitude  == NULL) ||
     (Length    == NULL) ||
     (Value     == NULL))    return(ECOA__FALSE);

  /* Set the length and allocate the value */

  if((*Value = (ECOA__uint8 *)calloc(1, *Length = 10)) == NULL)  return(ECOA__FALSE);

  /* Scale the supplied parameters */

  *(ECOA__int32 *) (*Value + 0) |= (ECOA__int32) (*Latitude  * KLV_LSB_ECOA_32_BIT_ANGLE);
  *(ECOA__int32 *) (*Value + 4) |= (ECOA__int32) (*Longitude * KLV_LSB_ECOA_32_BIT_ANGLE);
  *(ECOA__uint16 *)(*Value + 8) |= (ECOA__uint16)(*Altitude  * KLV_LSB_ECOA_ALTITUDE);

  /* Finished */

  return(ECOA__TRUE);
}

/* Procedure to de-scale a Sensor Data - Sensor Position */

ECOA__boolean8 Common__KLV_Descale_Sensor_Data_Sensor_Position(const ECOA__uint8 *Value, ECOA__double64 *Latitude, ECOA__double64 *Longitude, ECOA__float32 *Altitude)
{
  /* Check that a NULL value has not been supplied */

  if((Latitude  == NULL) ||
     (Longitude == NULL) ||
     (Altitude  == NULL) ||
     (Value     == NULL))    return(ECOA__FALSE);

  /* De-scale the supplied value */

  *Latitude  = (ECOA__double64)(*(ECOA__int32 *) (Value + 0)) / KLV_LSB_ECOA_32_BIT_ANGLE;
  *Longitude = (ECOA__double64)(*(ECOA__int32 *) (Value + 4)) / KLV_LSB_ECOA_32_BIT_ANGLE;
  *Altitude  = (ECOA__float32) (*(ECOA__uint16 *)(Value + 8)) / KLV_LSB_ECOA_ALTITUDE;

  /* Finished */

  return(ECOA__TRUE);
}

/* Procedure to scale a Sensor Data - Sensor Orientation */

ECOA__boolean8 Common__KLV_Scale_Sensor_Data_Sensor_Orientation(const ECOA__float32 *Bearing, const ECOA__float32 *Inclination, const ECOA__float32 *Bank, ECOA__uint8 *Length, ECOA__uint8 **Value)
{
  /* Check that a NULL value has not been supplied */

  if((Bearing     == NULL) ||
     (Inclination == NULL) ||
     (Bank        == NULL) ||
     (Length      == NULL) ||
     (Value       == NULL))    return(ECOA__FALSE);

  /* Set the length and allocate the value */

  if((*Value = (ECOA__uint8 *)calloc(1, *Length = 6)) == NULL)  return(ECOA__FALSE);

  /* Scale the supplied parameters */

  *(ECOA__int16 *)(*Value + 0) |= (ECOA__int16)(*Bearing     * KLV_LSB_ECOA_16_BIT_ANGLE);
  *(ECOA__int16 *)(*Value + 2) |= (ECOA__int16)(*Inclination * KLV_LSB_ECOA_16_BIT_ANGLE);
  *(ECOA__int16 *)(*Value + 4) |= (ECOA__int16)(*Bank        * KLV_LSB_ECOA_16_BIT_ANGLE);

  /* Finished */

  return(ECOA__TRUE);
}

/* Procedure to de-scale a Sensor Data - Sensor Orientation */

ECOA__boolean8 Common__KLV_Descale_Sensor_Data_Sensor_Orientation(const ECOA__uint8 *Value, ECOA__float32 *Bearing, ECOA__float32 *Inclination, ECOA__float32 *Bank)
{
  /* Check that a NULL value has not been supplied */

  if((Bearing     == NULL) ||
     (Inclination == NULL) ||
     (Bank        == NULL) ||
     (Value       == NULL))    return(ECOA__FALSE);

  /* De-scale the supplied value */

  *Bearing      = (ECOA__float32)(*(ECOA__int16 *)(Value + 0)) / KLV_LSB_ECOA_16_BIT_ANGLE;
  *Inclination  = (ECOA__float32)(*(ECOA__int16 *)(Value + 2)) / KLV_LSB_ECOA_16_BIT_ANGLE;
  *Bank         = (ECOA__float32)(*(ECOA__int16 *)(Value + 4)) / KLV_LSB_ECOA_16_BIT_ANGLE;

  /* Finished */

  return(ECOA__TRUE);
}

/* Procedure to scale a Sensor Data - Footprint */

ECOA__boolean8 Common__KLV_Scale_Sensor_Data_Footprint(const footprint_type *Footprint, ECOA__boolean8 Position_Valid, ECOA__boolean8 Altitude_Valid, ECOA__uint8 *Length, ECOA__uint8 **Value)
{
  /* Check that a NULL value has not been supplied */

  if((Footprint == NULL) ||
     (Value     == NULL))    return(ECOA__FALSE);

  /* Check the footprint position is valid */

  if(Position_Valid == ECOA__FALSE)  return(ECOA__FALSE);

  /* Set the length and allocate the value */

  if((*Value = (ECOA__uint8 *)calloc(1, *Length = 40)) == NULL)  return(ECOA__FALSE);

  /* Scale the supplied parameters */

  /* Top Left */

  *(ECOA__int32 *)(*Value + 0) |= (ECOA__int32)(Footprint->top_left.latitude  * KLV_LSB_ECOA_32_BIT_ANGLE);
  *(ECOA__int32 *)(*Value + 4) |= (ECOA__int32)(Footprint->top_left.longitude * KLV_LSB_ECOA_32_BIT_ANGLE);

  if(Altitude_Valid == ECOA__TRUE)  *(ECOA__uint16 *)(*Value + 8) |= (ECOA__uint16)(Footprint->top_left.altitude  * KLV_LSB_ECOA_ALTITUDE);

  /* Top Right */

  *(ECOA__int32 *)(*Value + 10) |= (ECOA__int32)(Footprint->top_right.latitude  * KLV_LSB_ECOA_32_BIT_ANGLE);
  *(ECOA__int32 *)(*Value + 14) |= (ECOA__int32)(Footprint->top_right.longitude * KLV_LSB_ECOA_32_BIT_ANGLE);

  if(Altitude_Valid == ECOA__TRUE)  *(ECOA__uint16 *)(*Value + 18) |= (ECOA__uint16)(Footprint->top_right.altitude  * KLV_LSB_ECOA_ALTITUDE);

  /* Bottom Right */

  *(ECOA__int32 *)(*Value + 20) |= (ECOA__int32)(Footprint->bottom_right.latitude  * KLV_LSB_ECOA_32_BIT_ANGLE);
  *(ECOA__int32 *)(*Value + 24) |= (ECOA__int32)(Footprint->bottom_right.longitude * KLV_LSB_ECOA_32_BIT_ANGLE);

  if(Altitude_Valid == ECOA__TRUE)  *(ECOA__uint16 *)(*Value + 28) |= (ECOA__uint16)(Footprint->bottom_right.altitude  * KLV_LSB_ECOA_ALTITUDE);

  /* Bottom Left */

  *(ECOA__int32 *)(*Value + 30) |= (ECOA__int32)(Footprint->bottom_left.latitude  * KLV_LSB_ECOA_32_BIT_ANGLE);
  *(ECOA__int32 *)(*Value + 34) |= (ECOA__int32)(Footprint->bottom_left.longitude * KLV_LSB_ECOA_32_BIT_ANGLE);

  if(Altitude_Valid == ECOA__TRUE)  *(ECOA__uint16 *)(*Value + 38) |= (ECOA__uint16)(Footprint->bottom_left.altitude  * KLV_LSB_ECOA_ALTITUDE);

  /* Finished */

  return(ECOA__TRUE);
}

/* Procedure to de-scale a Sensor Data - Footprint */

ECOA__boolean8 Common__KLV_Descale_Sensor_Data_Footprint(const ECOA__uint8 *Value, footprint_type *Footprint, ECOA__boolean8 *Position_Valid, ECOA__boolean8 *Altitude_Valid)
{
  /* Check that a NULL value has not been supplied */

  if((Footprint == NULL) ||
     (Value     == NULL))    return(ECOA__FALSE);

  /* De-scale the supplied value */

  /* Top Left */

  Footprint->top_left.latitude  = (ECOA__double64)(*(ECOA__int32 *) (Value + 0)) / KLV_LSB_ECOA_32_BIT_ANGLE;
  Footprint->top_left.longitude = (ECOA__double64)(*(ECOA__int32 *) (Value + 4)) / KLV_LSB_ECOA_32_BIT_ANGLE;
  Footprint->top_left.altitude  = (ECOA__double64)(*(ECOA__uint16 *)(Value + 8)) / KLV_LSB_ECOA_ALTITUDE;

  /* Top Right */

  Footprint->top_right.latitude  = (ECOA__double64)(*(ECOA__int32 *) (Value + 10)) / KLV_LSB_ECOA_32_BIT_ANGLE;
  Footprint->top_right.longitude = (ECOA__double64)(*(ECOA__int32 *) (Value + 14)) / KLV_LSB_ECOA_32_BIT_ANGLE;
  Footprint->top_right.altitude  = (ECOA__double64)(*(ECOA__uint16 *)(Value + 18)) / KLV_LSB_ECOA_ALTITUDE;

  /* Bottom Right */

  Footprint->bottom_right.latitude  = (ECOA__double64)(*(ECOA__int32 *) (Value + 20)) / KLV_LSB_ECOA_32_BIT_ANGLE;
  Footprint->bottom_right.longitude = (ECOA__double64)(*(ECOA__int32 *) (Value + 24)) / KLV_LSB_ECOA_32_BIT_ANGLE;
  Footprint->bottom_right.altitude  = (ECOA__double64)(*(ECOA__uint16 *)(Value + 28)) / KLV_LSB_ECOA_ALTITUDE;

  /* Bottom Left */

  Footprint->bottom_left.latitude  = (ECOA__double64)(*(ECOA__int32 *) (Value + 30)) / KLV_LSB_ECOA_32_BIT_ANGLE;
  Footprint->bottom_left.longitude = (ECOA__double64)(*(ECOA__int32 *) (Value + 34)) / KLV_LSB_ECOA_32_BIT_ANGLE;
  Footprint->bottom_left.altitude  = (ECOA__double64)(*(ECOA__uint16 *)(Value + 38)) / KLV_LSB_ECOA_ALTITUDE;

  /* TODO : Need to decide how to detect if the Altitude was invalid */

  *Position_Valid = 1;
  *Altitude_Valid = 1;

  /* Finished */

  return(ECOA__TRUE);
}

/* Procedure to scale a Track Data - Track Position */

ECOA__boolean8 Common__KLV_Scale_Track_Data_Track_Position(const ECOA__double64 *Latitude, const ECOA__double64 *Longitude, const ECOA__float32 *Altitude, ECOA__uint8 *Length, ECOA__uint8 **Value)

{
  /* Check that a NULL value has not been supplied */

  if((Latitude  == NULL) ||
     (Longitude == NULL) ||
     (Altitude  == NULL) ||
     (Length    == NULL) ||
     (Value     == NULL))    return(ECOA__FALSE);

  /* Set the length and allocate the value */

  if((*Value = (ECOA__uint8 *)calloc(1, *Length = 10)) == NULL)  return(ECOA__FALSE);

  /* Scale the supplied parameters */

  *(ECOA__int32 *) (*Value + 0) |= (ECOA__int32) (*Latitude  * KLV_LSB_ECOA_32_BIT_ANGLE);
  *(ECOA__int32 *) (*Value + 4) |= (ECOA__int32) (*Longitude * KLV_LSB_ECOA_32_BIT_ANGLE);
  *(ECOA__uint16 *)(*Value + 8) |= (ECOA__uint16)(*Altitude  * KLV_LSB_ECOA_ALTITUDE);

  /* Finished */

  return(ECOA__TRUE);
}

/* Procedure to de-scale a Track Data - Track Position */

ECOA__boolean8 Common__KLV_Descale_Track_Data_Track_Position(const ECOA__uint8 *Value, ECOA__double64 *Latitude, ECOA__double64 *Longitude, ECOA__float32 *Altitude)
{
  /* Check that a NULL value has not been supplied */

  if((Latitude  == NULL) ||
     (Longitude == NULL) ||
     (Altitude  == NULL) ||
     (Value     == NULL))    return(ECOA__FALSE);

  /* De-scale the supplied value */

  *Latitude  = (ECOA__double64)(*(ECOA__int32 *) (Value + 0)) / KLV_LSB_ECOA_32_BIT_ANGLE;
  *Longitude = (ECOA__double64)(*(ECOA__int32 *) (Value + 4)) / KLV_LSB_ECOA_32_BIT_ANGLE;
  *Altitude  = (ECOA__float32) (*(ECOA__uint16 *)(Value + 8)) / KLV_LSB_ECOA_ALTITUDE;

  /* Finished */

  return(ECOA__TRUE);
}

/* Procedure to scale a Track Data - Track Polar Position */

ECOA__boolean8 Common__KLV_Scale_Track_Data_Track_Polar_Position(const ECOA__float32 *Bearing, const ECOA__float32 *Elevation, const ECOA__float32 *Range, ECOA__uint8 *Length, ECOA__uint8 **Value)
{
  /* Check that a NULL value has not been supplied */

  if((Bearing   == NULL) ||
     (Elevation == NULL) ||
     (Range     == NULL) ||
     (Length    == NULL) ||
     (Value     == NULL))    return(ECOA__FALSE);

  /* Set the length and allocate the value */

  if((*Value = (ECOA__uint8 *)calloc(1, *Length = 6)) == NULL)  return(ECOA__FALSE);

  /* Scale the supplied parameters */

  *(ECOA__int16 *) (*Value + 0) |= (ECOA__int16) (*Bearing   * KLV_LSB_ECOA_16_BIT_ANGLE);
  *(ECOA__int16 *) (*Value + 2) |= (ECOA__int16) (*Elevation * KLV_LSB_ECOA_16_BIT_ANGLE);
  *(ECOA__uint16 *)(*Value + 4) |= (ECOA__uint16)(*Range     * KLV_LSB_ECOA_RANGE);

  /* Finished */

  return(ECOA__TRUE);
}

/* Procedure to de-scale a Track Data - Track Polar Position */

ECOA__boolean8 Common__KLV_Descale_Track_Data_Track_Polar_Position(const ECOA__uint8 *Value, ECOA__float32 *Bearing, ECOA__float32 *Elevation, ECOA__float32 *Range)
{
  /* Check that a NULL value has not been supplied */

  if((Bearing   == NULL) ||
     (Elevation == NULL) ||
     (Range     == NULL) ||
     (Value     == NULL))    return(ECOA__FALSE);

  /* De-scale the supplied value */

  *Bearing   = (ECOA__float32)(*(ECOA__int16 *) (Value + 0)) / KLV_LSB_ECOA_16_BIT_ANGLE;
  *Elevation = (ECOA__float32)(*(ECOA__int16 *) (Value + 2)) / KLV_LSB_ECOA_16_BIT_ANGLE;
  *Range     = (ECOA__float32)(*(ECOA__uint16 *)(Value + 4)) / KLV_LSB_ECOA_RANGE;

  /* Finished */

  return(ECOA__TRUE);
}

/* Procedure to scale a Track Data - Range Rate */

ECOA__boolean8 Common__KLV_Scale_Track_Data_Range_Rate(const ECOA__float32 *Range_Rate, ECOA__uint8 *Length, ECOA__uint8 **Value)
{
  /* Check that a NULL value has not been supplied */

  if((Range_Rate == NULL) ||
     (Length     == NULL) ||
     (Value      == NULL))    return(ECOA__FALSE);

  /* Set the length and allocate the value */

  if((*Value = (ECOA__uint8 *)calloc(1, *Length = 2)) == NULL)  return(ECOA__FALSE);

  /* Scale the supplied parameters */

  *(ECOA__int16 *)(*Value) |= (ECOA__int16)(*Range_Rate * KLV_LSB_ECOA_RATE);

  /* Finished */

  return(ECOA__TRUE);
}

/* Procedure to de-scale a Track Data - Range Rate */

ECOA__boolean8 Common__KLV_Descale_Track_Data_Range_Rate(const ECOA__uint8 *Value, ECOA__float32 *Range_Rate)
{
  /* Check that a NULL value has not been supplied */

  if((Range_Rate == NULL) ||
     (Value      == NULL))    return(ECOA__FALSE);

  /* De-scale the supplied value */

  *Range_Rate = (ECOA__float32)(*(ECOA__int16 *)Value) / KLV_LSB_ECOA_RATE;

  /* Finished */

  return(ECOA__TRUE);
}

/* Procedure to scale a Track Data - Velocity */

ECOA__boolean8 Common__KLV_Scale_Track_Data_Velocity(const ECOA__float32 *North, const ECOA__float32 *East, const ECOA__float32 *Vertical, ECOA__uint8 *Length, ECOA__uint8 **Value)
{
  /* Check that a NULL value has not been supplied */

  if((North    == NULL) ||
     (East     == NULL) ||
     (Vertical == NULL) ||
     (Length   == NULL) ||
     (Value    == NULL))    return(ECOA__FALSE);

  /* Set the length and allocate the value */

  if((*Value = (ECOA__uint8 *)calloc(1, *Length = 6)) == NULL)  return(ECOA__FALSE);

  /* Scale the supplied parameters */

  *(ECOA__int16 *)(*Value + 0) |= (ECOA__int16)(*North    * KLV_LSB_ECOA_RATE);
  *(ECOA__int16 *)(*Value + 2) |= (ECOA__int16)(*East     * KLV_LSB_ECOA_RATE);
  *(ECOA__int16 *)(*Value + 4) |= (ECOA__int16)(*Vertical * KLV_LSB_ECOA_RATE);

  /* Finished */

  return(ECOA__TRUE);
}

/* Procedure to de-scale a Track Data - Velocity */

ECOA__boolean8 Common__KLV_Descale_Track_Data_Velocity(const ECOA__uint8 *Value, ECOA__float32 *North, ECOA__float32 *East, ECOA__float32 *Vertical)
{
  /* Check that a NULL value has not been supplied */

  if((North    == NULL) ||
     (East     == NULL) ||
     (Vertical == NULL) ||
     (Value    == NULL))    return(ECOA__FALSE);

  /* De-scale the supplied value */

  *North    = (ECOA__float32)(*(ECOA__int16 *)(Value + 0)) / KLV_LSB_ECOA_RATE;
  *East     = (ECOA__float32)(*(ECOA__int16 *)(Value + 2)) / KLV_LSB_ECOA_RATE;
  *Vertical = (ECOA__float32)(*(ECOA__int16 *)(Value + 4)) / KLV_LSB_ECOA_RATE;

  /* Finished */

  return(ECOA__TRUE);
}

/* Procedure to scale a Track Data - Size */

ECOA__boolean8 Common__KLV_Scale_Track_Data_Size(const ECOA__float32 *Size, ECOA__uint8 *Length, ECOA__uint8 **Value)
{
  /* Check that a NULL value has not been supplied */

  if((Size   == NULL) ||
     (Length == NULL) ||
     (Value  == NULL))    return(ECOA__FALSE);

  /* Set the length and allocate the value */

  if((*Value = (ECOA__uint8 *)calloc(1, *Length = 2)) == NULL)  return(ECOA__FALSE);

  /* Scale the supplied parameters */

  *(ECOA__uint16 *)(*Value) |= (ECOA__uint16)(*Size * KLV_LSB_ECOA_SIZE);

  /* Finished */

  return(ECOA__TRUE);
}

/* Procedure to de-scale a Track Data - Size */

ECOA__boolean8 Common__KLV_Descale_Track_Data_Size(const ECOA__uint8 *Value, ECOA__float32 *Size)
{
  /* Check that a NULL value has not been supplied */

  if((Size  == NULL) ||
     (Value == NULL))    return(ECOA__FALSE);

  /* De-scale the supplied value */

  *Size = (ECOA__float32)(*(ECOA__uint16 *)Value) / KLV_LSB_ECOA_SIZE;

  /* Finished */

  return(ECOA__TRUE);
}

/* Procedure to scale a Track Data - Threat Range */

ECOA__boolean8 Common__KLV_Scale_Track_Data_Threat_Range(const ECOA__float32 *Threat_Range, ECOA__uint8 *Length, ECOA__uint8 **Value)
{
  /* Check that a NULL value has not been supplied */

  if((Threat_Range == NULL) ||
     (Length       == NULL) ||
     (Value        == NULL))    return(ECOA__FALSE);

  /* Set the length and allocate the value */

  if((*Value = (ECOA__uint8 *)calloc(1, *Length = 2)) == NULL)  return(ECOA__FALSE);

  /* Scale the supplied parameters */

  *(ECOA__uint16 *)(*Value) |= (ECOA__uint16)(*Threat_Range * KLV_LSB_ECOA_RANGE);

  /* Finished */

  return(ECOA__TRUE);
}

/* Procedure to de-scale a Track Data - Threat Range */

ECOA__boolean8 Common__KLV_Descale_Track_Data_Threat_Range(const ECOA__uint8 *Value, ECOA__float32 *Threat_Range)
{
  /* Check that a NULL value has not been supplied */

  if((Threat_Range == NULL) ||
     (Value        == NULL))    return(ECOA__FALSE);

  /* De-scale the supplied value */

  *Threat_Range = (ECOA__float32)(*(ECOA__uint16 *)Value) / KLV_LSB_ECOA_RANGE;

  /* Finished */

  return(ECOA__TRUE);
}

/* Procedure to scale a Track Data - Covariance */

ECOA__boolean8 Common__KLV_Scale_Track_Data_Covariance(const ECOA__float32 *Covariance, ECOA__uint8 *Length, ECOA__uint8 **Value)
{
  /* Check that a NULL value has not been supplied */

  if((Covariance == NULL) ||
     (Length     == NULL) ||
     (Value      == NULL))    return(ECOA__FALSE);

  /* Set the length and allocate the value */

  if((*Value = (ECOA__uint8 *)calloc(1, *Length = 2)) == NULL)  return(ECOA__FALSE);

  /* Scale the supplied parameters */

  *(ECOA__int16 *)(*Value) |= (ECOA__int16)(*Covariance * KLV_LSB_ECOA_COVARIANCE);

  /* Finished */

  return(ECOA__TRUE);
}

/* Procedure to de-scale a Track Data - Covariance */

ECOA__boolean8 Common__KLV_Descale_Track_Data_Covariance(const ECOA__uint8 *Value, ECOA__float32 *Covariance)
{
  /* Check that a NULL value has not been supplied */

  if((Covariance == NULL) ||
     (Value      == NULL))    return(ECOA__FALSE);

  /* De-scale the supplied value */

  *Covariance = (ECOA__float32)(*(ECOA__int16 *)Value) / KLV_LSB_ECOA_COVARIANCE;

  /* Finished */

  return(ECOA__TRUE);
}

/* Procedure to scale a Sensor Status - Sensor Position */

ECOA__boolean8 Common__KLV_Scale_Sensor_Status_Sensor_Position(const ECOA__double64 *Latitude, const ECOA__double64 *Longitude, const ECOA__double64 *Altitude, ECOA__uint8 *Length, ECOA__uint8 **Value)
{
  /* Check that a NULL value has not been supplied */

  if((Latitude  == NULL) ||
     (Longitude == NULL) ||
     (Altitude  == NULL) ||
     (Length    == NULL) ||
     (Value     == NULL))    return(ECOA__FALSE);

  /* Set the length and allocate the value */

  if((*Value = (ECOA__uint8 *)calloc(1, *Length = 10)) == NULL)  return(ECOA__FALSE);

  /* Scale the supplied parameters */

  *(ECOA__int32 *) (*Value + 0) |= (ECOA__int32) (*Latitude  * KLV_LSB_ECOA_32_BIT_ANGLE);
  *(ECOA__int32 *) (*Value + 4) |= (ECOA__int32) (*Longitude * KLV_LSB_ECOA_32_BIT_ANGLE);
  *(ECOA__uint16 *)(*Value + 8) |= (ECOA__uint16)(*Altitude  * KLV_LSB_ECOA_ALTITUDE);

  /* Finished */

  return(ECOA__TRUE);
}

/* Procedure to de-scale a Sensor Status - Sensor Position */

ECOA__boolean8 Common__KLV_Descale_Sensor_Status_Sensor_Position(const ECOA__uint8 *Value, ECOA__double64 *Latitude, ECOA__double64 *Longitude, ECOA__double64 *Altitude)
{
  /* Check that a NULL value has not been supplied */

  if((Latitude  == NULL) ||
     (Longitude == NULL) ||
     (Altitude  == NULL) ||
     (Value     == NULL))    return(ECOA__FALSE);

  /* De-scale the supplied value */

  *Latitude  = (ECOA__double64)(*(ECOA__int32 *) (Value + 0)) / KLV_LSB_ECOA_32_BIT_ANGLE;
  *Longitude = (ECOA__double64)(*(ECOA__int32 *) (Value + 4)) / KLV_LSB_ECOA_32_BIT_ANGLE;
  *Altitude  = (ECOA__double64) (*(ECOA__uint16 *)(Value + 8)) / KLV_LSB_ECOA_ALTITUDE;

  /* Finished */

  return(ECOA__TRUE);
}

/* Procedure to scale a Sensor Status - Platform Orientation */

ECOA__boolean8 Common__KLV_Scale_Sensor_Status_Platform_Orientation(const ECOA__double64 *Heading, const ECOA__double64 *Inclination, const ECOA__double64 *Bank, ECOA__uint8 *Length, ECOA__uint8 **Value)
{
  /* Check that a NULL value has not been supplied */

  if((Heading     == NULL) ||
     (Inclination == NULL) ||
     (Bank        == NULL) ||
     (Length      == NULL) ||
     (Value       == NULL))    return(ECOA__FALSE);

  /* Set the length and allocate the value */

  if((*Value = (ECOA__uint8 *)calloc(1, *Length = 6)) == NULL)  return(ECOA__FALSE);

  /* Scale the supplied parameters */

  *(ECOA__int16 *)(*Value + 0) |= (ECOA__int16)(*Heading     * KLV_LSB_ECOA_16_BIT_ANGLE);
  *(ECOA__int16 *)(*Value + 2) |= (ECOA__int16)(*Inclination * KLV_LSB_ECOA_16_BIT_ANGLE);
  *(ECOA__int16 *)(*Value + 4) |= (ECOA__int16)(*Bank        * KLV_LSB_ECOA_16_BIT_ANGLE);

  /* Finished */

  return(ECOA__TRUE);
}

/* Procedure to de-scale a Sensor Status - Platform Orientation */

ECOA__boolean8 Common__KLV_Descale_Sensor_Status_Platform_Orientation(const ECOA__uint8 *Value, ECOA__double64 *Heading, ECOA__double64 *Inclination, ECOA__double64 *Bank)
{
  /* Check that a NULL value has not been supplied */

  if((Heading     == NULL) ||
     (Inclination == NULL) ||
     (Bank        == NULL) ||
     (Value       == NULL))    return(ECOA__FALSE);

  /* De-scale the supplied value */

  *Heading      = (ECOA__double64)(*(ECOA__int16 *)(Value + 0)) / KLV_LSB_ECOA_16_BIT_ANGLE;
  *Inclination  = (ECOA__double64)(*(ECOA__int16 *)(Value + 2)) / KLV_LSB_ECOA_16_BIT_ANGLE;
  *Bank         = (ECOA__double64)(*(ECOA__int16 *)(Value + 4)) / KLV_LSB_ECOA_16_BIT_ANGLE;

  /* Finished */

  return(ECOA__TRUE);
}

/* Procedure to scale a Sensor Status - Sensor Orientation */

ECOA__boolean8 Common__KLV_Scale_Sensor_Status_Sensor_Orientation(const ECOA__float32 *Bearing, const ECOA__float32 *Inclination, const ECOA__float32 *Bank, ECOA__uint8 *Length, ECOA__uint8 **Value)
{
  /* Check that a NULL value has not been supplied */

  if((Bearing     == NULL) ||
     (Inclination == NULL) ||
     (Bank        == NULL) ||
     (Length      == NULL) ||
     (Value       == NULL))    return(ECOA__FALSE);

  /* Set the length and allocate the value */

  if((*Value = (ECOA__uint8 *)calloc(1, *Length = 6)) == NULL)  return(ECOA__FALSE);

  /* Scale the supplied parameters */

  *(ECOA__int16 *)(*Value + 0) |= (ECOA__int16)(*Bearing     * KLV_LSB_ECOA_16_BIT_ANGLE);
  *(ECOA__int16 *)(*Value + 2) |= (ECOA__int16)(*Inclination * KLV_LSB_ECOA_16_BIT_ANGLE);
  *(ECOA__int16 *)(*Value + 4) |= (ECOA__int16)(*Bank        * KLV_LSB_ECOA_16_BIT_ANGLE);

  /* Finished */

  return(ECOA__TRUE);
}

/* Procedure to de-scale a Sensor Status - Sensor Orientation */

ECOA__boolean8 Common__KLV_Descale_Sensor_Status_Sensor_Orientation(const ECOA__uint8 *Value, ECOA__float32 *Bearing, ECOA__float32 *Inclination, ECOA__float32 *Bank)
{
  /* Check that a NULL value has not been supplied */

  if((Bearing     == NULL) ||
     (Inclination == NULL) ||
     (Bank        == NULL) ||
     (Value       == NULL))    return(ECOA__FALSE);

  /* De-scale the supplied value */

  *Bearing      = (ECOA__float32)(*(ECOA__int16 *)(Value + 0)) / KLV_LSB_ECOA_16_BIT_ANGLE;
  *Inclination  = (ECOA__float32)(*(ECOA__int16 *)(Value + 2)) / KLV_LSB_ECOA_16_BIT_ANGLE;
  *Bank         = (ECOA__float32)(*(ECOA__int16 *)(Value + 4)) / KLV_LSB_ECOA_16_BIT_ANGLE;

  /* Finished */

  return(ECOA__TRUE);
}

/* Procedure to scale a Sensor Status - Sensor Angles */

ECOA__boolean8 Common__KLV_Scale_Sensor_Status_Sensor_Angles(const ECOA__float32 *Azimuth, const ECOA__float32 *Elevation, const ECOA__float32 *Roll, ECOA__uint8 *Length, ECOA__uint8 **Value)
{
  /* Check that a NULL value has not been supplied */

  if((Azimuth   == NULL) ||
     (Elevation == NULL) ||
     (Roll      == NULL) ||
     (Length    == NULL) ||
     (Value     == NULL))    return(ECOA__FALSE);

  /* Set the length and allocate the value */

  if((*Value = (ECOA__uint8 *)calloc(1, *Length = 6)) == NULL)  return(ECOA__FALSE);

  /* Scale the supplied parameters */

  *(ECOA__int16 *)(*Value + 0) |= (ECOA__int16)(*Azimuth   * KLV_LSB_ECOA_16_BIT_ANGLE);
  *(ECOA__int16 *)(*Value + 2) |= (ECOA__int16)(*Elevation * KLV_LSB_ECOA_16_BIT_ANGLE);
  *(ECOA__int16 *)(*Value + 4) |= (ECOA__int16)(*Roll      * KLV_LSB_ECOA_16_BIT_ANGLE);

  /* Finished */

  return(ECOA__TRUE);
}

/* Procedure to de-scale a Sensor Status - Sensor Angles */

ECOA__boolean8 Common__KLV_Descale_Sensor_Status_Sensor_Angles(const ECOA__uint8 *Value, ECOA__float32 *Azimuth, ECOA__float32 *Elevation, ECOA__float32 *Roll)
{
  /* Check that a NULL value has not been supplied */

  if((Azimuth   == NULL) ||
     (Elevation == NULL) ||
     (Roll      == NULL) ||
     (Value     == NULL))    return(ECOA__FALSE);

  /* De-scale the supplied value */

  *Azimuth    = (ECOA__float32)(*(ECOA__int16 *)(Value + 0)) / KLV_LSB_ECOA_16_BIT_ANGLE;
  *Elevation  = (ECOA__float32)(*(ECOA__int16 *)(Value + 2)) / KLV_LSB_ECOA_16_BIT_ANGLE;
  *Roll       = (ECOA__float32)(*(ECOA__int16 *)(Value + 4)) / KLV_LSB_ECOA_16_BIT_ANGLE;

  /* Finished */

  return(ECOA__TRUE);
}

/* Procedure to scale a Sensor Status - Target Position */

ECOA__boolean8 Common__KLV_Scale_Sensor_Status_Target_Position(const ECOA__double64 *Latitude, const ECOA__double64 *Longitude, const ECOA__double64 *Altitude, ECOA__uint8 *Length, ECOA__uint8 **Value)
{
  /* Check that a NULL value has not been supplied */

  if((Latitude  == NULL) ||
     (Longitude == NULL) ||
     (Altitude  == NULL) ||
     (Length    == NULL) ||
     (Value     == NULL))    return(ECOA__FALSE);

  /* Set the length and allocate the value */

  if((*Value = (ECOA__uint8 *)calloc(1, *Length = 10)) == NULL)  return(ECOA__FALSE);

  /* Scale the supplied parameters */

  *(ECOA__int32 *) (*Value + 0) |= (ECOA__int32) (*Latitude  * KLV_LSB_ECOA_32_BIT_ANGLE);
  *(ECOA__int32 *) (*Value + 4) |= (ECOA__int32) (*Longitude * KLV_LSB_ECOA_32_BIT_ANGLE);
  *(ECOA__uint16 *)(*Value + 8) |= (ECOA__uint16)(*Altitude  * KLV_LSB_ECOA_ALTITUDE);

  /* Finished */

  return(ECOA__TRUE);
}

/* Procedure to de-scale a Sensor Status - Target Position */

ECOA__boolean8 Common__KLV_Descale_Sensor_Status_Target_Position(const ECOA__uint8 *Value, ECOA__double64 *Latitude, ECOA__double64 *Longitude, ECOA__double64 *Altitude)
{
  /* Check that a NULL value has not been supplied */

  if((Latitude  == NULL) ||
     (Longitude == NULL) ||
     (Altitude  == NULL) ||
     (Value     == NULL))    return(ECOA__FALSE);

  /* De-scale the supplied value */

  *Latitude  = (ECOA__double64)(*(ECOA__int32 *) (Value + 0)) / KLV_LSB_ECOA_32_BIT_ANGLE;
  *Longitude = (ECOA__double64)(*(ECOA__int32 *) (Value + 4)) / KLV_LSB_ECOA_32_BIT_ANGLE;
  *Altitude  = (ECOA__double64) (*(ECOA__uint16 *)(Value + 8)) / KLV_LSB_ECOA_ALTITUDE;

  /* Finished */

  return(ECOA__TRUE);
}

/* Procedure to scale a Sensor Status - Sightline Position */

ECOA__boolean8 Common__KLV_Scale_Sensor_Status_Sightline_Position(const ECOA__double64 *Latitude, const ECOA__double64 *Longitude, const ECOA__double64 *Altitude, ECOA__uint8 *Length, ECOA__uint8 **Value)
{
  /* Check that a NULL value has not been supplied */

  if((Latitude  == NULL) ||
     (Longitude == NULL) ||
     (Altitude  == NULL) ||
     (Length    == NULL) ||
     (Value     == NULL))    return(ECOA__FALSE);

  /* Set the length and allocate the value */

  if((*Value = (ECOA__uint8 *)calloc(1, *Length = 10)) == NULL)  return(ECOA__FALSE);

  /* Scale the supplied parameters */

  *(ECOA__int32 *) (*Value + 0) |= (ECOA__int32) (*Latitude  * KLV_LSB_ECOA_32_BIT_ANGLE);
  *(ECOA__int32 *) (*Value + 4) |= (ECOA__int32) (*Longitude * KLV_LSB_ECOA_32_BIT_ANGLE);
  *(ECOA__uint16 *)(*Value + 8) |= (ECOA__uint16)(*Altitude  * KLV_LSB_ECOA_ALTITUDE);

  /* Finished */

  return(ECOA__TRUE);
}

/* Procedure to de-scale a Sensor Status - Sightline Position */

ECOA__boolean8 Common__KLV_Descale_Sensor_Status_Sightline_Position(const ECOA__uint8 *Value, ECOA__double64 *Latitude, ECOA__double64 *Longitude, ECOA__double64 *Altitude)
{
  /* Check that a NULL value has not been supplied */

  if((Latitude  == NULL) ||
     (Longitude == NULL) ||
     (Altitude  == NULL) ||
     (Value     == NULL))    return(ECOA__FALSE);

  /* De-scale the supplied value */

  *Latitude  = (ECOA__double64)(*(ECOA__int32 *) (Value + 0)) / KLV_LSB_ECOA_32_BIT_ANGLE;
  *Longitude = (ECOA__double64)(*(ECOA__int32 *) (Value + 4)) / KLV_LSB_ECOA_32_BIT_ANGLE;
  *Altitude  = (ECOA__double64) (*(ECOA__uint16 *)(Value + 8)) / KLV_LSB_ECOA_ALTITUDE;

  /* Finished */

  return(ECOA__TRUE);
}

/* Procedure to scale a Sensor Status - Sensor Range */

ECOA__boolean8 Common__KLV_Scale_Sensor_Status_Sensor_Range(const ECOA__float32 *Min, const ECOA__float32 *Max, ECOA__uint8 *Length, ECOA__uint8 **Value)
{
  /* Check that a NULL value has not been supplied */

  if((Min     == NULL) ||
     (Max     == NULL) ||
     (Length  == NULL) ||
     (Value   == NULL))    return(ECOA__FALSE);

  /* Set the length and allocate the value */

  if((*Value = (ECOA__uint8 *)calloc(1, *Length = 8)) == NULL)  return(ECOA__FALSE);

  /* Scale the supplied parameters */

  *(ECOA__uint32 *)(*Value + 0) |= (ECOA__uint32)(*Min);
  *(ECOA__uint32 *)(*Value + 4) |= (ECOA__uint32)(*Max);

  /* Finished */

  return(ECOA__TRUE);
}

/* Procedure to de-scale a Sensor Status - Sensor Range */

ECOA__boolean8 Common__KLV_Descale_Sensor_Status_Sensor_Range(const ECOA__uint8 *Value, ECOA__float32 *Min, ECOA__float32 *Max)
{
  /* Check that a NULL value has not been supplied */

  if((Min   == NULL) ||
     (Max   == NULL) ||
     (Value == NULL))    return(ECOA__FALSE);

  /* De-scale the supplied value */

  *Min  = (ECOA__float32)(*(ECOA__uint32 *)(Value + 0));
  *Max  = (ECOA__float32)(*(ECOA__uint32 *)(Value + 4));

  /* Finished */

  return(ECOA__TRUE);
}
