/*
 * ECOS Demo1B HMI Target class
 * 
 * K S Appleby  March 2012
 * 
 * $Id: Target.c,v 2.0 2017/08/02 15:38:06 ksappleb Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include <libxml/tree.h>

#include "Target.h"

#ifdef _MSC_VER
# define strdup _strdup
#endif

/**********************************************************************************
 * Private operations
 */

/* None */

/**********************************************************************************
 * Public operations
 */

Target targetCreate()
{
	Target trgt = (Target)malloc( sizeof( struct Targett ));
	trgt->id = 0;
	trgt->description = (char*)NULL;
	return trgt;
}

void   targetDestroy( Target trgt )
{
	if( trgt ){
		if( trgt->description )
			free( trgt->description );
		free( trgt );
	}
	trgt = (Target)NULL;
}

char*  targetToXML( Target trgt, char* buf )
{
	char lbuf[1024];
	  /**/
	if( buf != NULL ){
		strcat( buf, "<TARGET>\n" );
		sprintf( lbuf, "<ID>%d</ID>\n", trgt->id );
		  strcat( buf, lbuf );
		strcat( buf, "<POSITION>\n" );
		lbuf[0] = '\0';
		pairToXML( trgt->position, "LAT", "LONG", lbuf );
		  strcat( buf, lbuf );
		strcat( buf, "</POSITION>\n" );
		sprintf( lbuf, "<HEIGHT>%g</HEIGHT>\n", trgt->height );
		  strcat( buf, lbuf );
		strcat( buf, "<DESCRIPTION>\n" );
		strcat( buf, trgt->description );
		strcat( buf, "</DESCRIPTION>\n" );
		strcat( buf, "</TARGET>\n" );
	}
	return buf;
}

char*  targetToString( Target trgt, char* buf )
{
	char lbuf[1024];
	  /**/
	if( buf != NULL ){
		sprintf( lbuf, "  ID          : %d\n", trgt->id );
		  strcat( buf, lbuf );
		strcat( buf,  "  POSITION    : " );
		pairToString( trgt->position, "LAT", "LONG", lbuf );
		  strcat( buf, lbuf );
		sprintf( lbuf, "  HEIGHT      : %g\n", trgt->height );
		  strcat( buf, lbuf );
		sprintf( lbuf, "  DESCRIPTION : %s\n", trgt->description );
		  strcat( buf, lbuf );
	}
	return buf;
}

void targetFillFromXML( Target trgt, xmlDocPtr pDoc, xmlNodePtr pNode )
{
	xmlChar*   p;
	xmlNodePtr cur = pNode->xmlChildrenNode;
	do {
		while( cur != NULL && 
			( xmlIsBlankNode( cur ) || strcmp( (char*)cur->name, "comment" )==0 )){
			cur = cur->next;
	    }
		if( cur == NULL ) break;
		if( xmlStrcmp( cur->name, (const xmlChar *)"ID" ) == 0 ){
			trgt->id = atol( (char*)(p = xmlNodeGetContent( cur )));	xmlFree(p);
		}else
		if( xmlStrcmp( cur->name, (const xmlChar *)"POSITION" ) == 0 ){
			pairFillFromXML( &(trgt->position), pDoc, cur, "LAT", "LONG" );
		}else
		if( xmlStrcmp( cur->name, (const xmlChar *)"HEIGHT" ) == 0 ){
			trgt->height = atof( (char*)(p = xmlNodeGetContent( cur ))); xmlFree(p);
		}else
		if( xmlStrcmp( cur->name, (const xmlChar *)"DESCRIPTION" ) == 0 ){
			if( trgt->description )
				free( trgt->description );
			trgt->description = strdup( (char*)(p = xmlNodeGetContent( cur ))); xmlFree(p);
		}else{
			fprintf(stderr,"%s: wrong node type (found %s)\n", (pDoc->name ? pDoc->name : "DESCRIPTION"), cur->name );
			break;
		}
		cur = cur->next;
	}while( cur != NULL );
}
