/*
 * ECOS Demo1B HMI Targets class
 * 
 * K S Appleby  March 2012
 * 
 * $Id: Targets.c,v 2.0 2017/08/02 15:38:06 ksappleb Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "Targets.h"

#if defined(_WIN32 )
# include <malloc.h>
# define alloca _alloca
#endif

#define MAXTARGETS 16

struct Targetst {
	unsigned int NumTargets;
	Target       Targets[MAXTARGETS];
};

/**********************************************************************************
 * Private operations
 */

/* None */

/**********************************************************************************
 * Public operations
 */

Targets targetsCreate()
{
	int i;
	Targets trgts = (Targets)malloc( sizeof( struct Targetst ));
	if( trgts ){
		trgts->NumTargets = 0;
		for( i = 0; i < MAXTARGETS; i++ )
			trgts->Targets[i] = (Target)NULL;
	}
	return trgts;
}

void targetsDestroy( Targets trgts )
{
	unsigned int trgt;
	if( trgts ){
		for( trgt = 0; trgt < trgts->NumTargets; trgt++ )
			targetDestroy( trgts->Targets[trgt] );
		free( trgts );
	}
	trgts = (Targets)NULL;
}

unsigned int targetsGetNumTargets( Targets trgts )
{
	if( trgts )
		return trgts->NumTargets;
	else
		return 0;
}

Target targetsGetTarget( Targets trgts, int id )
{
	unsigned int i = 0;
	if( trgts ){
		while( i < trgts->NumTargets ){
			if( trgts->Targets[i]->id == id ) break;
			i++;
		}
		return targetsGetTargetByIndex( trgts, i );
	}else
		return (Target)NULL;
}

Target targetsGetTargetByIndex( Targets trgts, unsigned int indx )
{
	if( trgts && ( indx < trgts->NumTargets ))
		return trgts->Targets[indx];
	else
		return (Target)NULL;
}

void targetsAddTarget( Targets trgts, Target trgt )
{
	if( trgts )
		if( trgts->NumTargets < MAXTARGETS )
			trgts->Targets[ trgts->NumTargets++ ] = trgt;
}

int targetsLoadTargets( Targets trgts, char*  uri )
{
	xmlDocPtr  pDoc = NULL;
	xmlNodePtr cur;
	
	  /* Init the XML doc. */
	if( strncmp( uri, "file://", 7 ) == 0 )         /* Absolute URL? */
		pDoc = xmlParseFile( uri );
	else
	if( strncmp( uri, "file:", 5 ) == 0 )           /* Relative Path ? */
		pDoc = xmlParseFile( &(uri[5]) );
	else
	if( strncmp( uri, "urn:ecoatargs:", 14 ) == 0 ) /* Home grown URN ? */
		pDoc = xmlParseMemory( &(uri[14]), strlen(&(uri[14])) );

	if( pDoc == NULL ){
		fprintf( stderr, "%s: invalid document\n", uri );
		return -1;
	}

	cur = xmlDocGetRootElement( pDoc );
	if( cur == NULL ) {
		fprintf( stderr, "%s: empty document\n", uri );
	}else
	if( xmlStrcmp( cur->name, (const xmlChar*)"ECOA" ) == 0 ) {
		cur = cur->xmlChildrenNode;
		while( cur != NULL && 
			( xmlIsBlankNode( cur ) || strcmp( cur->name, "comment" )==0 )){
			cur = cur->next;
	    }
		if( xmlStrcmp( cur->name, (const xmlChar *)"TARGETS" ) == 0 )
			targetsFillFromXML( trgts, pDoc, cur );
		else{
			fprintf( stderr, "%s: wrong document type (found %s)\n", uri, cur->name );
		}
	}else{
		fprintf( stderr, "%s: wrong document type (found %s)\n", uri, cur->name );
	}
	xmlFreeDoc( pDoc );
	//
	return 0;
}

/**********************************************************************************/

char* targetsToXML( Targets trgts, char* buf )
{
	char lbuf[1024];
	unsigned int  i;
	  /**/
	if( buf != NULL ){
		strcat( buf, "<TARGETS>\n" );
		if( trgts->NumTargets > 0 ){
			for( i = 0; i < trgts->NumTargets; i++ ){
				lbuf[0] = '\0';
				targetToXML( trgts->Targets[i], lbuf );
				strcat( buf, lbuf );
			}
		}
		strcat( buf, "</TARGETS>\n" );
	}
	return buf;
}

char* targetsToString( Targets trgts, char* buf )
{
	char lbuf[1024];
	unsigned int  i;
	  /**/
	if( buf != NULL ){
		strcat( buf, "TARGET LIST:\n" );
		if( trgts->NumTargets > 0 )
			for( i = 0; i < trgts->NumTargets; i++ ){
				sprintf( lbuf, " TARGET %4d: \n", i );
				targetToString( trgts->Targets[i], lbuf );
				strcat( buf, lbuf );
			}
		else
			strcat( buf, "  (nil)\n" );
	}
	return buf;
}

void targetsFillFromXML( Targets trgts, xmlDocPtr pDoc, xmlNodePtr pNode )
{
	xmlNodePtr cur = pNode->xmlChildrenNode;
	trgts->NumTargets = 0;
	do {
		while( cur != NULL && 
			( xmlIsBlankNode( cur ) || strcmp( cur->name, "comment" )==0 )){
			cur = cur->next;
	    }
		if( cur == NULL ) break;
		if( xmlStrcmp( cur->name, (const xmlChar *)"TARGET" ) == 0 ){
			Target trgt = targetCreate();
			targetFillFromXML( trgt, pDoc, cur );
			trgts->Targets[ trgts->NumTargets++ ] = trgt;
		}else{
			fprintf(stderr,"%s: wrong node type (found %s)\n", (pDoc->name ? pDoc->name : "TARGET"), cur->name );
			break;
		}
		cur = cur->next;
	}while( cur != NULL );
}

